@@ -4,14 +4,14 @@
 
 'use strict';
 
-let { Test } = require('./Test.jsm');
-let { Test: Test2 } = require('./Test.jsm');
+var { Test } = require('./Test.jsm');
+var { Test: Test2 } = require('./Test.jsm');
 exports.localJSM = Test.square(16) === 256;
 exports.localJSMCached = Test === Test2;
 
 (function () {
-let { Promise } = require('resource://gre/modules/Promise.jsm');
-let { defer } = require('resource://gre/modules/Promise.jsm').Promise;
+var { Promise } = require('resource://gre/modules/Promise.jsm');
+var { defer } = require('resource://gre/modules/Promise.jsm').Promise;
 
 exports.isCachedAbsolute = Promise.defer === defer;
 
@@ -23,8 +23,8 @@ exports.isLoadedAbsolute = function (val) {
 })();
 
 (function () {
-let { Promise } = require('modules/Promise.jsm');
-let { defer } = require('modules/Promise.jsm').Promise;
+var { Promise } = require('modules/Promise.jsm');
+var { defer } = require('modules/Promise.jsm').Promise;
 exports.isCachedPath = Promise.defer === defer;
 
 exports.isLoadedPath = function (val) {
@@ -35,8 +35,8 @@ exports.isLoadedPath = function (val) {
 })();
 
 (function () {
-let { defer } = require('resource://gre/modules/commonjs/sdk/core/promise.js');
-let { defer: defer2 } = require('resource://gre/modules/commonjs/sdk/core/promise.js');
+var { defer } = require('resource://gre/modules/commonjs/sdk/core/promise.js');
+var { defer: defer2 } = require('resource://gre/modules/commonjs/sdk/core/promise.js');
 exports.isCachedJSAbsolute = defer === defer2;
 exports.isLoadedJSAbsolute = function (val) {
   let { promise, resolve } = defer();
