@@ -9,7 +9,7 @@ const windows = require("sdk/windows").browserWindows;
 const app = require("sdk/system/xul-app");
 const { viewFor } = require("sdk/view/core");
 const { modelFor } = require("sdk/model/core");
-const { getTabId, isTab } = require("sdk/tabs/utils");
+const { getBrowserForTab, getTabId, isTab } = require("sdk/tabs/utils");
 const { defer } = require("sdk/lang/functional");
 
 function tabExistenceInTabs(assert, found, tab, tabs) {
@@ -202,4 +202,27 @@ exports["test tab.readyState"] = (assert, done) => {
   });
 }
 
+exports["test tab.readyState for existent tabs"] = function* (assert) {
+  assert.equal(tabs.length, 1, "tabs contains an existent tab");
+
+  function frameScript() {
+    sendAsyncMessage("test:contentDocument.readyState", content.document.readyState);
+  }
+
+  for (let tab of tabs) {
+    let browserForTab = getBrowserForTab(viewFor(tab));
+    let mm = browserForTab.messageManager;
+
+    yield new Promise((resolve) => {
+      mm.addMessageListener("test:contentDocument.readyState", function listener(evt) {
+        mm.removeMessageListener("test:contentDocument.readyState", listener);
+        assert.equal(evt.data, tab.readyState,
+                     "tab.readyState has the same value of the associated contentDocument.readyState CPOW");
+        resolve();
+      });
+      mm.loadFrameScript(`data:,new ${frameScript};`, false);
+    });
+  }
+}
+
 require("sdk/test").run(module.exports);
