@@ -10,6 +10,7 @@ const { viewFor } = require('sdk/view/core');
 const { modelFor } = require('sdk/model/core');
 const { Ci } = require("chrome");
 const { isBrowser, getWindowTitle } = require("sdk/window/utils");
+const { after, cleanUI } = require("sdk/test/utils");
 
 // TEST: browserWindows Iterator
 exports.testBrowserWindowsIterator = function(assert) {
@@ -65,20 +66,18 @@ exports.testWindowActivateMethod_simple = function(assert) {
 };
 
 
-exports["test getView(window)"] = function(assert, done) {
-  browserWindows.once("open", window => {
-    const view = viewFor(window);
-
-    assert.ok(view instanceof Ci.nsIDOMWindow, "view is a window");
-    assert.ok(isBrowser(view), "view is a browser window");
-    assert.equal(getWindowTitle(view), window.title,
-                 "window has a right title");
-
-    window.close(done);
+exports["test getView(window)"] = function*(assert) {
+  let window = yield new Promise(resolve => {
+    browserWindows.once("open", resolve);
+    browserWindows.open({ url: "data:text/html;charset=utf-8,<title>yo</title>" });
   });
 
+  const view = viewFor(window);
 
-  browserWindows.open({ url: "data:text/html;charset=utf-8,<title>yo</title>" });
+  assert.ok(view instanceof Ci.nsIDOMWindow, "view is a window");
+  assert.ok(isBrowser(view), "view is a browser window");
+  assert.equal(getWindowTitle(view), window.title,
+               "window has a right title");
 };
 
 
@@ -97,4 +96,9 @@ exports["test modelFor(window)"] = function(assert, done) {
   browserWindows.open({ url: "data:text/html;charset=utf-8,<title>yo</title>" });
 };
 
+after(exports, function*(name, assert) {
+  assert.pass("cleaning the ui.");
+  yield cleanUI();
+});
+
 require('sdk/test').run(exports);
