@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -21,12 +21,12 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(DOMError)
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
-DOMError::DOMError(nsPIDOMWindow* aWindow)
+DOMError::DOMError(nsPIDOMWindowInner* aWindow)
   : mWindow(aWindow)
 {
 }
 
-DOMError::DOMError(nsPIDOMWindow* aWindow, nsresult aValue)
+DOMError::DOMError(nsPIDOMWindowInner* aWindow, nsresult aValue)
   : mWindow(aWindow)
 {
   nsCString name, message;
@@ -36,13 +36,13 @@ DOMError::DOMError(nsPIDOMWindow* aWindow, nsresult aValue)
   CopyUTF8toUTF16(message, mMessage);
 }
 
-DOMError::DOMError(nsPIDOMWindow* aWindow, const nsAString& aName)
+DOMError::DOMError(nsPIDOMWindowInner* aWindow, const nsAString& aName)
   : mWindow(aWindow)
   , mName(aName)
 {
 }
 
-DOMError::DOMError(nsPIDOMWindow* aWindow, const nsAString& aName,
+DOMError::DOMError(nsPIDOMWindowInner* aWindow, const nsAString& aName,
                    const nsAString& aMessage)
   : mWindow(aWindow)
   , mName(aName)
@@ -55,9 +55,9 @@ DOMError::~DOMError()
 }
 
 JSObject*
-DOMError::WrapObject(JSContext* aCx)
+DOMError::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
 {
-  return DOMErrorBinding::Wrap(aCx, this);
+  return DOMErrorBinding::Wrap(aCx, this, aGivenProto);
 }
 
 /* static */ already_AddRefed<DOMError>
@@ -65,11 +65,11 @@ DOMError::Constructor(const GlobalObject& aGlobal,
                       const nsAString& aName, const nsAString& aMessage,
                       ErrorResult& aRv)
 {
-  nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(aGlobal.GetAsSupports());
+  nsCOMPtr<nsPIDOMWindowInner> window = do_QueryInterface(aGlobal.GetAsSupports());
 
   // Window is null for chrome code.
 
-  nsRefPtr<DOMError> ret = new DOMError(window, aName, aMessage);
+  RefPtr<DOMError> ret = new DOMError(window, aName, aMessage);
   return ret.forget();
 }
 
