@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,12 +11,13 @@
 #include "nsWrapperCache.h"
 #include "nsCOMPtr.h"
 #include "nsString.h"
-#include "nsPIDOMWindow.h"
 
 #define DOMERROR_IID \
 { 0x220cb63f, 0xa37d, 0x4ba4, \
  { 0x8e, 0x31, 0xfc, 0xde, 0xec, 0x48, 0xe1, 0x66 } }
 
+class nsPIDOMWindowInner;
+
 namespace mozilla {
 
 class ErrorResult;
@@ -28,7 +29,7 @@ class GlobalObject;
 class DOMError : public nsISupports,
                  public nsWrapperCache
 {
-  nsCOMPtr<nsPIDOMWindow> mWindow;
+  nsCOMPtr<nsPIDOMWindowInner> mWindow;
   nsString mName;
   nsString mMessage;
 
@@ -44,22 +45,22 @@ public:
   // aWindow can be null if this DOMError is not associated with a particular
   // window.
 
-  explicit DOMError(nsPIDOMWindow* aWindow);
+  explicit DOMError(nsPIDOMWindowInner* aWindow);
 
-  DOMError(nsPIDOMWindow* aWindow, nsresult aValue);
+  DOMError(nsPIDOMWindowInner* aWindow, nsresult aValue);
 
-  DOMError(nsPIDOMWindow* aWindow, const nsAString& aName);
+  DOMError(nsPIDOMWindowInner* aWindow, const nsAString& aName);
 
-  DOMError(nsPIDOMWindow* aWindow, const nsAString& aName,
+  DOMError(nsPIDOMWindowInner* aWindow, const nsAString& aName,
            const nsAString& aMessage);
 
-  nsPIDOMWindow* GetParentObject() const
+  nsPIDOMWindowInner* GetParentObject() const
   {
     return mWindow;
   }
 
   virtual JSObject*
-  WrapObject(JSContext* aCx) override;
+  WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;
 
   static already_AddRefed<DOMError>
   Constructor(const GlobalObject& global, const nsAString& name,
