@@ -7,7 +7,6 @@
 
 SimpleTest.waitForExplicitFinish();
 browserElementTestHelpers.setEnabledPref(true);
-browserElementTestHelpers.addPermission();
 
 // We'll need to get the appId from the current document,
 // it's either SpecialPowers.Ci.nsIScriptSecurityManager.NO_APP_ID when
@@ -67,21 +66,19 @@ function createFrames() {
    // Create an input field to receive string from input method iframes.
    gInputFrame = document.createElement('iframe');
    gInputFrame.setAttribute('mozbrowser', 'true');
-   gInputFrame.src =
-     'data:text/html,<input autofocus value="hello" />' +
-     '<p>This is targetted mozbrowser frame.</p>';
+   gInputFrame.src = 'file_browserElement_SetInputMethodActive.html';
    document.body.appendChild(gInputFrame);
    gInputFrame.addEventListener('mozbrowserloadend', countLoadend);
 
    for (let i = 0; i < 2; i++) {
-    let frame = gInputMethodFrames[i] = document.createElement('iframe');
-    frame.setAttribute('mozbrowser', 'true');
-    if (currentAppManifestURL) {
-      frame.setAttribute('mozapp', currentAppManifestURL);
-    }
-    frame.src = 'file_empty.html#' + i;
-     document.body.appendChild(frame);
+     let frame = gInputMethodFrames[i] = document.createElement('iframe');
+     frame.setAttribute('mozbrowser', 'true');
+     if (currentAppManifestURL) {
+       frame.setAttribute('mozapp', currentAppManifestURL);
+     }
      frame.addEventListener('mozbrowserloadend', countLoadend);
+     frame.src = 'file_empty.html#' + i;
+     document.body.appendChild(frame);
    }
  }
 
@@ -93,7 +90,7 @@ function setPermissions() {
       url: SimpleTest.getTestFileURL('/file_empty.html'),
       originAttributes: {
         appId: currentAppId,
-        inBrowser: true
+        inIsolatedMozBrowser: true
       }
     }
   }];
@@ -122,7 +119,7 @@ function setPermissions() {
 
     input.onblur = function() {
       // "Expected" lost of focus since the test is finished.
-      if (input.value === '#0#1hello') {
+      if (input.value === 'hello#0#1') {
         return;
       }
 
@@ -231,7 +228,7 @@ function next(msg) {
 
         case 'input':
           if (gFrameMsgCounts.input === 1) {
-            is(value, '#0hello',
+            is(value, 'hello#0',
               'Failed to get correct input from the first iframe.');
           } else {
             ok(false, 'Unexpected multiple messages from input.')
@@ -286,7 +283,7 @@ function next(msg) {
 
         case 'input':
           if (gFrameMsgCounts.input === 2) {
-            is(value, '#0#1hello',
+            is(value, 'hello#0#1',
                'Failed to get correct input from the second iframe.');
           } else {
             ok(false, 'Unexpected multiple messages from input.')
