@@ -131,6 +131,30 @@ DeviceStorageStatics::InitDirs()
   sMutex.AssertCurrentThreadOwns();
   DS_LOG_INFO("");
 
+#if !defined(MOZ_WIDGET_GONK)
+  if (!XRE_IsParentProcess()) {
+    // For gonk, we have the parent process forward the directory information
+    // to the child using ContentParent::ForwardKnownInfo. On desktop, this
+    // winds up slowing down the startup (in particular ts_paint), so rather
+    // than penalize all e10s processes, we do a synchronous IPC call here,
+    // which only penalizes child processes which actually use DeviceStorage.
+
+    dom::ContentChild* child = dom::ContentChild::GetSingleton();
+    DeviceStorageLocationInfo locationInfo;
+    child->SendGetDeviceStorageLocations(&locationInfo);
+
+    NS_NewLocalFile(locationInfo.apps(),     true, getter_AddRefs(sInstance->mDirs[TYPE_APPS]));
+    NS_NewLocalFile(locationInfo.crashes(),  true, getter_AddRefs(sInstance->mDirs[TYPE_CRASHES]));
+    NS_NewLocalFile(locationInfo.pictures(), true, getter_AddRefs(sInstance->mDirs[TYPE_PICTURES]));
+    NS_NewLocalFile(locationInfo.videos(),   true, getter_AddRefs(sInstance->mDirs[TYPE_VIDEOS]));
+    NS_NewLocalFile(locationInfo.music(),    true, getter_AddRefs(sInstance->mDirs[TYPE_MUSIC]));
+    NS_NewLocalFile(locationInfo.sdcard(),   true, getter_AddRefs(sInstance->mDirs[TYPE_SDCARD]));
+
+    sInstance->mInitialized = true;
+    return;
+  }
+#endif
+
   nsCOMPtr<nsIProperties> dirService
     = do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID);
   MOZ_ASSERT(dirService);
@@ -174,13 +198,6 @@ DeviceStorageStatics::InitDirs()
     NS_NewLocalFile(path, /* aFollowLinks */ true,
                     getter_AddRefs(mDirs[TYPE_SDCARD]));
   }
-#ifdef MOZ_B2GDROID
-  if (NS_SUCCEEDED(mozilla::AndroidBridge::GetExternalPublicDirectory(
-      NS_LITERAL_STRING(DEVICESTORAGE_APPS), path))) {
-    NS_NewLocalFile(path, /* aFollowLinks */ true,
-                    getter_AddRefs(mDirs[TYPE_APPS]));
-  }
-#endif
 
 #elif defined (XP_UNIX)
   dirService->Get(NS_UNIX_XDG_PICTURES_DIR,
@@ -215,10 +232,8 @@ DeviceStorageStatics::InitDirs()
   }
 #endif // !MOZ_WIDGET_ANDROID
 
-#ifndef MOZ_B2GDROID
   dirService->Get(NS_APP_USER_PROFILE_50_DIR, NS_GET_IID(nsIFile),
                   getter_AddRefs(mDirs[TYPE_APPS]));
-#endif
 
   if (mDirs[TYPE_APPS]) {
     mDirs[TYPE_APPS]->AppendRelativeNativePath(NS_LITERAL_CSTRING("webapps"));
@@ -271,6 +286,13 @@ DeviceStorageStatics::DumpDirs()
     nullptr
   };
 
+  const char* ptStr;
+  if (XRE_IsParentProcess()) {
+    ptStr = "parent";
+  } else {
+    ptStr = "child";
+  }
+
   for (uint32_t i = 0; i < TYPE_COUNT; ++i) {
     MOZ_ASSERT(storageTypes[i]);
 
@@ -278,8 +300,8 @@ DeviceStorageStatics::DumpDirs()
     if (mDirs[i]) {
       mDirs[i]->GetPath(path);
     }
-    DS_LOG_INFO("%s: '%s'",
-      storageTypes[i], NS_LossyConvertUTF16toASCII(path).get());
+    DS_LOG_INFO("(%s) %s: '%s'",
+      ptStr, storageTypes[i], NS_LossyConvertUTF16toASCII(path).get());
   }
 #endif
 }
@@ -297,6 +319,23 @@ DeviceStorageStatics::Shutdown()
   Preferences::RemoveObserver(this, kPrefWritableName);
 }
 
+/* static */ void
+DeviceStorageStatics::GetDeviceStorageLocationsForIPC(
+  DeviceStorageLocationInfo* aLocationInfo)
+{
+  MOZ_ASSERT(XRE_IsParentProcess());
+  MOZ_ASSERT(NS_IsMainThread());
+
+  InitializeDirs();
+
+  GetDirPath(TYPE_APPS,     aLocationInfo->apps());
+  GetDirPath(TYPE_CRASHES,  aLocationInfo->crashes());
+  GetDirPath(TYPE_PICTURES, aLocationInfo->pictures());
+  GetDirPath(TYPE_VIDEOS,   aLocationInfo->videos());
+  GetDirPath(TYPE_MUSIC,    aLocationInfo->music());
+  GetDirPath(TYPE_SDCARD,   aLocationInfo->sdcard());
+}
+
 /* static */ already_AddRefed<nsIFile>
 DeviceStorageStatics::GetDir(DeviceStorageType aType)
 {
@@ -332,6 +371,16 @@ DeviceStorageStatics::GetDir(DeviceStorageType aType)
   return file.forget();
 }
 
+/* static */ void
+DeviceStorageStatics::GetDirPath(DeviceStorageType aType, nsString& aDirPath)
+{
+  aDirPath.Truncate();
+  nsCOMPtr<nsIFile> file = GetDir(aType);
+  if (file) {
+    file->GetPath(aDirPath);
+  }
+}
+
 /* static */ bool
 DeviceStorageStatics::HasOverrideRootDir()
 {
@@ -439,7 +488,7 @@ DeviceStorageStatics::AddListener(nsDOMDeviceStorage* aListener)
   MOZ_ASSERT(sInstance->mInitialized);
   if (sInstance->mListeners.IsEmpty()) {
     NS_DispatchToMainThread(
-      NS_NewRunnableMethod(sInstance.get(), &DeviceStorageStatics::Register));
+      NewRunnableMethod(sInstance.get(), &DeviceStorageStatics::Register));
   }
 
   RefPtr<ListenerWrapper> wrapper =
@@ -470,7 +519,7 @@ DeviceStorageStatics::RemoveListener(nsDOMDeviceStorage* aListener)
 
   if (removed && sInstance->mListeners.IsEmpty()) {
     NS_DispatchToMainThread(
-      NS_NewRunnableMethod(sInstance.get(), &DeviceStorageStatics::Deregister));
+      NewRunnableMethod(sInstance.get(), &DeviceStorageStatics::Deregister));
   }
 }
 
@@ -524,7 +573,16 @@ DeviceStorageStatics::ResetOverrideRootDir()
   nsCOMPtr<nsIFile> f;
   DS_LOG_INFO("");
 
-  if (Preferences::GetBool(kPrefTesting, false)) {
+  // For users running on desktop, it's convenient to be able to override
+  // all of the directories to point to a single tree, much like what happens
+  // on a real device.
+  const nsAdoptingString& overrideRootDir =
+    mozilla::Preferences::GetString(kPrefOverrideRootDir);
+  if (overrideRootDir && !overrideRootDir.IsEmpty()) {
+    NS_NewLocalFile(overrideRootDir, false, getter_AddRefs(f));
+  }
+
+  if (!f && Preferences::GetBool(kPrefTesting, false)) {
     DS_LOG_INFO("temp");
     nsCOMPtr<nsIProperties> dirService
       = do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID);
@@ -534,15 +592,6 @@ DeviceStorageStatics::ResetOverrideRootDir()
       f->AppendRelativeNativePath(
         NS_LITERAL_CSTRING("device-storage-testing"));
     }
-  } else {
-    // For users running on desktop, it's convenient to be able to override
-    // all of the directories to point to a single tree, much like what happens
-    // on a real device.
-    const nsAdoptingString& overrideRootDir =
-      mozilla::Preferences::GetString(kPrefOverrideRootDir);
-    if (overrideRootDir && !overrideRootDir.IsEmpty()) {
-      NS_NewLocalFile(overrideRootDir, false, getter_AddRefs(f));
-    }
   }
 
   if (f) {
@@ -654,9 +703,9 @@ DeviceStorageStatics::Observe(nsISupports* aSubject,
     // of a file in a specific location while a low device storage situation
     // exists or after recovery of a low storage situation. For IceCat OS,
     // these notifications are specific for apps storage.
-    if (!NS_strcmp(aData, MOZ_UTF16("full"))) {
+    if (!NS_strcmp(aData, u"full")) {
       sInstance->mLowDiskSpace = true;
-    } else if (!NS_strcmp(aData, MOZ_UTF16("free"))) {
+    } else if (!NS_strcmp(aData, u"free")) {
       sInstance->mLowDiskSpace = false;
     } else {
       return NS_OK;
@@ -802,7 +851,7 @@ DeviceStorageStatics::ListenerWrapper::ListenerWrapper(nsDOMDeviceStorage* aList
 DeviceStorageStatics::ListenerWrapper::~ListenerWrapper()
 {
   // Even weak pointers are not thread safe
-  NS_ProxyRelease(mOwningThread, mListener);
+  NS_ProxyRelease(mOwningThread, mListener.forget());
 }
 
 bool
