@@ -13,6 +13,7 @@
 #include "MP4ESDS.h"
 #include "AMRBox.h"
 #include "AVCBox.h"
+#include "EVRCBox.h"
 #include "VideoUtils.h"
 
 namespace mozilla {
@@ -81,7 +82,7 @@ MediaDataBox::Write()
   for (uint32_t l = 0; l < types.Length(); l++) {
     if (mTrackType & types[l]) {
       FragmentBuffer* frag = mControl->GetFragment(types[l]);
-      nsTArray<nsRefPtr<EncodedFrame>> frames;
+      nsTArray<RefPtr<EncodedFrame>> frames;
 
       // Here is the last time we get fragment frames, flush it!
       rv = frag->GetFirstFragment(frames, true);
@@ -118,7 +119,7 @@ TrackRunBox::fillSampleTable()
 {
   uint32_t table_size = 0;
   nsresult rv;
-  nsTArray<nsRefPtr<EncodedFrame>> frames;
+  nsTArray<RefPtr<EncodedFrame>> frames;
   FragmentBuffer* frag = mControl->GetFragment(mTrackType);
 
   rv = frag->GetFirstFragment(frames);
@@ -126,7 +127,7 @@ TrackRunBox::fillSampleTable()
     return 0;
   }
   uint32_t len = frames.Length();
-  sample_info_table = new tbl[len];
+  sample_info_table = MakeUnique<tbl[]>(len);
   // Create sample table according to 14496-12 8.8.8.2.
   for (uint32_t i = 0; i < len; i++) {
     // Sample size.
@@ -418,7 +419,7 @@ MovieFragmentBox::Generate(uint32_t* aBoxSize)
   // Correct data_offset if there are both audio and video track in
   // this fragment. This offset means the offset in the MediaDataBox.
   if (mTrackType & (Audio_Track | Video_Track)) {
-    nsTArray<nsRefPtr<MuxerOperation>> truns;
+    nsTArray<RefPtr<MuxerOperation>> truns;
     rv = Find(NS_LITERAL_CSTRING("trun"), truns);
     NS_ENSURE_SUCCESS(rv, rv);
     uint32_t len = truns.Length();
@@ -660,12 +661,14 @@ SampleDescriptionBox::SampleDescriptionBox(uint32_t aType, ISOControl* aControl)
 }
 
 nsresult
-SampleDescriptionBox::CreateAudioSampleEntry(nsRefPtr<SampleEntryBox>& aSampleEntry)
+SampleDescriptionBox::CreateAudioSampleEntry(RefPtr<SampleEntryBox>& aSampleEntry)
 {
   if (mAudioMeta->GetKind() == TrackMetadataBase::METADATA_AMR) {
     aSampleEntry = new AMRSampleEntry(mControl);
   } else if (mAudioMeta->GetKind() == TrackMetadataBase::METADATA_AAC) {
     aSampleEntry = new MP4AudioSampleEntry(mControl);
+  } else if (mAudioMeta->GetKind() == TrackMetadataBase::METADATA_EVRC) {
+    aSampleEntry = new EVRCSampleEntry(mControl);
   } else {
     MOZ_ASSERT(0);
   }
@@ -673,7 +676,7 @@ SampleDescriptionBox::CreateAudioSampleEntry(nsRefPtr<SampleEntryBox>& aSampleEn
 }
 
 nsresult
-SampleDescriptionBox::CreateVideoSampleEntry(nsRefPtr<SampleEntryBox>& aSampleEntry)
+SampleDescriptionBox::CreateVideoSampleEntry(RefPtr<SampleEntryBox>& aSampleEntry)
 {
   if (mVideoMeta->GetKind() == TrackMetadataBase::METADATA_AVC) {
     aSampleEntry = new AVCSampleEntry(mControl);
@@ -1250,6 +1253,17 @@ FileTypeBox::Generate(uint32_t* aBoxSize)
     compatible_brands.AppendElement("3gp7");
     compatible_brands.AppendElement("3gp6");
     compatible_brands.AppendElement("isom");
+  } else if (mControl->GetMuxingType() == ISOMediaWriter::TYPE_FRAG_3G2) {
+    major_brand = "3g2a";
+    // 3GPP2 Release 0 and A and 3GPP Release 6 allow movie fragmentation
+    compatible_brands.AppendElement("3gp9");
+    compatible_brands.AppendElement("3gp8");
+    compatible_brands.AppendElement("3gp7");
+    compatible_brands.AppendElement("3gp6");
+    compatible_brands.AppendElement("isom");
+    compatible_brands.AppendElement("3g2c");
+    compatible_brands.AppendElement("3g2b");
+    compatible_brands.AppendElement("3g2a");
   } else {
     MOZ_ASSERT(0);
   }
@@ -1322,7 +1336,7 @@ DefaultContainerImpl::Generate(uint32_t* aBoxSize)
 
 nsresult
 DefaultContainerImpl::Find(const nsACString& aType,
-                           nsTArray<nsRefPtr<MuxerOperation>>& aOperations)
+                           nsTArray<RefPtr<MuxerOperation>>& aOperations)
 {
   nsresult rv = Box::Find(aType, aOperations);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1366,7 +1380,7 @@ Box::Write()
 }
 
 nsresult
-Box::Find(const nsACString& aType, nsTArray<nsRefPtr<MuxerOperation>>& aOperations)
+Box::Find(const nsACString& aType, nsTArray<RefPtr<MuxerOperation>>& aOperations)
 {
   if (boxType == aType) {
     aOperations.AppendElement(this);
@@ -1387,8 +1401,7 @@ FullBox::FullBox(const nsACString& aType, uint8_t aVersion, uint32_t aFlags,
                  ISOControl* aControl)
   : Box(aType, aControl)
 {
-  // Cast to uint64_t due to VC2010  bug.
-  std::bitset<24> tmp_flags((uint64_t)aFlags);
+  std::bitset<24> tmp_flags(aFlags);
   version = aVersion;
   flags = tmp_flags;
   size += sizeof(version) + flags.size() / CHAR_BIT;
