@@ -35,6 +35,10 @@ public:
   SourceBuffer* IndexedGetter(uint32_t aIndex, bool& aFound);
 
   uint32_t Length();
+
+  IMPL_EVENT_HANDLER(addsourcebuffer);
+  IMPL_EVENT_HANDLER(removesourcebuffer);
+
   /** End WebIDL methods. */
 
   NS_DECL_ISUPPORTS_INHERITED
@@ -71,9 +75,6 @@ public:
   // Mark all SourceBuffers input buffers as ended.
   void Ended();
 
-  // Evicts data for the given time range from each SourceBuffer in the list.
-  void Evict(double aStart, double aEnd);
-
   // Returns the highest end time of any of the Sourcebuffers.
   double GetHighestBufferedEndTime();
 
@@ -84,6 +85,9 @@ public:
   //  No event is fired and no action is performed on the sourcebuffers.
   void ClearSimple();
 
+  double HighestStartTime();
+  double HighestEndTime();
+
 private:
   ~SourceBufferList();
 
