@@ -18,7 +18,7 @@
 #include "mozilla/Telemetry.h"
 #include "mozilla/ArrayUtils.h"
 
-// NSPR_LOG_MODULES=LoadManager:5
+// MOZ_LOG=LoadManager:5
 mozilla::LazyLogModule gLoadManagerLog("LoadManager");
 #undef LOG
 #undef LOG_ENABLED
@@ -32,7 +32,8 @@ namespace mozilla {
 NS_IMPL_ISUPPORTS(LoadManagerSingleton, nsIObserver)
 
 
-LoadManagerSingleton::LoadManagerSingleton(int aLoadMeasurementInterval,
+LoadManagerSingleton::LoadManagerSingleton(bool aEncoderOnly,
+                                           int aLoadMeasurementInterval,
                                            int aAveragingMeasurements,
                                            float aHighLoadThreshold,
                                            float aLowLoadThreshold)
@@ -50,9 +51,11 @@ LoadManagerSingleton::LoadManagerSingleton(int aLoadMeasurementInterval,
        mLoadMeasurementInterval, mAveragingMeasurements,
        mHighLoadThreshold, mLowLoadThreshold));
   MOZ_ASSERT(mHighLoadThreshold > mLowLoadThreshold);
-  mLoadMonitor = new LoadMonitor(mLoadMeasurementInterval);
-  mLoadMonitor->Init(mLoadMonitor);
-  mLoadMonitor->SetLoadChangeCallback(this);
+  if (!aEncoderOnly) {
+    mLoadMonitor = new LoadMonitor(mLoadMeasurementInterval);
+    mLoadMonitor->Init(mLoadMonitor);
+    mLoadMonitor->SetLoadChangeCallback(this);
+  }
 
   mLastStateChange = TimeStamp::Now();
   for (auto &in_state : mTimeInState) {
@@ -181,36 +184,36 @@ LoadManagerSingleton::RemoveObserver(webrtc::CPULoadStateObserver * aObserver)
     LOG(("LoadManager - Element to remove not found"));
   }
   if (mObservers.Length() == 0) {
+    // Record how long we spent in the final state for later Telemetry or display
+    TimeStamp now = TimeStamp::Now();
+    mTimeInState[mCurrentState] += (now - mLastStateChange).ToMilliseconds();
+
+    float total = 0;
+    for (size_t i = 0; i < MOZ_ARRAY_LENGTH(mTimeInState); i++) {
+      total += mTimeInState[i];
+    }
+    // Don't include short calls; we don't have reasonable load data, and
+    // such short calls rarely reach a stable state.  Keep relatively
+    // short calls separate from longer ones
+    bool log = total > 5*PR_MSEC_PER_SEC;
+    bool small = log && total < 30*PR_MSEC_PER_SEC;
+    if (log) {
+      // Note: We don't care about rounding here; thus total may be < 100
+      Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_RELAXED_SHORT :
+                            Telemetry::WEBRTC_LOAD_STATE_RELAXED,
+                            (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadRelaxed]/total * 100));
+      Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_NORMAL_SHORT :
+                            Telemetry::WEBRTC_LOAD_STATE_NORMAL,
+                            (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadNormal]/total * 100));
+      Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_STRESSED_SHORT :
+                            Telemetry::WEBRTC_LOAD_STATE_STRESSED,
+                            (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadStressed]/total * 100));
+    }
+    for (auto &in_state : mTimeInState) {
+      in_state = 0;
+    }
+
     if (mLoadMonitor) {
-      // Record how long we spent in the final state for later Telemetry or display
-      TimeStamp now = TimeStamp::Now();
-      mTimeInState[mCurrentState] += (now - mLastStateChange).ToMilliseconds();
-
-      float total = 0;
-      for (size_t i = 0; i < MOZ_ARRAY_LENGTH(mTimeInState); i++) {
-        total += mTimeInState[i];
-      }
-      // Don't include short calls; we don't have reasonable load data, and
-      // such short calls rarely reach a stable state.  Keep relatively
-      // short calls separate from longer ones
-      bool log = total > 5*PR_MSEC_PER_SEC;
-      bool small = log && total < 30*PR_MSEC_PER_SEC;
-      if (log) {
-        // Note: We don't care about rounding here; thus total may be < 100
-        Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_RELAXED_SHORT :
-                                      Telemetry::WEBRTC_LOAD_STATE_RELAXED,
-                              (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadRelaxed]/total * 100));
-        Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_NORMAL_SHORT :
-                                      Telemetry::WEBRTC_LOAD_STATE_NORMAL,
-                              (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadNormal]/total * 100));
-        Telemetry::Accumulate(small ? Telemetry::WEBRTC_LOAD_STATE_STRESSED_SHORT :
-                                      Telemetry::WEBRTC_LOAD_STATE_STRESSED,
-                              (uint32_t) (mTimeInState[webrtc::CPULoadState::kLoadStressed]/total * 100));
-      }
-      for (auto &in_state : mTimeInState) {
-        in_state = 0;
-      }
-
       // Dance to avoid deadlock on mLock!
       RefPtr<LoadMonitor> loadMonitor = mLoadMonitor.forget();
       MutexAutoUnlock unlock(mLock);
