@@ -33,6 +33,10 @@ var _fromToTestLists = {
     new AnimTestcaseFromTo("currentColor", "rgb(100, 100, 100)",
                            { fromComp: "rgb(50, 50, 50)",
                              midComp: "rgb(75, 75, 75)" }),
+    new AnimTestcaseFromTo("rgba(10, 20, 30, 0.2)", "rgba(50, 50, 50, 1)",
+                             // (rgb(10, 20, 30) * 0.2 * 0.5 + rgb(50, 50, 50) * 1.0 * 0.5) * (1 / 0.6)
+                           { midComp: "rgba(43, 45, 47, 0.6)",
+                             toComp:  "rgb(50, 50, 50)"}),
   ],
   colorFromInheritBlack: [
     new AnimTestcaseFromTo("inherit", "rgb(200, 200, 200)",
@@ -154,12 +158,12 @@ var _fromToTestLists = {
   ],
   URIsAndNone: [
     new AnimTestcaseFromTo("url(#idA)", "url(#idB)",
-                           { fromComp: "url(\"" + document.URL + "#idA\")",
-                             toComp: "url(\"" + document.URL + "#idB\")"}),
+                           { fromComp: "url(\"#idA\")",
+                             toComp: "url(\"#idB\")"}),
     new AnimTestcaseFromTo("none", "url(#idB)",
-                           { toComp: "url(\"" + document.URL + "#idB\")"}),
+                           { toComp: "url(\"#idB\")"}),
     new AnimTestcaseFromTo("url(#idB)", "inherit",
-                           { fromComp: "url(\"" + document.URL + "#idB\")",
+                           { fromComp: "url(\"#idB\")",
                              toComp: "none"}),
   ],
 };
