@@ -10,7 +10,7 @@
 
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/ContentParent.h"
-#include "mozilla/unused.h"
+#include "mozilla/Unused.h"
 #include "nsIDiskSpaceWatcher.h"
 #include "nsThreadUtils.h"
 
@@ -68,13 +68,13 @@ DOMStorageDBChild::~DOMStorageDBChild()
 }
 
 nsTHashtable<nsCStringHashKey>&
-DOMStorageDBChild::ScopesHavingData()
+DOMStorageDBChild::OriginsHavingData()
 {
-  if (!mScopesHavingData) {
-    mScopesHavingData = new nsTHashtable<nsCStringHashKey>;
+  if (!mOriginsHavingData) {
+    mOriginsHavingData = new nsTHashtable<nsCStringHashKey>;
   }
 
-  return *mScopesHavingData;
+  return *mOriginsHavingData;
 }
 
 nsresult
@@ -102,7 +102,7 @@ DOMStorageDBChild::AsyncPreload(DOMStorageCacheBridge* aCache, bool aPriority)
     // Adding ref to cache for the time of preload.  This ensures a reference to
     // to the cache and that all keys will load into this cache object.
     mLoadingCaches.PutEntry(aCache);
-    SendAsyncPreload(aCache->Scope(), aPriority);
+    SendAsyncPreload(aCache->OriginSuffix(), aCache->OriginNoSuffix(), aPriority);
   } else {
     // No IPC, no love.  But the LoadDone call is expected.
     aCache->LoadDone(NS_ERROR_UNEXPECTED);
@@ -113,7 +113,7 @@ void
 DOMStorageDBChild::AsyncGetUsage(DOMStorageUsageBridge* aUsage)
 {
   if (mIPCOpen) {
-    SendAsyncGetUsage(aUsage->Scope());
+    SendAsyncGetUsage(aUsage->OriginScope());
   }
 }
 
@@ -136,7 +136,8 @@ DOMStorageDBChild::SyncPreload(DOMStorageCacheBridge* aCache, bool aForceSync)
   // case the async preload has already loaded some keys.
   InfallibleTArray<nsString> keys, values;
   nsresult rv;
-  SendPreload(aCache->Scope(), aCache->LoadedCount(), &keys, &values, &rv);
+  SendPreload(aCache->OriginSuffix(), aCache->OriginNoSuffix(), aCache->LoadedCount(),
+              &keys, &values, &rv);
 
   for (uint32_t i = 0; i < keys.Length(); ++i) {
     aCache->LoadItem(keys[i], values[i]);
@@ -154,8 +155,9 @@ DOMStorageDBChild::AsyncAddItem(DOMStorageCacheBridge* aCache,
     return mStatus;
   }
 
-  SendAsyncAddItem(aCache->Scope(), nsString(aKey), nsString(aValue));
-  ScopesHavingData().PutEntry(aCache->Scope());
+  SendAsyncAddItem(aCache->OriginSuffix(), aCache->OriginNoSuffix(),
+                   nsString(aKey), nsString(aValue));
+  OriginsHavingData().PutEntry(aCache->Origin());
   return NS_OK;
 }
 
@@ -168,8 +170,9 @@ DOMStorageDBChild::AsyncUpdateItem(DOMStorageCacheBridge* aCache,
     return mStatus;
   }
 
-  SendAsyncUpdateItem(aCache->Scope(), nsString(aKey), nsString(aValue));
-  ScopesHavingData().PutEntry(aCache->Scope());
+  SendAsyncUpdateItem(aCache->OriginSuffix(), aCache->OriginNoSuffix(),
+                      nsString(aKey), nsString(aValue));
+  OriginsHavingData().PutEntry(aCache->Origin());
   return NS_OK;
 }
 
@@ -181,7 +184,8 @@ DOMStorageDBChild::AsyncRemoveItem(DOMStorageCacheBridge* aCache,
     return mStatus;
   }
 
-  SendAsyncRemoveItem(aCache->Scope(), nsString(aKey));
+  SendAsyncRemoveItem(aCache->OriginSuffix(), aCache->OriginNoSuffix(),
+                      nsString(aKey));
   return NS_OK;
 }
 
@@ -192,44 +196,47 @@ DOMStorageDBChild::AsyncClear(DOMStorageCacheBridge* aCache)
     return mStatus;
   }
 
-  SendAsyncClear(aCache->Scope());
-  ScopesHavingData().RemoveEntry(aCache->Scope());
+  SendAsyncClear(aCache->OriginSuffix(), aCache->OriginNoSuffix());
+  OriginsHavingData().RemoveEntry(aCache->Origin());
   return NS_OK;
 }
 
 bool
-DOMStorageDBChild::ShouldPreloadScope(const nsACString& aScope)
+DOMStorageDBChild::ShouldPreloadOrigin(const nsACString& aOrigin)
 {
-  // Return true if we didn't receive the aScope list yet.
+  // Return true if we didn't receive the origins list yet.
   // I tend to rather preserve a bit of early-after-start performance
-  // then a bit of memory here.
-  return !mScopesHavingData || mScopesHavingData->Contains(aScope);
+  // than a bit of memory here.
+  return !mOriginsHavingData || mOriginsHavingData->Contains(aOrigin);
 }
 
 bool
 DOMStorageDBChild::RecvObserve(const nsCString& aTopic,
-                               const nsCString& aScopePrefix)
+                               const nsString& aOriginAttributesPattern,
+                               const nsCString& aOriginScope)
 {
-  DOMStorageObserver::Self()->Notify(aTopic.get(), aScopePrefix);
+  DOMStorageObserver::Self()->Notify(
+    aTopic.get(), aOriginAttributesPattern, aOriginScope);
   return true;
 }
 
 bool
-DOMStorageDBChild::RecvScopesHavingData(nsTArray<nsCString>&& aScopes)
+DOMStorageDBChild::RecvOriginsHavingData(nsTArray<nsCString>&& aOrigins)
 {
-  for (uint32_t i = 0; i < aScopes.Length(); ++i) {
-    ScopesHavingData().PutEntry(aScopes[i]);
+  for (uint32_t i = 0; i < aOrigins.Length(); ++i) {
+    OriginsHavingData().PutEntry(aOrigins[i]);
   }
 
   return true;
 }
 
 bool
-DOMStorageDBChild::RecvLoadItem(const nsCString& aScope,
+DOMStorageDBChild::RecvLoadItem(const nsCString& aOriginSuffix,
+                                const nsCString& aOriginNoSuffix,
                                 const nsString& aKey,
                                 const nsString& aValue)
 {
-  DOMStorageCache* aCache = mManager->GetCache(aScope);
+  DOMStorageCache* aCache = mManager->GetCache(aOriginSuffix, aOriginNoSuffix);
   if (aCache) {
     aCache->LoadItem(aKey, aValue);
   }
@@ -238,9 +245,11 @@ DOMStorageDBChild::RecvLoadItem(const nsCString& aScope,
 }
 
 bool
-DOMStorageDBChild::RecvLoadDone(const nsCString& aScope, const nsresult& aRv)
+DOMStorageDBChild::RecvLoadDone(const nsCString& aOriginSuffix,
+                                const nsCString& aOriginNoSuffix,
+                                const nsresult& aRv)
 {
-  DOMStorageCache* aCache = mManager->GetCache(aScope);
+  DOMStorageCache* aCache = mManager->GetCache(aOriginSuffix, aOriginNoSuffix);
   if (aCache) {
     aCache->LoadDone(aRv);
 
@@ -252,9 +261,9 @@ DOMStorageDBChild::RecvLoadDone(const nsCString& aScope, const nsresult& aRv)
 }
 
 bool
-DOMStorageDBChild::RecvLoadUsage(const nsCString& aScope, const int64_t& aUsage)
+DOMStorageDBChild::RecvLoadUsage(const nsCString& aOriginNoSuffix, const int64_t& aUsage)
 {
-  RefPtr<DOMStorageUsageBridge> scopeUsage = mManager->GetScopeUsage(aScope);
+  RefPtr<DOMStorageUsageBridge> scopeUsage = mManager->GetOriginUsage(aOriginNoSuffix);
   scopeUsage->LoadUsage(aUsage);
   return true;
 }
@@ -291,7 +300,7 @@ DOMStorageDBParent::ReleaseIPDLReference()
 
 namespace {
 
-class SendInitialChildDataRunnable : public nsRunnable
+class SendInitialChildDataRunnable : public Runnable
 {
 public:
   explicit SendInitialChildDataRunnable(DOMStorageDBParent* aParent)
@@ -299,7 +308,7 @@ public:
   {}
 
 private:
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     if (!mParent->IPCOpen()) {
       return NS_OK;
@@ -308,8 +317,8 @@ private:
     DOMStorageDBBridge* db = DOMStorageCache::GetDatabase();
     if (db) {
       InfallibleTArray<nsCString> scopes;
-      db->GetScopesHavingData(&scopes);
-      mozilla::Unused << mParent->SendScopesHavingData(scopes);
+      db->GetOriginsHavingData(&scopes);
+      mozilla::Unused << mParent->SendOriginsHavingData(scopes);
     }
 
     // We need to check if the device is in a low disk space situation, so
@@ -317,14 +326,15 @@ private:
     nsCOMPtr<nsIDiskSpaceWatcher> diskSpaceWatcher =
       do_GetService("@mozilla.org/toolkit/disk-space-watcher;1");
     if (!diskSpaceWatcher) {
-      NS_WARNING("Could not get disk information from DiskSpaceWatcher");
       return NS_OK;
     }
+
     bool lowDiskSpace = false;
     diskSpaceWatcher->GetIsDiskFull(&lowDiskSpace);
+
     if (lowDiskSpace) {
       mozilla::Unused << mParent->SendObserve(
-        nsDependentCString("low-disk-space"), EmptyCString());
+        nsDependentCString("low-disk-space"), EmptyString(), EmptyCString());
     }
 
     return NS_OK;
@@ -361,22 +371,10 @@ DOMStorageDBParent::~DOMStorageDBParent()
   }
 }
 
-mozilla::ipc::IProtocol*
-DOMStorageDBParent::CloneProtocol(Channel* aChannel,
-                                  mozilla::ipc::ProtocolCloneContext* aCtx)
-{
-  ContentParent* contentParent = aCtx->GetContentParent();
-  nsAutoPtr<PStorageParent> actor(contentParent->AllocPStorageParent());
-  if (!actor || !contentParent->RecvPStorageConstructor(actor)) {
-    return nullptr;
-  }
-  return actor.forget();
-}
-
 DOMStorageDBParent::CacheParentBridge*
-DOMStorageDBParent::NewCache(const nsACString& aScope)
+DOMStorageDBParent::NewCache(const nsACString& aOriginSuffix, const nsACString& aOriginNoSuffix)
 {
-  return new CacheParentBridge(this, aScope);
+  return new CacheParentBridge(this, aOriginSuffix, aOriginNoSuffix);
 }
 
 void
@@ -386,19 +384,21 @@ DOMStorageDBParent::ActorDestroy(ActorDestroyReason aWhy)
 }
 
 bool
-DOMStorageDBParent::RecvAsyncPreload(const nsCString& aScope, const bool& aPriority)
+DOMStorageDBParent::RecvAsyncPreload(const nsCString& aOriginSuffix,
+                                     const nsCString& aOriginNoSuffix,
+                                     const bool& aPriority)
 {
   DOMStorageDBBridge* db = DOMStorageCache::StartDatabase();
   if (!db) {
     return false;
   }
 
-  db->AsyncPreload(NewCache(aScope), aPriority);
+  db->AsyncPreload(NewCache(aOriginSuffix, aOriginNoSuffix), aPriority);
   return true;
 }
 
 bool
-DOMStorageDBParent::RecvAsyncGetUsage(const nsCString& aScope)
+DOMStorageDBParent::RecvAsyncGetUsage(const nsCString& aOriginNoSuffix)
 {
   DOMStorageDBBridge* db = DOMStorageCache::StartDatabase();
   if (!db) {
@@ -406,7 +406,7 @@ DOMStorageDBParent::RecvAsyncGetUsage(const nsCString& aScope)
   }
 
   // The object releases it self in LoadUsage method
-  RefPtr<UsageParentBridge> usage = new UsageParentBridge(this, aScope);
+  RefPtr<UsageParentBridge> usage = new UsageParentBridge(this, aOriginNoSuffix);
   db->AsyncGetUsage(usage);
   return true;
 }
@@ -420,13 +420,15 @@ namespace {
 class SyncLoadCacheHelper : public DOMStorageCacheBridge
 {
 public:
-  SyncLoadCacheHelper(const nsCString& aScope,
+  SyncLoadCacheHelper(const nsCString& aOriginSuffix,
+                      const nsCString& aOriginNoSuffix,
                       uint32_t aAlreadyLoadedCount,
                       InfallibleTArray<nsString>* aKeys,
                       InfallibleTArray<nsString>* aValues,
                       nsresult* rv)
   : mMonitor("DOM Storage SyncLoad IPC")
-  , mScope(aScope)
+  , mSuffix(aOriginSuffix)
+  , mOrigin(aOriginNoSuffix)
   , mKeys(aKeys)
   , mValues(aValues)
   , mRv(rv)
@@ -437,7 +439,12 @@ public:
     *mRv = NS_ERROR_UNEXPECTED;
   }
 
-  virtual const nsCString& Scope() const { return mScope; }
+  virtual const nsCString Origin() const
+  {
+    return DOMStorageManager::CreateOrigin(mSuffix, mOrigin);
+  }
+  virtual const nsCString& OriginNoSuffix() const { return mOrigin; }
+  virtual const nsCString& OriginSuffix() const { return mSuffix; }
   virtual bool Loaded() { return mLoaded; }
   virtual uint32_t LoadedCount() { return mLoadedCount; }
   virtual bool LoadItem(const nsAString& aKey, const nsString& aValue)
@@ -473,7 +480,7 @@ public:
 
 private:
   Monitor mMonitor;
-  nsCString mScope;
+  nsCString mSuffix, mOrigin;
   InfallibleTArray<nsString>* mKeys;
   InfallibleTArray<nsString>* mValues;
   nsresult* mRv;
@@ -484,7 +491,8 @@ private:
 } // namespace
 
 bool
-DOMStorageDBParent::RecvPreload(const nsCString& aScope,
+DOMStorageDBParent::RecvPreload(const nsCString& aOriginSuffix,
+                                const nsCString& aOriginNoSuffix,
                                 const uint32_t& aAlreadyLoadedCount,
                                 InfallibleTArray<nsString>* aKeys,
                                 InfallibleTArray<nsString>* aValues,
@@ -496,14 +504,15 @@ DOMStorageDBParent::RecvPreload(const nsCString& aScope,
   }
 
   RefPtr<SyncLoadCacheHelper> cache(
-    new SyncLoadCacheHelper(aScope, aAlreadyLoadedCount, aKeys, aValues, aRv));
+    new SyncLoadCacheHelper(aOriginSuffix, aOriginNoSuffix, aAlreadyLoadedCount, aKeys, aValues, aRv));
 
   db->SyncPreload(cache, true);
   return true;
 }
 
 bool
-DOMStorageDBParent::RecvAsyncAddItem(const nsCString& aScope,
+DOMStorageDBParent::RecvAsyncAddItem(const nsCString& aOriginSuffix,
+                                     const nsCString& aOriginNoSuffix,
                                      const nsString& aKey,
                                      const nsString& aValue)
 {
@@ -512,7 +521,7 @@ DOMStorageDBParent::RecvAsyncAddItem(const nsCString& aScope,
     return false;
   }
 
-  nsresult rv = db->AsyncAddItem(NewCache(aScope), aKey, aValue);
+  nsresult rv = db->AsyncAddItem(NewCache(aOriginSuffix, aOriginNoSuffix), aKey, aValue);
   if (NS_FAILED(rv) && mIPCOpen) {
     mozilla::Unused << SendError(rv);
   }
@@ -521,7 +530,8 @@ DOMStorageDBParent::RecvAsyncAddItem(const nsCString& aScope,
 }
 
 bool
-DOMStorageDBParent::RecvAsyncUpdateItem(const nsCString& aScope,
+DOMStorageDBParent::RecvAsyncUpdateItem(const nsCString& aOriginSuffix,
+                                        const nsCString& aOriginNoSuffix,
                                         const nsString& aKey,
                                         const nsString& aValue)
 {
@@ -530,7 +540,7 @@ DOMStorageDBParent::RecvAsyncUpdateItem(const nsCString& aScope,
     return false;
   }
 
-  nsresult rv = db->AsyncUpdateItem(NewCache(aScope), aKey, aValue);
+  nsresult rv = db->AsyncUpdateItem(NewCache(aOriginSuffix, aOriginNoSuffix), aKey, aValue);
   if (NS_FAILED(rv) && mIPCOpen) {
     mozilla::Unused << SendError(rv);
   }
@@ -539,7 +549,8 @@ DOMStorageDBParent::RecvAsyncUpdateItem(const nsCString& aScope,
 }
 
 bool
-DOMStorageDBParent::RecvAsyncRemoveItem(const nsCString& aScope,
+DOMStorageDBParent::RecvAsyncRemoveItem(const nsCString& aOriginSuffix,
+                                        const nsCString& aOriginNoSuffix,
                                         const nsString& aKey)
 {
   DOMStorageDBBridge* db = DOMStorageCache::StartDatabase();
@@ -547,7 +558,7 @@ DOMStorageDBParent::RecvAsyncRemoveItem(const nsCString& aScope,
     return false;
   }
 
-  nsresult rv = db->AsyncRemoveItem(NewCache(aScope), aKey);
+  nsresult rv = db->AsyncRemoveItem(NewCache(aOriginSuffix, aOriginNoSuffix), aKey);
   if (NS_FAILED(rv) && mIPCOpen) {
     mozilla::Unused << SendError(rv);
   }
@@ -556,14 +567,15 @@ DOMStorageDBParent::RecvAsyncRemoveItem(const nsCString& aScope,
 }
 
 bool
-DOMStorageDBParent::RecvAsyncClear(const nsCString& aScope)
+DOMStorageDBParent::RecvAsyncClear(const nsCString& aOriginSuffix,
+                                   const nsCString& aOriginNoSuffix)
 {
   DOMStorageDBBridge* db = DOMStorageCache::StartDatabase();
   if (!db) {
     return false;
   }
 
-  nsresult rv = db->AsyncClear(NewCache(aScope));
+  nsresult rv = db->AsyncClear(NewCache(aOriginSuffix, aOriginNoSuffix));
   if (NS_FAILED(rv) && mIPCOpen) {
     mozilla::Unused << SendError(rv);
   }
@@ -587,18 +599,13 @@ DOMStorageDBParent::RecvAsyncFlush()
 
 nsresult
 DOMStorageDBParent::Observe(const char* aTopic,
-                            const nsACString& aScopePrefix)
+                            const nsAString& aOriginAttributesPattern,
+                            const nsACString& aOriginScope)
 {
   if (mIPCOpen) {
-#ifdef MOZ_NUWA_PROCESS
-    if (!(static_cast<ContentParent*>(Manager())->IsNuwaProcess() &&
-          ContentParent::IsNuwaReady())) {
-#endif
       mozilla::Unused << SendObserve(nsDependentCString(aTopic),
-                                     nsCString(aScopePrefix));
-#ifdef MOZ_NUWA_PROCESS
-    }
-#endif
+                                     nsString(aOriginAttributesPattern),
+                                     nsCString(aOriginScope));
   }
 
   return NS_OK;
@@ -607,7 +614,7 @@ DOMStorageDBParent::Observe(const char* aTopic,
 namespace {
 
 // Results must be sent back on the main thread
-class LoadRunnable : public nsRunnable
+class LoadRunnable : public Runnable
 {
 public:
   enum TaskType {
@@ -617,35 +624,39 @@ public:
 
   LoadRunnable(DOMStorageDBParent* aParent,
                TaskType aType,
-               const nsACString& aScope,
+               const nsACString& aOriginSuffix,
+               const nsACString& aOriginNoSuffix,
                const nsAString& aKey = EmptyString(),
                const nsAString& aValue = EmptyString())
   : mParent(aParent)
   , mType(aType)
-  , mScope(aScope)
+  , mSuffix(aOriginSuffix)
+  , mOrigin(aOriginNoSuffix)
   , mKey(aKey)
   , mValue(aValue)
   { }
 
   LoadRunnable(DOMStorageDBParent* aParent,
                TaskType aType,
-               const nsACString& aScope,
+               const nsACString& aOriginSuffix,
+               const nsACString& aOriginNoSuffix,
                nsresult aRv)
   : mParent(aParent)
   , mType(aType)
-  , mScope(aScope)
+  , mSuffix(aOriginSuffix)
+  , mOrigin(aOriginNoSuffix)
   , mRv(aRv)
   { }
 
 private:
   RefPtr<DOMStorageDBParent> mParent;
   TaskType mType;
-  nsCString mScope;
+  nsCString mSuffix, mOrigin;
   nsString mKey;
   nsString mValue;
   nsresult mRv;
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     if (!mParent->IPCOpen()) {
       return NS_OK;
@@ -654,10 +665,10 @@ private:
     switch (mType)
     {
     case loadItem:
-      mozilla::Unused << mParent->SendLoadItem(mScope, mKey, mValue);
+      mozilla::Unused << mParent->SendLoadItem(mSuffix, mOrigin, mKey, mValue);
       break;
     case loadDone:
-      mozilla::Unused << mParent->SendLoadDone(mScope, mRv);
+      mozilla::Unused << mParent->SendLoadDone(mSuffix, mOrigin, mRv);
       break;
     }
 
@@ -669,6 +680,12 @@ private:
 
 // DOMStorageDBParent::CacheParentBridge
 
+const nsCString
+DOMStorageDBParent::CacheParentBridge::Origin() const
+{
+  return DOMStorageManager::CreateOrigin(mOriginSuffix, mOriginNoSuffix);
+}
+
 bool
 DOMStorageDBParent::CacheParentBridge::LoadItem(const nsAString& aKey, const nsString& aValue)
 {
@@ -679,7 +696,7 @@ DOMStorageDBParent::CacheParentBridge::LoadItem(const nsAString& aKey, const nsS
   ++mLoadedCount;
 
   RefPtr<LoadRunnable> r =
-    new LoadRunnable(mParent, LoadRunnable::loadItem, mScope, aKey, aValue);
+    new LoadRunnable(mParent, LoadRunnable::loadItem, mOriginSuffix, mOriginNoSuffix, aKey, aValue);
   NS_DispatchToMainThread(r);
   return true;
 }
@@ -695,7 +712,7 @@ DOMStorageDBParent::CacheParentBridge::LoadDone(nsresult aRv)
   mLoaded = true;
 
   RefPtr<LoadRunnable> r =
-    new LoadRunnable(mParent, LoadRunnable::loadDone, mScope, aRv);
+    new LoadRunnable(mParent, LoadRunnable::loadDone, mOriginSuffix, mOriginNoSuffix, aRv);
   NS_DispatchToMainThread(r);
 }
 
@@ -710,28 +727,28 @@ DOMStorageDBParent::CacheParentBridge::LoadWait()
 
 namespace {
 
-class UsageRunnable : public nsRunnable
+class UsageRunnable : public Runnable
 {
 public:
-  UsageRunnable(DOMStorageDBParent* aParent, const nsACString& aScope, const int64_t& aUsage)
+  UsageRunnable(DOMStorageDBParent* aParent, const nsACString& aOriginScope, const int64_t& aUsage)
   : mParent(aParent)
-  , mScope(aScope)
+  , mOriginScope(aOriginScope)
   , mUsage(aUsage)
   {}
 
 private:
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     if (!mParent->IPCOpen()) {
       return NS_OK;
     }
 
-    mozilla::Unused << mParent->SendLoadUsage(mScope, mUsage);
+    mozilla::Unused << mParent->SendLoadUsage(mOriginScope, mUsage);
     return NS_OK;
   }
 
   RefPtr<DOMStorageDBParent> mParent;
-  nsCString mScope;
+  nsCString mOriginScope;
   int64_t mUsage;
 };
 
@@ -740,7 +757,7 @@ private:
 void
 DOMStorageDBParent::UsageParentBridge::LoadUsage(const int64_t aUsage)
 {
-  RefPtr<UsageRunnable> r = new UsageRunnable(mParent, mScope, aUsage);
+  RefPtr<UsageRunnable> r = new UsageRunnable(mParent, mOriginScope, aUsage);
   NS_DispatchToMainThread(r);
 }
 
