@@ -9,7 +9,6 @@
 
 #include "MozMtpCommon.h"
 
-#include "nsAutoPtr.h"
 #include "mozilla/UniquePtr.h"
 
 #include "Volume.h"
@@ -26,7 +25,7 @@ public:
 
   MozMtpStorage(Volume* aVolume, MozMtpServer* aMozMtpServer);
 
-  typedef nsTArray<nsRefPtr<MozMtpStorage> > Array;
+  typedef nsTArray<RefPtr<MozMtpStorage> > Array;
 
 private:
   virtual ~MozMtpStorage();
@@ -35,7 +34,7 @@ private:
   void StorageAvailable();
   void StorageUnavailable();
 
-  nsRefPtr<MozMtpServer>  mMozMtpServer;
+  RefPtr<MozMtpServer>  mMozMtpServer;
   UniquePtr<MtpStorage>   mMtpStorage;
   RefPtr<Volume>          mVolume;
   MtpStorageID            mStorageID;
