@@ -31,13 +31,17 @@
 #include "HalLog.h"
 #include "nsDebug.h"
 #include "base/message_loop.h"
+#include "base/task.h"
+#include "mozilla/ClearOnShutdown.h"
 #include "mozilla/FileUtils.h"
-#include "nsAutoPtr.h"
+#include "mozilla/Monitor.h"
 #include "nsThreadUtils.h"
 #include "nsXULAppAPI.h"
 
 #include "UeventPoller.h"
 
+using namespace mozilla;
+
 namespace mozilla {
 namespace hal_impl {
 
@@ -72,8 +76,9 @@ public:
   void UnregisterObserver(IUeventObserver *aObserver)
   {
     mUeventObserverList.RemoveObserver(aObserver);
-    if (mUeventObserverList.Length() == 0)
+    if (mUeventObserverList.Length() == 0) {
       ShutdownUevent();  // this will destroy self
+    }
   }
 
 private:
@@ -92,13 +97,16 @@ bool
 NetlinkPoller::OpenSocket()
 {
   mSocket.rwget() = socket(PF_NETLINK, SOCK_DGRAM, NETLINK_KOBJECT_UEVENT);
-  if (mSocket.get() < 0)
+  if (mSocket.get() < 0) {
     return false;
+  }
 
   int sz = kBuffsize;
 
-  if (setsockopt(mSocket.get(), SOL_SOCKET, SO_RCVBUFFORCE, &sz, sizeof(sz)) < 0)
+  if (setsockopt(mSocket.get(), SOL_SOCKET, SO_RCVBUFFORCE, &sz,
+                 sizeof(sz)) < 0) {
     return false;
+  }
 
   // add FD_CLOEXEC flag
   int flags = fcntl(mSocket.get(), F_GETFD);
@@ -106,12 +114,14 @@ NetlinkPoller::OpenSocket()
       return false;
   }
   flags |= FD_CLOEXEC;
-  if (fcntl(mSocket.get(), F_SETFD, flags) == -1)
+  if (fcntl(mSocket.get(), F_SETFD, flags) == -1) {
     return false;
+  }
 
   // set non-blocking
-  if (fcntl(mSocket.get(), F_SETFL, O_NONBLOCK) == -1)
+  if (fcntl(mSocket.get(), F_SETFL, O_NONBLOCK) == -1) {
     return false;
+  }
 
   struct sockaddr_nl saddr;
   bzero(&saddr, sizeof(saddr));
@@ -150,19 +160,21 @@ NetlinkPoller::OpenSocket()
   return true;
 }
 
-static nsAutoPtr<NetlinkPoller> sPoller;
+static StaticAutoPtr<NetlinkPoller> sPoller;
 
-class UeventInitTask : public Task
+class UeventInitTask : public Runnable
 {
-  virtual void Run()
+  NS_IMETHOD Run() override
   {
     if (!sPoller) {
-      return;
+      return NS_OK;
     }
     if (sPoller->OpenSocket()) {
-      return;
+      return NS_OK;
     }
-    sPoller->GetIOLoop()->PostDelayedTask(FROM_HERE, new UeventInitTask(), 1000);
+    sPoller->GetIOLoop()->PostDelayedTask(MakeAndAddRef<UeventInitTask>(),
+                                          1000);
+    return NS_OK;
   }
 };
 
@@ -190,13 +202,76 @@ NetlinkPoller::OnFileCanReadWithoutBlocking(int fd)
   }
 }
 
+static bool sShutdown = false;
+
+class ShutdownNetlinkPoller;
+static StaticAutoPtr<ShutdownNetlinkPoller> sShutdownPoller;
+static Monitor* sMonitor = nullptr;
+
+class ShutdownNetlinkPoller {
+public:
+  ~ShutdownNetlinkPoller()
+  {
+    // This is called from KillClearOnShutdown() on the main thread.
+    MOZ_ASSERT(NS_IsMainThread());
+    MOZ_ASSERT(XRE_GetIOMessageLoop());
+
+    {
+      MonitorAutoLock lock(*sMonitor);
+
+      XRE_GetIOMessageLoop()->PostTask(
+          NewRunnableFunction(ShutdownUeventIOThread));
+
+      while (!sShutdown) {
+        lock.Wait();
+      }
+    }
+
+    sShutdown = true;
+    delete sMonitor;
+  }
+
+  static void MaybeInit()
+  {
+    MOZ_ASSERT(MessageLoop::current() == XRE_GetIOMessageLoop());
+    if (sShutdown || sMonitor) {
+      // Don't init twice or init after shutdown.
+      return;
+    }
+
+    sMonitor = new Monitor("ShutdownNetlinkPoller.monitor");
+    {
+      ShutdownNetlinkPoller* shutdownPoller = new ShutdownNetlinkPoller();
+
+      nsCOMPtr<nsIRunnable> runnable = NS_NewRunnableFunction([=] () -> void
+      {
+        sShutdownPoller = shutdownPoller;
+        ClearOnShutdown(&sShutdownPoller); // Must run on the main thread.
+      });
+      MOZ_ASSERT(runnable);
+      MOZ_ALWAYS_SUCCEEDS(
+        NS_DispatchToMainThread(runnable, NS_DISPATCH_NORMAL));
+    }
+  }
+private:
+  ShutdownNetlinkPoller() = default;
+  static void ShutdownUeventIOThread()
+  {
+    MonitorAutoLock l(*sMonitor);
+    ShutdownUevent(); // Must run on the IO thread.
+    sShutdown = true;
+    l.NotifyAll();
+  }
+};
+
 static void
 InitializeUevent()
 {
   MOZ_ASSERT(!sPoller);
   sPoller = new NetlinkPoller();
-  sPoller->GetIOLoop()->PostTask(FROM_HERE, new UeventInitTask());
+  sPoller->GetIOLoop()->PostTask(MakeAndAddRef<UeventInitTask>());
 
+  ShutdownNetlinkPoller::MaybeInit();
 }
 
 static void
@@ -210,8 +285,13 @@ RegisterUeventListener(IUeventObserver *aObserver)
 {
   MOZ_ASSERT(MessageLoop::current() == XRE_GetIOMessageLoop());
 
-  if (!sPoller)
+  if (sShutdown) {
+    return;
+  }
+
+  if (!sPoller) {
     InitializeUevent();
+  }
   sPoller->RegisterObserver(aObserver);
 }
 
@@ -220,6 +300,10 @@ UnregisterUeventListener(IUeventObserver *aObserver)
 {
   MOZ_ASSERT(MessageLoop::current() == XRE_GetIOMessageLoop());
 
+  if (sShutdown) {
+    return;
+  }
+
   sPoller->UnregisterObserver(aObserver);
 }
 
