@@ -4,8 +4,13 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "ImageLogging.h"
+#include "ImageLogging.h"  // Must appear first.
+
 #include "nsJPEGDecoder.h"
+
+#include <cstdint>
+
+#include "imgFrame.h"
 #include "Orientation.h"
 #include "EXIF.h"
 
@@ -18,7 +23,7 @@
 #include "jerror.h"
 
 #include "gfxPlatform.h"
-#include "mozilla/Endian.h"
+#include "mozilla/EndianUtils.h"
 #include "mozilla/Telemetry.h"
 
 extern "C" {
@@ -36,30 +41,9 @@ static void cmyk_convert_rgb(JSAMPROW row, JDIMENSION width);
 namespace mozilla {
 namespace image {
 
-#if defined(PR_LOGGING)
-static PRLogModuleInfo*
-GetJPEGLog()
-{
-  static PRLogModuleInfo* sJPEGLog;
-  if (!sJPEGLog) {
-    sJPEGLog = PR_NewLogModule("JPEGDecoder");
-  }
-  return sJPEGLog;
-}
+static mozilla::LazyLogModule sJPEGLog("JPEGDecoder");
 
-static PRLogModuleInfo*
-GetJPEGDecoderAccountingLog()
-{
-  static PRLogModuleInfo* sJPEGDecoderAccountingLog;
-  if (!sJPEGDecoderAccountingLog) {
-    sJPEGDecoderAccountingLog = PR_NewLogModule("JPEGDecoderAccounting");
-  }
-  return sJPEGDecoderAccountingLog;
-}
-#else
-#define GetJPEGLog()
-#define GetJPEGDecoderAccountingLog()
-#endif
+static mozilla::LazyLogModule sJPEGDecoderAccountingLog("JPEGDecoderAccounting");
 
 static qcms_profile*
 GetICCProfile(struct jpeg_decompress_struct& info)
@@ -85,11 +69,15 @@ METHODDEF(void) my_error_exit (j_common_ptr cinfo);
 // Normal JFIF markers can't have more bytes than this.
 #define MAX_JPEG_MARKER_LENGTH  (((uint32_t)1 << 16) - 1)
 
-
 nsJPEGDecoder::nsJPEGDecoder(RasterImage* aImage,
                              Decoder::DecodeStyle aDecodeStyle)
  : Decoder(aImage)
+ , mLexer(Transition::ToUnbuffered(State::FINISHED_JPEG_DATA,
+                                   State::JPEG_DATA,
+                                   SIZE_MAX),
+          Transition::TerminateSuccess())
  , mDecodeStyle(aDecodeStyle)
+ , mSampleSize(0)
 {
   mState = JPEG_HEADER;
   mReading = true;
@@ -111,7 +99,7 @@ nsJPEGDecoder::nsJPEGDecoder(RasterImage* aImage,
 
   mCMSMode = 0;
 
-  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+  MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
          ("nsJPEGDecoder::nsJPEGDecoder: Creating JPEG decoder %p",
           this));
 }
@@ -130,36 +118,22 @@ nsJPEGDecoder::~nsJPEGDecoder()
     qcms_profile_release(mInProfile);
   }
 
-  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+  MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
          ("nsJPEGDecoder::~nsJPEGDecoder: Destroying JPEG decoder %p",
           this));
 }
 
-Telemetry::ID
-nsJPEGDecoder::SpeedHistogram()
+Maybe<Telemetry::ID>
+nsJPEGDecoder::SpeedHistogram() const
 {
-  return Telemetry::IMAGE_DECODE_SPEED_JPEG;
+  return Some(Telemetry::IMAGE_DECODE_SPEED_JPEG);
 }
 
 nsresult
-nsJPEGDecoder::SetTargetSize(const nsIntSize& aSize)
-{
-  // Make sure the size is reasonable.
-  if (MOZ_UNLIKELY(aSize.width <= 0 || aSize.height <= 0)) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // Create a downscaler that we'll filter our output through.
-  mDownscaler.emplace(aSize);
-
-  return NS_OK;
-}
-
-void
 nsJPEGDecoder::InitInternal()
 {
   mCMSMode = gfxPlatform::GetCMSMode();
-  if (GetDecodeFlags() & imgIContainer::FLAG_DECODE_NO_COLORSPACE_CONVERSION) {
+  if (GetSurfaceFlags() & SurfaceFlags::NO_COLORSPACE_CONVERSION) {
     mCMSMode = eCMSMode_Off;
   }
 
@@ -169,10 +143,9 @@ nsJPEGDecoder::InitInternal()
   mErr.pub.error_exit = my_error_exit;
   // Establish the setjmp return context for my_error_exit to use.
   if (setjmp(mErr.setjmp_buffer)) {
-    // If we get here, the JPEG code has signaled an error.
-    // We need to clean up the JPEG object, close the input file, and return.
-    PostDecoderError(NS_ERROR_FAILURE);
-    return;
+    // If we get here, the JPEG code has signaled an error, and initialization
+    // has failed.
+    return NS_ERROR_FAILURE;
   }
 
   // Step 1: allocate and initialize JPEG decompression object
@@ -190,73 +163,89 @@ nsJPEGDecoder::InitInternal()
   mSourceMgr.term_source = term_source;
 
   // Record app markers for ICC data
-  for (uint32_t m = 0; m < 16; m++)
+  for (uint32_t m = 0; m < 16; m++) {
     jpeg_save_markers(&mInfo, JPEG_APP0 + m, 0xFFFF);
+  }
+
+  return NS_OK;
 }
 
-void
+nsresult
 nsJPEGDecoder::FinishInternal()
 {
   // If we're not in any sort of error case, force our state to JPEG_DONE.
   if ((mState != JPEG_DONE && mState != JPEG_SINK_NON_JPEG_TRAILER) &&
       (mState != JPEG_ERROR) &&
-      !IsSizeDecode()) {
+      !IsMetadataDecode()) {
     mState = JPEG_DONE;
   }
+
+  return NS_OK;
 }
 
-void
-nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
+LexerResult
+nsJPEGDecoder::DoDecode(SourceBufferIterator& aIterator, IResumable* aOnResume)
 {
-  mSegment = (const JOCTET*)aBuffer;
-  mSegmentLen = aCount;
+  MOZ_ASSERT(!HasError(), "Shouldn't call DoDecode after error!");
 
-  MOZ_ASSERT(!HasError(), "Shouldn't call WriteInternal after error!");
+  return mLexer.Lex(aIterator, aOnResume,
+                    [=](State aState, const char* aData, size_t aLength) {
+    switch (aState) {
+      case State::JPEG_DATA:
+        return ReadJPEGData(aData, aLength);
+      case State::FINISHED_JPEG_DATA:
+        return FinishedJPEGData();
+    }
+    MOZ_CRASH("Unknown State");
+  });
+}
+
+LexerTransition<nsJPEGDecoder::State>
+nsJPEGDecoder::ReadJPEGData(const char* aData, size_t aLength)
+{
+  mSegment = reinterpret_cast<const JOCTET*>(aData);
+  mSegmentLen = aLength;
 
   // Return here if there is a fatal error within libjpeg.
   nsresult error_code;
   // This cast to nsresult makes sense because setjmp() returns whatever we
   // passed to longjmp(), which was actually an nsresult.
-  if ((error_code = (nsresult)setjmp(mErr.setjmp_buffer)) != NS_OK) {
+  if ((error_code = static_cast<nsresult>(setjmp(mErr.setjmp_buffer))) != NS_OK) {
     if (error_code == NS_ERROR_FAILURE) {
-      PostDataError();
-      // Error due to corrupt stream - return NS_OK and consume silently
-      // so that ImageLib doesn't throw away a partial image load
+      // Error due to corrupt data. Make sure that we don't feed any more data
+      // to libjpeg-turbo.
       mState = JPEG_SINK_NON_JPEG_TRAILER;
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+      MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
              ("} (setjmp returned NS_ERROR_FAILURE)"));
-      return;
     } else {
-      // Error due to reasons external to the stream (probably out of
-      // memory) - let ImageLib attempt to clean up, even though
-      // mozilla is seconds away from falling flat on its face.
-      PostDecoderError(error_code);
+      // Error for another reason. (Possibly OOM.)
       mState = JPEG_ERROR;
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+      MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
              ("} (setjmp returned an error)"));
-      return;
     }
+
+    return Transition::TerminateFailure();
   }
 
-  PR_LOG(GetJPEGLog(), PR_LOG_DEBUG,
+  MOZ_LOG(sJPEGLog, LogLevel::Debug,
          ("[this=%p] nsJPEGDecoder::Write -- processing JPEG data\n", this));
 
   switch (mState) {
     case JPEG_HEADER: {
-      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_HEADER"
+      LOG_SCOPE((mozilla::LogModule*)sJPEGLog, "nsJPEGDecoder::Write -- entering JPEG_HEADER"
                 " case");
 
       // Step 3: read file parameters with jpeg_read_header()
       if (jpeg_read_header(&mInfo, TRUE) == JPEG_SUSPENDED) {
-        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+        MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                ("} (JPEG_SUSPENDED)"));
-        return; // I/O suspension
+        return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
       }
 
-      int sampleSize = mImage->GetRequestedSampleSize();
-      if (sampleSize > 0) {
+      // If we have a sample size specified for -moz-sample-size, use it.
+      if (mSampleSize > 0) {
         mInfo.scale_num = 1;
-        mInfo.scale_denom = sampleSize;
+        mInfo.scale_denom = mSampleSize;
       }
 
       // Used to set up image size so arrays can be allocated
@@ -268,12 +257,12 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
       if (HasError()) {
         // Setting the size led to an error.
         mState = JPEG_ERROR;
-        return;
+        return Transition::TerminateFailure();
       }
 
-      // If we're doing a size decode, we're done.
-      if (IsSizeDecode()) {
-        return;
+      // If we're doing a metadata decode, we're done.
+      if (IsMetadataDecode()) {
+        return Transition::TerminateSuccess();
       }
 
       // We're doing a full decode.
@@ -313,10 +302,9 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
           break;
         default:
           mState = JPEG_ERROR;
-          PostDataError();
-          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+          MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                  ("} (unknown colorpsace (1))"));
-          return;
+          return Transition::TerminateFailure();
       }
 
       if (!mismatch) {
@@ -330,10 +318,9 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
             break;
           default:
             mState = JPEG_ERROR;
-            PostDataError();
-            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+            MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                    ("} (unknown colorpsace (2))"));
-            return;
+            return Transition::TerminateFailure();
         }
 #if 0
         // We don't currently support CMYK profiles. The following
@@ -389,11 +376,9 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
           break;
         default:
           mState = JPEG_ERROR;
-          PostDataError();
-          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+          MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                  ("} (unknown colorpsace (3))"));
-          return;
-          break;
+          return Transition::TerminateFailure();
       }
     }
 
@@ -402,35 +387,39 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
     mInfo.buffered_image = mDecodeStyle == PROGRESSIVE &&
                            jpeg_has_multiple_scans(&mInfo);
 
-    if (!mImageData) {
+    MOZ_ASSERT(!mImageData, "Already have a buffer allocated?");
+    nsresult rv = AllocateFrame(/* aFrameNum = */ 0, OutputSize(),
+                                FullOutputFrame(), SurfaceFormat::B8G8R8X8);
+    if (NS_FAILED(rv)) {
       mState = JPEG_ERROR;
-      PostDecoderError(NS_ERROR_OUT_OF_MEMORY);
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+      MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
              ("} (could not initialize image frame)"));
-      return;
+      return Transition::TerminateFailure();
     }
 
+    MOZ_ASSERT(mImageData, "Should have a buffer now");
+
     if (mDownscaler) {
-      nsresult rv = mDownscaler->BeginFrame(GetSize(),
+      nsresult rv = mDownscaler->BeginFrame(Size(), Nothing(),
                                             mImageData,
                                             /* aHasAlpha = */ false);
       if (NS_FAILED(rv)) {
         mState = JPEG_ERROR;
-        return;
+        return Transition::TerminateFailure();
       }
     }
 
-
-    PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+    MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
            ("        JPEGDecoderAccounting: nsJPEGDecoder::"
             "Write -- created image frame with %ux%u pixels",
             mInfo.output_width, mInfo.output_height));
 
     mState = JPEG_START_DECOMPRESS;
+    MOZ_FALLTHROUGH; // to start decompressing.
   }
 
   case JPEG_START_DECOMPRESS: {
-    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering"
+    LOG_SCOPE((mozilla::LogModule*)sJPEGLog, "nsJPEGDecoder::Write -- entering"
                             " JPEG_START_DECOMPRESS case");
     // Step 4: set parameters for decompression
 
@@ -445,29 +434,29 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
 
     // Step 5: Start decompressor
     if (jpeg_start_decompress(&mInfo) == FALSE) {
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+      MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
              ("} (I/O suspension after jpeg_start_decompress())"));
-      return; // I/O suspension
+      return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
     }
 
-
     // If this is a progressive JPEG ...
     mState = mInfo.buffered_image ?
              JPEG_DECOMPRESS_PROGRESSIVE : JPEG_DECOMPRESS_SEQUENTIAL;
+    MOZ_FALLTHROUGH; // to decompress sequential JPEG.
   }
 
   case JPEG_DECOMPRESS_SEQUENTIAL: {
     if (mState == JPEG_DECOMPRESS_SEQUENTIAL) {
-      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- "
+      LOG_SCOPE((mozilla::LogModule*)sJPEGLog, "nsJPEGDecoder::Write -- "
                               "JPEG_DECOMPRESS_SEQUENTIAL case");
 
       bool suspend;
       OutputScanlines(&suspend);
 
       if (suspend) {
-        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+        MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                ("} (I/O suspension after OutputScanlines() - SEQUENTIAL)"));
-        return; // I/O suspension
+        return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
       }
 
       // If we've completed image output ...
@@ -475,11 +464,12 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                    "We didn't process all of the data!");
       mState = JPEG_DONE;
     }
+    MOZ_FALLTHROUGH; // to decompress progressive JPEG.
   }
 
   case JPEG_DECOMPRESS_PROGRESSIVE: {
     if (mState == JPEG_DECOMPRESS_PROGRESSIVE) {
-      LOG_SCOPE(GetJPEGLog(),
+      LOG_SCOPE((mozilla::LogModule*)sJPEGLog,
                 "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_PROGRESSIVE case");
 
       int status;
@@ -501,10 +491,10 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
             scan--;
 
           if (!jpeg_start_output(&mInfo, scan)) {
-            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+            MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                    ("} (I/O suspension after jpeg_start_output() -"
                     " PROGRESSIVE)"));
-            return; // I/O suspension
+            return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
           }
         }
 
@@ -521,17 +511,17 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
             // jpeg_start_output() multiple times for the same scan
             mInfo.output_scanline = 0xffffff;
           }
-          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+          MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                  ("} (I/O suspension after OutputScanlines() - PROGRESSIVE)"));
-          return; // I/O suspension
+          return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
         }
 
         if (mInfo.output_scanline == mInfo.output_height) {
           if (!jpeg_finish_output(&mInfo)) {
-            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+            MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
                    ("} (I/O suspension after jpeg_finish_output() -"
                     " PROGRESSIVE)"));
-            return; // I/O suspension
+            return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
           }
 
           if (jpeg_input_complete(&mInfo) &&
@@ -547,41 +537,55 @@ nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
 
       mState = JPEG_DONE;
     }
+    MOZ_FALLTHROUGH; // to finish decompressing.
   }
 
   case JPEG_DONE: {
-    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::ProcessData -- entering"
+    LOG_SCOPE((mozilla::LogModule*)sJPEGLog, "nsJPEGDecoder::ProcessData -- entering"
                             " JPEG_DONE case");
 
     // Step 7: Finish decompression
 
     if (jpeg_finish_decompress(&mInfo) == FALSE) {
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+      MOZ_LOG(sJPEGDecoderAccountingLog, LogLevel::Debug,
              ("} (I/O suspension after jpeg_finish_decompress() - DONE)"));
-      return; // I/O suspension
+      return Transition::ContinueUnbuffered(State::JPEG_DATA); // I/O suspension
     }
 
+    // Make sure we don't feed any more data to libjpeg-turbo.
     mState = JPEG_SINK_NON_JPEG_TRAILER;
 
-    // we're done dude
-    break;
+    // We're done.
+    return Transition::TerminateSuccess();
   }
   case JPEG_SINK_NON_JPEG_TRAILER:
-    PR_LOG(GetJPEGLog(), PR_LOG_DEBUG,
+    MOZ_LOG(sJPEGLog, LogLevel::Debug,
            ("[this=%p] nsJPEGDecoder::ProcessData -- entering"
             " JPEG_SINK_NON_JPEG_TRAILER case\n", this));
 
-    break;
+    MOZ_ASSERT_UNREACHABLE("Should stop getting data after entering state "
+                           "JPEG_SINK_NON_JPEG_TRAILER");
+
+    return Transition::TerminateSuccess();
 
   case JPEG_ERROR:
-    MOZ_ASSERT(false,
-               "Should always return immediately after error and not re-enter "
-               "decoder");
+    MOZ_ASSERT_UNREACHABLE("Should stop getting data after entering state "
+                           "JPEG_ERROR");
+
+    return Transition::TerminateFailure();
   }
 
-  PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-         ("} (end of function)"));
-  return;
+  MOZ_ASSERT_UNREACHABLE("Escaped the JPEG decoder state machine");
+  return Transition::TerminateFailure();
+}
+
+LexerTransition<nsJPEGDecoder::State>
+nsJPEGDecoder::FinishedJPEGData()
+{
+  // Since we set up an unbuffered read for SIZE_MAX bytes, if we actually read
+  // all that data something is really wrong.
+  MOZ_ASSERT_UNREACHABLE("Read the entire address space?");
+  return Transition::TerminateFailure();
 }
 
 Orientation
@@ -610,7 +614,7 @@ nsJPEGDecoder::ReadOrientationFromEXIF()
 void
 nsJPEGDecoder::NotifyDone()
 {
-  PostFrameStop(Opacity::OPAQUE);
+  PostFrameStop(Opacity::FULLY_OPAQUE);
   PostDecodeDone();
 }
 
@@ -722,19 +726,18 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
       }
   }
 
-  if (top != mInfo.output_scanline) {
+  if (mDownscaler && mDownscaler->HasInvalidation()) {
+    DownscalerInvalidRect invalidRect = mDownscaler->TakeInvalidRect();
+    PostInvalidation(invalidRect.mOriginalSizeRect,
+                     Some(invalidRect.mTargetSizeRect));
+    MOZ_ASSERT(!mDownscaler->HasInvalidation());
+  } else if (!mDownscaler && top != mInfo.output_scanline) {
     PostInvalidation(nsIntRect(0, top,
                                mInfo.output_width,
-                               mInfo.output_scanline - top),
-                     mDownscaler ? Some(mDownscaler->TakeInvalidRect())
-                                 : Nothing());
+                               mInfo.output_scanline - top));
   }
-
-  MOZ_ASSERT(!mDownscaler || !mDownscaler->HasInvalidation(),
-             "Didn't send downscaler's invalidation");
 }
 
-
 // Override the standard error method in the IJG JPEG decoder code.
 METHODDEF(void)
 my_error_exit (j_common_ptr cinfo)
@@ -838,7 +841,6 @@ skip_input_data (j_decompress_ptr jd, long num_bytes)
   }
 }
 
-
 /******************************************************************************/
 /* data source manager method
         This is called whenever bytes_in_buffer has reached zero and more
@@ -958,7 +960,6 @@ term_source (j_decompress_ptr jd)
 } // namespace image
 } // namespace mozilla
 
-
 ///*************** Inverted CMYK -> RGB conversion *************************
 /// Input is (Inverted) CMYK stored as 4 bytes per pixel.
 /// Output is RGB stored as 3 bytes per pixel.
