@@ -1,17 +1,14 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "CrossProcessMutex.h"
-#include "mozilla/unused.h"
+#include "mozilla/Unused.h"
 #include "nsDebug.h"
 #include "nsISupportsImpl.h"
 
-#ifdef OS_MACOSX
-#include "nsCocoaFeatures.h"
-#endif
-
 namespace {
 
 struct MutexData {
@@ -19,7 +16,7 @@ struct MutexData {
   mozilla::Atomic<int32_t> mCount;
 };
 
-}
+} // namespace
 
 namespace mozilla {
 
@@ -45,15 +42,6 @@ CrossProcessMutex::CrossProcessMutex(const char*)
     : mMutex(nullptr)
     , mCount(nullptr)
 {
-#ifdef OS_MACOSX
-  if (!nsCocoaFeatures::OnLionOrLater()) {
-    // Don't allow using the cross-process mutex before OS X 10.7 because it
-    // probably doesn't work very well. See discussion in bug 1072093 for more
-    // details.
-    MOZ_CRASH();
-  }
-#endif
-
   mSharedBuffer = new ipc::SharedMemoryBasic;
   if (!mSharedBuffer->Create(sizeof(MutexData))) {
     MOZ_CRASH();
@@ -82,11 +70,15 @@ CrossProcessMutex::CrossProcessMutex(CrossProcessMutexHandle aHandle)
     : mMutex(nullptr)
     , mCount(nullptr)
 {
-  if (!ipc::SharedMemoryBasic::IsHandleValid(aHandle)) {
+  mSharedBuffer = new ipc::SharedMemoryBasic;
+
+  if (!mSharedBuffer->IsHandleValid(aHandle)) {
     MOZ_CRASH();
   }
 
-  mSharedBuffer = new ipc::SharedMemoryBasic(aHandle);
+  if (!mSharedBuffer->SetHandle(aHandle)) {
+    MOZ_CRASH();
+  }
 
   if (!mSharedBuffer->Map(sizeof(MutexData))) {
     MOZ_CRASH();
@@ -117,7 +109,7 @@ CrossProcessMutex::~CrossProcessMutex()
 
   if (count == 0) {
     // Nothing can be done if the destroy fails so ignore return code.
-    unused << pthread_mutex_destroy(mMutex);
+    Unused << pthread_mutex_destroy(mMutex);
   }
 
   MOZ_COUNT_DTOR(CrossProcessMutex);
@@ -138,15 +130,15 @@ CrossProcessMutex::Unlock()
 }
 
 CrossProcessMutexHandle
-CrossProcessMutex::ShareToProcess(base::ProcessHandle aHandle)
+CrossProcessMutex::ShareToProcess(base::ProcessId aTargetPid)
 {
   CrossProcessMutexHandle result = ipc::SharedMemoryBasic::NULLHandle();
 
-  if (mSharedBuffer && !mSharedBuffer->ShareToProcess(aHandle, &result)) {
+  if (mSharedBuffer && !mSharedBuffer->ShareToProcess(aTargetPid, &result)) {
     MOZ_CRASH();
   }
 
   return result;
 }
 
-}
+} // namespace mozilla
