@@ -18,6 +18,7 @@
 #include <vector>
 
 #include "ClearKeyDecryptionManager.h"
+#include "psshparser/PsshParser.h"
 #include "gmp-api/gmp-decryption.h"
 #include <assert.h>
 
@@ -156,7 +157,11 @@ ClearKeyDecryptor::ClearKeyDecryptor()
 
 ClearKeyDecryptor::~ClearKeyDecryptor()
 {
-  CK_LOGD("ClearKeyDecryptor dtor; key = %08x...", *(uint32_t*)&mKey[0]);
+  if (HasKey()) {
+    CK_LOGD("ClearKeyDecryptor dtor; key = %08x...", *(uint32_t*)&mKey[0]);
+  } else {
+    CK_LOGD("ClearKeyDecryptor dtor");
+  }
 }
 
 void
@@ -200,7 +205,7 @@ ClearKeyDecryptor::Decrypt(uint8_t* aBuffer, uint32_t aBufferSize,
 
   assert(aMetadata.mIV.size() == 8 || aMetadata.mIV.size() == 16);
   std::vector<uint8_t> iv(aMetadata.mIV);
-  iv.insert(iv.end(), CLEARKEY_KEY_LEN - aMetadata.mIV.size(), 0);
+  iv.insert(iv.end(), CENC_KEY_LEN - aMetadata.mIV.size(), 0);
 
   ClearKeyUtils::DecryptAES(mKey, tmp, iv);
 
