@@ -11,9 +11,9 @@
 #ifndef WEBRTC_COMMON_AUDIO_FIR_FILTER_SSE_H_
 #define WEBRTC_COMMON_AUDIO_FIR_FILTER_SSE_H_
 
+#include "webrtc/base/scoped_ptr.h"
 #include "webrtc/common_audio/fir_filter.h"
 #include "webrtc/system_wrappers/interface/aligned_malloc.h"
-#include "webrtc/system_wrappers/interface/scoped_ptr.h"
 
 namespace webrtc {
 
@@ -23,13 +23,13 @@ class FIRFilterSSE2 : public FIRFilter {
                 size_t coefficients_length,
                 size_t max_input_length);
 
-  virtual void Filter(const float* in, size_t length, float* out) OVERRIDE;
+  void Filter(const float* in, size_t length, float* out) override;
 
  private:
   size_t coefficients_length_;
   size_t state_length_;
-  scoped_ptr<float[], AlignedFreeDeleter> coefficients_;
-  scoped_ptr<float[], AlignedFreeDeleter> state_;
+  rtc::scoped_ptr<float[], AlignedFreeDeleter> coefficients_;
+  rtc::scoped_ptr<float[], AlignedFreeDeleter> state_;
 };
 
 }  // namespace webrtc
