@@ -28,14 +28,10 @@ VideoReceiver::VideoReceiver(Clock* clock, EventFactory* event_factory)
     : clock_(clock),
       process_crit_sect_(CriticalSectionWrapper::CreateCriticalSection()),
       _receiveCritSect(CriticalSectionWrapper::CreateCriticalSection()),
-      _receiverInited(false),
       _receiveState(kReceiveStateInitial),
       _timing(clock_),
-      _dualTiming(clock_, &_timing),
       _receiver(&_timing, clock_, event_factory, true),
-      _dualReceiver(&_dualTiming, clock_, event_factory, false),
       _decodedFrameCallback(_timing, clock_),
-      _dualDecodedFrameCallback(_dualTiming, clock_),
       _frameTypeCallback(NULL),
       _receiveStatsCallback(NULL),
       _decoderTimingCallback(NULL),
@@ -43,7 +39,6 @@ VideoReceiver::VideoReceiver(Clock* clock, EventFactory* event_factory)
       _receiveStateCallback(NULL),
       render_buffer_callback_(NULL),
       _decoder(NULL),
-      _dualDecoder(NULL),
 #ifdef DEBUG_DECODER_BIT_STREAM
       _bitStreamBeforeDecoder(NULL),
 #endif
@@ -52,7 +47,7 @@ VideoReceiver::VideoReceiver(Clock* clock, EventFactory* event_factory)
       _scheduleKeyRequest(false),
       max_nack_list_size_(0),
       pre_decode_image_callback_(NULL),
-      _codecDataBase(),
+      _codecDataBase(NULL),
       _receiveStatsTimer(1000, clock_),
       _retransmissionTimer(10, clock_),
       _keyRequestTimer(500, clock_) {
@@ -63,9 +58,6 @@ VideoReceiver::VideoReceiver(Clock* clock, EventFactory* event_factory)
 }
 
 VideoReceiver::~VideoReceiver() {
-  if (_dualDecoder != NULL) {
-    _codecDataBase.ReleaseDecoder(_dualDecoder);
-  }
   delete _receiveCritSect;
 #ifdef DEBUG_DECODER_BIT_STREAM
   fclose(_bitStreamBeforeDecoder);
@@ -83,7 +75,7 @@ int32_t VideoReceiver::Process() {
       uint32_t bitRate;
       uint32_t frameRate;
       _receiver.ReceiveStatistics(&bitRate, &frameRate);
-      _receiveStatsCallback->OnReceiveStatisticsUpdate(bitRate, frameRate);
+      _receiveStatsCallback->OnReceiveRatesUpdated(bitRate, frameRate);
     }
 
     if (_decoderTimingCallback != NULL) {
@@ -182,10 +174,9 @@ void VideoReceiver::SetReceiveState(VideoReceiveState state) {
   }
 }
 
-int32_t VideoReceiver::TimeUntilNextProcess() {
-  uint32_t timeUntilNextProcess = _receiveStatsTimer.TimeUntilProcess();
-  if ((_receiver.NackMode() != kNoNack) ||
-      (_dualReceiver.State() != kPassive)) {
+int64_t VideoReceiver::TimeUntilNextProcess() {
+  int64_t timeUntilNextProcess = _receiveStatsTimer.TimeUntilProcess();
+  if (_receiver.NackMode() != kNoNack) {
     // We need a Process call more often if we are relying on
     // retransmissions
     timeUntilNextProcess =
@@ -197,7 +188,7 @@ int32_t VideoReceiver::TimeUntilNextProcess() {
   return timeUntilNextProcess;
 }
 
-int32_t VideoReceiver::SetReceiveChannelParameters(uint32_t rtt) {
+int32_t VideoReceiver::SetReceiveChannelParameters(int64_t rtt) {
   CriticalSectionScoped receiveCs(_receiveCritSect);
   _receiver.UpdateRtt(rtt);
   return 0;
@@ -211,8 +202,6 @@ int32_t VideoReceiver::SetVideoProtection(VCMVideoProtection videoProtection,
                                           bool enable) {
   // By default, do not decode with errors.
   _receiver.SetDecodeErrorMode(kNoErrors);
-  // The dual decoder should always be error free.
-  _dualReceiver.SetDecodeErrorMode(kNoErrors);
   switch (videoProtection) {
     case kProtectionNack:
     case kProtectionNackReceiver: {
@@ -226,22 +215,6 @@ int32_t VideoReceiver::SetVideoProtection(VCMVideoProtection videoProtection,
       break;
     }
 
-    case kProtectionDualDecoder: {
-      CriticalSectionScoped cs(_receiveCritSect);
-      if (enable) {
-        // Enable NACK but don't wait for retransmissions and don't
-        // add any extra delay.
-        _receiver.SetNackMode(kNack, 0, 0);
-        // Enable NACK and always wait for retransmissions and
-        // compensate with extra delay.
-        _dualReceiver.SetNackMode(kNack, -1, -1);
-        _receiver.SetDecodeErrorMode(kWithErrors);
-      } else {
-        _dualReceiver.SetNackMode(kNoNack, -1, -1);
-      }
-      break;
-    }
-
     case kProtectionKeyOnLoss: {
       CriticalSectionScoped cs(_receiveCritSect);
       if (enable) {
@@ -283,9 +256,12 @@ int32_t VideoReceiver::SetVideoProtection(VCMVideoProtection videoProtection,
     }
     case kProtectionNackSender:
     case kProtectionFEC:
-    case kProtectionPeriodicKeyFrames:
       // Ignore encoder modes.
       return VCM_OK;
+    case kProtectionNone:
+      // TODO(pbos): Implement like sender and remove enable parameter. Ignored
+      // for now.
+      break;
   }
   return VCM_OK;
 }
@@ -297,16 +273,10 @@ int32_t VideoReceiver::InitializeReceiver() {
     return ret;
   }
 
-  ret = _dualReceiver.Initialize();
-  if (ret < 0) {
-    return ret;
-  }
-
   {
     CriticalSectionScoped receive_cs(_receiveCritSect);
     _codecDataBase.ResetReceiver();
     _timing.Reset();
-    _receiverInited = true;
   }
 
   {
@@ -337,6 +307,7 @@ int32_t VideoReceiver::RegisterReceiveCallback(
 int32_t VideoReceiver::RegisterReceiveStatisticsCallback(
     VCMReceiveStatisticsCallback* receiveStats) {
   CriticalSectionScoped cs(process_crit_sect_.get());
+  _receiver.RegisterStatsCallback(receiveStats);
   _receiveStatsCallback = receiveStats;
   return VCM_OK;
 }
@@ -394,6 +365,10 @@ int VideoReceiver::RegisterRenderBufferSizeCallback(
   return VCM_OK;
 }
 
+void VideoReceiver::TriggerDecoderShutdown() {
+  _receiver.TriggerDecoderShutdown();
+}
+
 // Decode next frame, blocking.
 // Should be called as often as possible to get the most out of the decoder.
 int32_t VideoReceiver::Decode(uint16_t maxWaitTimeMs) {
@@ -401,46 +376,11 @@ int32_t VideoReceiver::Decode(uint16_t maxWaitTimeMs) {
   bool supports_render_scheduling;
   {
     CriticalSectionScoped cs(_receiveCritSect);
-    if (!_receiverInited) {
-      return VCM_UNINITIALIZED;
-    }
-    if (!_codecDataBase.DecoderRegistered()) {
-      return VCM_NO_CODEC_REGISTERED;
-    }
     supports_render_scheduling = _codecDataBase.SupportsRenderScheduling();
   }
 
-  const bool dualReceiverEnabledNotReceiving = (
-      _dualReceiver.State() != kReceiving && _dualReceiver.NackMode() == kNack);
-
-  VCMEncodedFrame* frame =
-      _receiver.FrameForDecoding(maxWaitTimeMs,
-                                 nextRenderTimeMs,
-                                 supports_render_scheduling,
-                                 &_dualReceiver);
-
-  if (dualReceiverEnabledNotReceiving && _dualReceiver.State() == kReceiving) {
-    // Dual receiver is enabled (kNACK enabled), but was not receiving
-    // before the call to FrameForDecoding(). After the call the state
-    // changed to receiving, and therefore we must copy the primary decoder
-    // state to the dual decoder to make it possible for the dual decoder to
-    // start decoding retransmitted frames and recover.
-    CriticalSectionScoped cs(_receiveCritSect);
-    if (_dualDecoder != NULL) {
-      _codecDataBase.ReleaseDecoder(_dualDecoder);
-    }
-    _dualDecoder = _codecDataBase.CreateDecoderCopy();
-    if (_dualDecoder != NULL) {
-      _dualDecoder->RegisterDecodeCompleteCallback(&_dualDecodedFrameCallback);
-      SetReceiveState(kReceiveStateDecodingWithErrors);
-    } else {
-      _dualReceiver.Reset();
-      // presume we have an error, though we might not yet
-      SetReceiveState(kReceiveStateWaitingKey);
-    }
-  } else {
-    SetReceiveState(_receiver.ReceiveState());
-  }
+  VCMEncodedFrame* frame = _receiver.FrameForDecoding(
+      maxWaitTimeMs, nextRenderTimeMs, supports_render_scheduling);
 
   if (frame == NULL) {
     return VCM_FRAME_NOT_READY;
@@ -453,7 +393,7 @@ int32_t VideoReceiver::Decode(uint16_t maxWaitTimeMs) {
 
     if (pre_decode_image_callback_) {
       EncodedImage encoded_image(frame->EncodedImage());
-      pre_decode_image_callback_->Encoded(encoded_image);
+      pre_decode_image_callback_->Encoded(encoded_image, NULL, NULL);
     }
 
 #ifdef DEBUG_DECODER_BIT_STREAM
@@ -507,45 +447,6 @@ int32_t VideoReceiver::RequestKeyFrame() {
   return VCM_OK;
 }
 
-int32_t VideoReceiver::DecodeDualFrame(uint16_t maxWaitTimeMs) {
-  CriticalSectionScoped cs(_receiveCritSect);
-  if (_dualReceiver.State() != kReceiving ||
-      _dualReceiver.NackMode() != kNack) {
-    // The dual receiver is currently not receiving or
-    // dual decoder mode is disabled.
-    return VCM_OK;
-  }
-  int64_t dummyRenderTime;
-  int32_t decodeCount = 0;
-  // The dual decoder's state is copied from the main decoder, which may
-  // decode with errors. Make sure that the dual decoder does not introduce
-  // error.
-  _dualReceiver.SetDecodeErrorMode(kNoErrors);
-  VCMEncodedFrame* dualFrame =
-      _dualReceiver.FrameForDecoding(maxWaitTimeMs, dummyRenderTime);
-  if (dualFrame != NULL && _dualDecoder != NULL) {
-    // Decode dualFrame and try to catch up
-    int32_t ret =
-        _dualDecoder->Decode(*dualFrame, clock_->TimeInMilliseconds());
-    if (ret != WEBRTC_VIDEO_CODEC_OK) {
-      LOG(LS_ERROR) << "Failed to decode frame with dual decoder. Error code: "
-                    << ret;
-      _dualReceiver.ReleaseFrame(dualFrame);
-      return VCM_CODEC_ERROR;
-    }
-    if (_receiver.DualDecoderCaughtUp(dualFrame, _dualReceiver)) {
-      // Copy the complete decoder state of the dual decoder
-      // to the primary decoder.
-      _codecDataBase.CopyDecoder(*_dualDecoder);
-      _codecDataBase.ReleaseDecoder(_dualDecoder);
-      _dualDecoder = NULL;
-    }
-    decodeCount++;
-  }
-  _dualReceiver.ReleaseFrame(dualFrame);
-  return decodeCount;
-}
-
 // Must be called from inside the receive side critical section.
 int32_t VideoReceiver::Decode(const VCMEncodedFrame& frame) {
   TRACE_EVENT_ASYNC_STEP1("webrtc",
@@ -594,6 +495,7 @@ int32_t VideoReceiver::Decode(const VCMEncodedFrame& frame) {
       case kKeyOnLoss: {
         request_key_frame = true;
         ret = VCM_OK;
+        break;
       }
       default:
         break;
@@ -612,19 +514,14 @@ int32_t VideoReceiver::ResetDecoder() {
   bool reset_key_request = false;
   {
     CriticalSectionScoped cs(_receiveCritSect);
+    _receiver.Initialize();
+    _timing.Reset();
+    reset_key_request = true;
     if (_decoder != NULL) {
-      _receiver.Initialize();
-      _timing.Reset();
-      reset_key_request = true;
+      //      _receiver.Initialize();
+      //      _timing.Reset();
       _decoder->Reset();
     }
-    if (_dualReceiver.State() != kPassive) {
-      _dualReceiver.Initialize();
-    }
-    if (_dualDecoder != NULL) {
-      _codecDataBase.ReleaseDecoder(_dualDecoder);
-      _dualDecoder = NULL;
-    }
   }
   if (reset_key_request) {
     CriticalSectionScoped cs(process_crit_sect_.get());
@@ -665,7 +562,7 @@ VideoCodecType VideoReceiver::ReceiveCodec() const {
 
 // Incoming packet from network parsed and ready for decode, non blocking.
 int32_t VideoReceiver::IncomingPacket(const uint8_t* incomingPayload,
-                                      uint32_t payloadLength,
+                                      size_t payloadLength,
                                       const WebRtcRTPHeader& rtpInfo) {
   if (rtpInfo.frameType == kVideoFrameKey) {
     TRACE_EVENT1("webrtc",
@@ -680,20 +577,8 @@ int32_t VideoReceiver::IncomingPacket(const uint8_t* incomingPayload,
     payloadLength = 0;
   }
   const VCMPacket packet(incomingPayload, payloadLength, rtpInfo);
-  int32_t ret;
-  if (_dualReceiver.State() != kPassive) {
-    ret = _dualReceiver.InsertPacket(
-        packet, rtpInfo.type.Video.width, rtpInfo.type.Video.height);
-    if (ret == VCM_FLUSH_INDICATOR) {
-      RequestKeyFrame();
-      ResetDecoder();
-      SetReceiveState(kReceiveStateWaitingKey);
-    } else if (ret < 0) {
-      return ret;
-    }
-  }
-  ret = _receiver.InsertPacket(
-      packet, rtpInfo.type.Video.width, rtpInfo.type.Video.height);
+  int32_t ret = _receiver.InsertPacket(packet, rtpInfo.type.Video.width,
+                                       rtpInfo.type.Video.height);
   // TODO(holmer): Investigate if this somehow should use the key frame
   // request scheduling to throttle the requests.
   if (ret == VCM_FLUSH_INDICATOR) {
@@ -729,14 +614,10 @@ int32_t VideoReceiver::NackList(uint16_t* nackList, uint16_t* size) {
   VCMNackStatus nackStatus = kNackOk;
   uint16_t nack_list_length = 0;
   // Collect sequence numbers from the default receiver
-  // if in normal nack mode. Otherwise collect them from
-  // the dual receiver if the dual receiver is receiving.
+  // if in normal nack mode.
   if (_receiver.NackMode() != kNoNack) {
     nackStatus = _receiver.NackList(nackList, *size, &nack_list_length);
   }
-  if (nack_list_length == 0 && _dualReceiver.State() != kPassive) {
-    nackStatus = _dualReceiver.NackList(nackList, *size, &nack_list_length);
-  }
   *size = nack_list_length;
   if (nackStatus == kNackKeyFrameRequest) {
       SetReceiveState(kReceiveStateWaitingKey);
@@ -750,11 +631,6 @@ int32_t VideoReceiver::NackList(uint16_t* nackList, uint16_t* size) {
   return VCM_OK;
 }
 
-int32_t VideoReceiver::ReceivedFrameCount(VCMFrameCount* frameCount) const {
-  _receiver.ReceivedFrameCount(frameCount);
-  return VCM_OK;
-}
-
 uint32_t VideoReceiver::DiscardedPackets() const {
   return _receiver.DiscardedPackets();
 }
@@ -766,7 +642,6 @@ int VideoReceiver::SetReceiverRobustnessMode(
   switch (robustnessMode) {
     case VideoCodingModule::kNone:
       _receiver.SetNackMode(kNoNack, -1, -1);
-      _dualReceiver.SetNackMode(kNoNack, -1, -1);
       if (decode_error_mode == kNoErrors) {
         _keyRequestMode = kKeyOnLoss;
       } else {
@@ -776,7 +651,6 @@ int VideoReceiver::SetReceiverRobustnessMode(
     case VideoCodingModule::kHardNack:
       // Always wait for retransmissions (except when decoding with errors).
       _receiver.SetNackMode(kNack, -1, -1);
-      _dualReceiver.SetNackMode(kNoNack, -1, -1);
       _keyRequestMode = kKeyOnError;  // TODO(hlundin): On long NACK list?
       break;
     case VideoCodingModule::kSoftNack:
@@ -787,21 +661,9 @@ int VideoReceiver::SetReceiverRobustnessMode(
       // Enable hybrid NACK/FEC. Always wait for retransmissions and don't add
       // extra delay when RTT is above kLowRttNackMs.
       _receiver.SetNackMode(kNack, media_optimization::kLowRttNackMs, -1);
-      _dualReceiver.SetNackMode(kNoNack, -1, -1);
       _keyRequestMode = kKeyOnError;
       break;
 #endif
-    case VideoCodingModule::kDualDecoder:
-      if (decode_error_mode == kNoErrors) {
-        return VCM_PARAMETER_ERROR;
-      }
-      // Enable NACK but don't wait for retransmissions and don't add any extra
-      // delay.
-      _receiver.SetNackMode(kNack, 0, 0);
-      // Enable NACK, compensate with extra delay and wait for retransmissions.
-      _dualReceiver.SetNackMode(kNack, -1, -1);
-      _keyRequestMode = kKeyOnError;
-      break;
     case VideoCodingModule::kReferenceSelection:
 #if 1
       assert(false);  // TODO(hlundin): Not completed.
@@ -811,13 +673,10 @@ int VideoReceiver::SetReceiverRobustnessMode(
         return VCM_PARAMETER_ERROR;
       }
       _receiver.SetNackMode(kNoNack, -1, -1);
-      _dualReceiver.SetNackMode(kNoNack, -1, -1);
       break;
 #endif
   }
   _receiver.SetDecodeErrorMode(decode_error_mode);
-  // The dual decoder should never decode with errors.
-  _dualReceiver.SetDecodeErrorMode(kNoErrors);
   return VCM_OK;
 }
 
@@ -835,8 +694,6 @@ void VideoReceiver::SetNackSettings(size_t max_nack_list_size,
   }
   _receiver.SetNackSettings(
       max_nack_list_size, max_packet_age_to_nack, max_incomplete_time_ms);
-  _dualReceiver.SetNackSettings(
-      max_nack_list_size, max_packet_age_to_nack, max_incomplete_time_ms);
 }
 
 int VideoReceiver::SetMinReceiverDelay(int desired_delay_ms) {
