@@ -38,7 +38,7 @@ class NetworkPredictor {
   int64_t last_loss_rate_update_time_ms_;
 
   // An exponential filter is used to predict packet loss rate.
-  scoped_ptr<rtc::ExpFilter> loss_rate_filter_;
+  rtc::scoped_ptr<rtc::ExpFilter> loss_rate_filter_;
 };
 
 }  // namespace voe
