@@ -22,11 +22,9 @@
 #include "nsIFileURL.h"
 
 #include "mozilla/Preferences.h"
-#include "mozilla/net/RemoteOpenFileChild.h"
+#include "mozilla/Telemetry.h"
 #include "nsITabChild.h"
 #include "private/pprio.h"
-#include "nsINetworkInterceptController.h"
-#include "InterceptedJARChannel.h"
 #include "nsInputStreamPump.h"
 
 using namespace mozilla;
@@ -50,7 +48,7 @@ static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 //
 // set NSPR_LOG_MODULES=nsJarProtocol:5
 //
-static PRLogModuleInfo *gJarProtocolLog = nullptr;
+static LazyLogModule gJarProtocolLog("nsJarProtocol");
 
 #define LOG(args)     MOZ_LOG(gJarProtocolLog, mozilla::LogLevel::Debug, args)
 #define LOG_ENABLED() MOZ_LOG_TEST(gJarProtocolLog, mozilla::LogLevel::Debug)
@@ -195,19 +193,14 @@ nsJARInputThunk::IsNonBlocking(bool *nonBlocking)
 
 nsJARChannel::nsJARChannel()
     : mOpened(false)
-    , mAppURI(nullptr)
+    , mContentDisposition(0)
     , mContentLength(-1)
     , mLoadFlags(LOAD_NORMAL)
     , mStatus(NS_OK)
     , mIsPending(false)
     , mIsUnsafe(true)
-    , mOpeningRemote(false)
-    , mSynthesizedStreamLength(0)
     , mBlockRemoteFiles(false)
 {
-    if (!gJarProtocolLog)
-        gJarProtocolLog = PR_NewLogModule("nsJarProtocol");
-
     mBlockRemoteFiles = Preferences::GetBool("network.jar.block-remote-files", false);
 
     // hold an owning reference to the jar handler
@@ -216,7 +209,7 @@ nsJARChannel::nsJARChannel()
 
 nsJARChannel::~nsJARChannel()
 {
-    NS_ReleaseOnMainThread(mLoadInfo);
+    NS_ReleaseOnMainThread(mLoadInfo.forget());
 
     // release owning reference to the jar handler
     nsJARProtocolHandler *handler = gJarHandler;
@@ -229,7 +222,6 @@ NS_IMPL_ISUPPORTS_INHERITED(nsJARChannel,
                             nsIChannel,
                             nsIStreamListener,
                             nsIRequestObserver,
-                            nsIRemoteOpenFileListener,
                             nsIThreadRetargetableRequest,
                             nsIThreadRetargetableStreamListener,
                             nsIJARChannel)
@@ -339,7 +331,6 @@ nsJARChannel::LookupFile(bool aAllowAsync)
         return NS_OK;
 
     nsresult rv;
-    nsCOMPtr<nsIURI> uri;
 
     rv = mJarURI->GetJARFile(getter_AddRefs(mJarBaseURI));
     if (NS_FAILED(rv))
@@ -361,65 +352,7 @@ nsJARChannel::LookupFile(bool aAllowAsync)
         if (fileURL)
             fileURL->GetFile(getter_AddRefs(mJarFile));
     }
-    // if we're in child process and have special "remoteopenfile:://" scheme,
-    // create special nsIFile that gets file handle from parent when opened.
-    if (!mJarFile && !gJarHandler->IsMainProcess()) {
-        nsAutoCString scheme;
-        rv = mJarBaseURI->GetScheme(scheme);
-        if (NS_SUCCEEDED(rv) && scheme.EqualsLiteral("remoteopenfile")) {
-            RefPtr<RemoteOpenFileChild> remoteFile = new RemoteOpenFileChild();
-            rv = remoteFile->Init(mJarBaseURI, mAppURI);
-            NS_ENSURE_SUCCESS(rv, rv);
-            mJarFile = remoteFile;
 
-            nsIZipReaderCache *jarCache = gJarHandler->JarCache();
-            if (jarCache) {
-                bool cached = false;
-                rv = jarCache->IsCached(mJarFile, &cached);
-                if (NS_SUCCEEDED(rv) && cached) {
-                    // zipcache already has file mmapped: don't open on parent,
-                    // just return and proceed to cache hit in CreateJarInput().
-                    // When the file descriptor is needed, get it from JAR cache
-                    // if available, otherwise do the remote open to get a new
-                    // one.
-                    #if defined(XP_WIN) || defined(MOZ_WIDGET_COCOA)
-                    // Windows/OSX desktop builds skip remoting, we don't need
-                    // file descriptor here.
-                    return NS_OK;
-                    #else
-                    PRFileDesc *fd = nullptr;
-                    jarCache->GetFd(mJarFile, &fd);
-                    if (fd) {
-                        return SetRemoteNSPRFileDesc(fd);
-                    }
-                    #endif
-                }
-            }
-
-            if (!aAllowAsync) {
-                mJarFile = nullptr;
-                return NS_OK;
-            }
-
-            mOpeningRemote = true;
-
-            if (gJarHandler->RemoteOpenFileInProgress(remoteFile, this)) {
-                // JarHandler will trigger OnRemoteFileOpen() after the first
-                // request for this file completes and we'll get a JAR cache
-                // hit.
-                return NS_OK;
-            }
-
-            // Open file on parent: OnRemoteFileOpenComplete called when done
-            nsCOMPtr<nsITabChild> tabChild;
-            NS_QueryNotificationCallbacks(this, tabChild);
-            nsCOMPtr<nsILoadContext> loadContext;
-            NS_QueryNotificationCallbacks(this, loadContext);
-            rv = remoteFile->AsyncRemoteFileOpen(PR_RDONLY, this, tabChild,
-                                                 loadContext);
-            NS_ENSURE_SUCCESS(rv, rv);
-        }
-    }
     // try to handle a nested jar
     if (!mJarFile) {
         nsCOMPtr<nsIJARURI> jarURI = do_QueryInterface(mJarBaseURI);
@@ -480,24 +413,6 @@ nsJARChannel::FireOnProgress(uint64_t aProgress)
   mProgressSink->OnProgress(this, nullptr, aProgress, mContentLength);
 }
 
-nsresult
-nsJARChannel::SetRemoteNSPRFileDesc(PRFileDesc *fd)
-{
-    PROsfd osfd = dup(PR_FileDesc2NativeHandle(fd));
-    if (osfd == -1) {
-        return NS_ERROR_FAILURE;
-    }
-
-    RemoteOpenFileChild* remoteFile =
-        static_cast<RemoteOpenFileChild*>(mJarFile.get());
-    nsresult rv = remoteFile->SetNSPRFileDesc(PR_ImportFile(osfd));
-    if (NS_FAILED(rv)) {
-        close(osfd);
-    }
-
-    return rv;
-}
-
 //-----------------------------------------------------------------------------
 // nsIRequest
 //-----------------------------------------------------------------------------
@@ -520,8 +435,6 @@ nsJARChannel::GetStatus(nsresult *status)
 {
     if (mPump && NS_SUCCEEDED(mStatus))
         mPump->GetStatus(status);
-    else if (mSynthesizedResponsePump && NS_SUCCEEDED(mStatus))
-        mSynthesizedResponsePump->GetStatus(status);
     else
         *status = mStatus;
     return NS_OK;
@@ -533,8 +446,6 @@ nsJARChannel::Cancel(nsresult status)
     mStatus = status;
     if (mPump)
         return mPump->Cancel(status);
-    if (mSynthesizedResponsePump)
-        return mSynthesizedResponsePump->Cancel(status);
 
     NS_ASSERTION(!mIsPending, "need to implement cancel when downloading");
     return NS_OK;
@@ -545,8 +456,6 @@ nsJARChannel::Suspend()
 {
     if (mPump)
         return mPump->Suspend();
-    if (mSynthesizedResponsePump)
-        return mSynthesizedResponsePump->Suspend();
 
     NS_ASSERTION(!mIsPending, "need to implement suspend when downloading");
     return NS_OK;
@@ -557,8 +466,6 @@ nsJARChannel::Resume()
 {
     if (mPump)
         return mPump->Resume();
-    if (mSynthesizedResponsePump)
-        return mSynthesizedResponsePump->Resume();
 
     NS_ASSERTION(!mIsPending, "need to implement resume when downloading");
     return NS_OK;
@@ -615,11 +522,7 @@ nsJARChannel::SetOriginalURI(nsIURI *aURI)
 NS_IMETHODIMP
 nsJARChannel::GetURI(nsIURI **aURI)
 {
-    if (mAppURI) {
-        NS_IF_ADDREF(*aURI = mAppURI);
-    } else {
-        NS_IF_ADDREF(*aURI = mJarURI);
-    }
+    NS_IF_ADDREF(*aURI = mJarURI);
 
     return NS_OK;
 }
@@ -668,25 +571,6 @@ nsJARChannel::SetNotificationCallbacks(nsIInterfaceRequestor *aCallbacks)
     return NS_OK;
 }
 
-nsresult
-nsJARChannel::OverrideSecurityInfo(nsISupports* aSecurityInfo)
-{
-  MOZ_RELEASE_ASSERT(!mSecurityInfo,
-                     "This can only be called when we don't have a security info object already");
-  MOZ_RELEASE_ASSERT(aSecurityInfo,
-                     "This can only be called with a valid security info object");
-  mSecurityInfo = aSecurityInfo;
-  return NS_OK;
-}
-
-void
-nsJARChannel::OverrideURI(nsIURI* aRedirectedURI)
-{
-  MOZ_RELEASE_ASSERT(mLoadFlags & LOAD_REPLACE,
-                     "This can only happen if the LOAD_REPLACE flag is set");
-  mAppURI = aRedirectedURI;
-}
-
 NS_IMETHODIMP
 nsJARChannel::GetSecurityInfo(nsISupports **aSecurityInfo)
 {
@@ -862,78 +746,6 @@ nsJARChannel::Open2(nsIInputStream** aStream)
     return Open(aStream);
 }
 
-bool
-nsJARChannel::BypassServiceWorker() const
-{
-  return mLoadFlags & LOAD_BYPASS_SERVICE_WORKER;
-}
-
-bool
-nsJARChannel::ShouldIntercept()
-{
-    LOG(("nsJARChannel::ShouldIntercept [this=%x]\n", this));
-    // We only intercept app:// requests
-    if (!mAppURI) {
-      return false;
-    }
-
-    nsCOMPtr<nsINetworkInterceptController> controller;
-    NS_QueryNotificationCallbacks(mCallbacks, mLoadGroup,
-                                  NS_GET_IID(nsINetworkInterceptController),
-                                  getter_AddRefs(controller));
-    bool shouldIntercept = false;
-    if (controller && !BypassServiceWorker() && mLoadInfo) {
-      nsresult rv = controller->ShouldPrepareForIntercept(mAppURI,
-                                                          nsContentUtils::IsNonSubresourceRequest(this),
-                                                          &shouldIntercept);
-      NS_ENSURE_SUCCESS(rv, false);
-    }
-
-    return shouldIntercept;
-}
-
-void nsJARChannel::ResetInterception()
-{
-    LOG(("nsJARChannel::ResetInterception [this=%x]\n", this));
-
-    // Continue with the original request.
-    nsresult rv = ContinueAsyncOpen();
-    NS_ENSURE_SUCCESS_VOID(rv);
-}
-
-void
-nsJARChannel::OverrideWithSynthesizedResponse(nsIInputStream* aSynthesizedInput,
-                                              const nsACString& aContentType)
-{
-    // In our current implementation, the FetchEvent handler will copy the
-    // response stream completely into the pipe backing the input stream so we
-    // can treat the available as the length of the stream.
-    uint64_t available;
-    nsresult rv = aSynthesizedInput->Available(&available);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      mSynthesizedStreamLength = -1;
-    } else {
-      mSynthesizedStreamLength = int64_t(available);
-    }
-
-    rv = nsInputStreamPump::Create(getter_AddRefs(mSynthesizedResponsePump),
-                                   aSynthesizedInput,
-                                   int64_t(-1), int64_t(-1), 0, 0, true);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      aSynthesizedInput->Close();
-      return;
-    }
-
-    SetContentType(aContentType);
-
-    mIsUnsafe = false;
-
-    FinishAsyncOpen();
-
-    rv = mSynthesizedResponsePump->AsyncRead(this, nullptr);
-    NS_ENSURE_SUCCESS_VOID(rv);
-}
-
 NS_IMETHODIMP
 nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
 {
@@ -960,48 +772,6 @@ nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
     mListenerContext = ctx;
     mIsPending = true;
 
-// Bug 1171651 -  Disable the interception of app:// URIs in service workers
-//                on release builds
-#ifndef RELEASE_BUILD
-    // Check if this channel should intercept the network request and prepare
-    // for a possible synthesized response instead.
-    if (ShouldIntercept()) {
-      nsCOMPtr<nsINetworkInterceptController> controller;
-      NS_QueryNotificationCallbacks(mCallbacks, mLoadGroup,
-                                    NS_GET_IID(nsINetworkInterceptController),
-                                    getter_AddRefs(controller));
-
-      RefPtr<InterceptedJARChannel> intercepted =
-        new InterceptedJARChannel(this, controller);
-      intercepted->NotifyController();
-
-      // We get the JAREntry so we can infer the content type later in case
-      // that it isn't provided along with the synthesized response.
-      nsresult rv = mJarURI->GetJAREntry(mJarEntry);
-      if (NS_FAILED(rv)) {
-          return rv;
-      }
-
-      return NS_OK;
-    }
-#endif
-
-    return ContinueAsyncOpen();
-}
-
-NS_IMETHODIMP
-nsJARChannel::AsyncOpen2(nsIStreamListener *aListener)
-{
-  nsCOMPtr<nsIStreamListener> listener = aListener;
-  nsresult rv = nsContentSecurityManager::doContentSecurityCheck(this, listener);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return AsyncOpen(listener, nullptr);
-}
-
-nsresult
-nsJARChannel::ContinueAsyncOpen()
-{
-    LOG(("nsJARChannel::ContinueAsyncOpen [this=%x]\n", this));
     nsresult rv = LookupFile(true);
     if (NS_FAILED(rv)) {
         mIsPending = false;
@@ -1021,6 +791,12 @@ nsJARChannel::ContinueAsyncOpen()
             return NS_ERROR_UNSAFE_CONTENT_TYPE;
         }
 
+        static bool reportedRemoteJAR = false;
+        if (!reportedRemoteJAR) {
+            reportedRemoteJAR = true;
+            Telemetry::Accumulate(Telemetry::REMOTE_JAR_PROTOCOL_USED, 1);
+        }
+
         // kick off an async download of the base URI...
         nsCOMPtr<nsIStreamListener> downloader = new MemoryDownloader(this);
         uint32_t loadFlags =
@@ -1043,9 +819,8 @@ nsJARChannel::ContinueAsyncOpen()
         else {
             rv = channel->AsyncOpen(downloader, nullptr);
         }
-    } else if (mOpeningRemote) {
-        // nothing to do: already asked parent to open file.
-    } else {
+    }
+    else {
         rv = OpenLocalFile();
     }
 
@@ -1056,19 +831,21 @@ nsJARChannel::ContinueAsyncOpen()
         return rv;
     }
 
-
-    FinishAsyncOpen();
-
-    return NS_OK;
-}
-
-void
-nsJARChannel::FinishAsyncOpen()
-{
     if (mLoadGroup)
         mLoadGroup->AddRequest(this, nullptr);
 
     mOpened = true;
+
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsJARChannel::AsyncOpen2(nsIStreamListener *aListener)
+{
+  nsCOMPtr<nsIStreamListener> listener = aListener;
+  nsresult rv = nsContentSecurityManager::doContentSecurityCheck(this, listener);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return AsyncOpen(listener, nullptr);
 }
 
 //-----------------------------------------------------------------------------
@@ -1081,20 +858,6 @@ nsJARChannel::GetIsUnsafe(bool *isUnsafe)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsJARChannel::SetAppURI(nsIURI *aURI) {
-    NS_ENSURE_ARG_POINTER(aURI);
-
-    nsAutoCString scheme;
-    aURI->GetScheme(scheme);
-    if (!scheme.EqualsLiteral("app")) {
-        return NS_ERROR_INVALID_ARG;
-    }
-
-    mAppURI = aURI;
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsJARChannel::GetJarFile(nsIFile **aFile)
 {
@@ -1183,9 +946,7 @@ nsJARChannel::OnDownloadComplete(MemoryDownloader* aDownloader,
         } else {
             nsCOMPtr<nsIJARChannel> innerJARChannel(do_QueryInterface(channel));
             if (innerJARChannel) {
-                bool unsafe;
-                innerJARChannel->GetIsUnsafe(&unsafe);
-                mIsUnsafe = unsafe;
+                mIsUnsafe = innerJARChannel->GetIsUnsafe();
             }
         }
 
@@ -1229,50 +990,6 @@ nsJARChannel::OnDownloadComplete(MemoryDownloader* aDownloader,
     }
 }
 
-//-----------------------------------------------------------------------------
-// nsIRemoteOpenFileListener
-//-----------------------------------------------------------------------------
-nsresult
-nsJARChannel::OnRemoteFileOpenComplete(nsresult aOpenStatus)
-{
-    nsresult rv = aOpenStatus;
-
-    // NS_ERROR_ALREADY_OPENED here means we'll hit JAR cache in
-    // OpenLocalFile().
-    if (NS_SUCCEEDED(rv) || rv == NS_ERROR_ALREADY_OPENED) {
-        #if defined(XP_WIN) || defined(MOZ_WIDGET_COCOA)
-        // Windows/OSX desktop builds skip remoting, we don't need file
-        // descriptor here.
-        #else
-        // Set file descriptor from Jar cache into remote Jar file, if it
-        // has not been set previously.
-        mozilla::AutoFDClose fd;
-        mJarFile->OpenNSPRFileDesc(PR_RDONLY, 0, &fd.rwget());
-        if (!fd) {
-            nsIZipReaderCache *jarCache = gJarHandler->JarCache();
-            if (!jarCache) {
-                rv = NS_ERROR_FAILURE;
-            }
-            PRFileDesc *jar_fd = nullptr;
-            jarCache->GetFd(mJarFile, &jar_fd);
-            // If we failed to get fd here, an error rv would be returned
-            // by SetRemoteNSPRFileDesc(), which would then stop the
-            // channel by NotifyError().
-            rv = SetRemoteNSPRFileDesc(jar_fd);
-        }
-        #endif
-        if (NS_SUCCEEDED(rv) || rv == NS_ERROR_ALREADY_OPENED) {
-            rv = OpenLocalFile();
-        }
-    }
-
-    if (NS_FAILED(rv)) {
-        NotifyError(rv);
-    }
-
-    return NS_OK;
-}
-
 //-----------------------------------------------------------------------------
 // nsIStreamListener
 //-----------------------------------------------------------------------------
@@ -1300,19 +1017,19 @@ nsJARChannel::OnStopRequest(nsIRequest *req, nsISupports *ctx, nsresult status)
 
     if (mListener) {
         mListener->OnStopRequest(this, mListenerContext, status);
-        mListener = 0;
-        mListenerContext = 0;
+        mListener = nullptr;
+        mListenerContext = nullptr;
     }
 
     if (mLoadGroup)
         mLoadGroup->RemoveRequest(this, nullptr, status);
 
-    mPump = 0;
+    mPump = nullptr;
     mIsPending = false;
 
     // Drop notification callbacks to prevent cycles.
-    mCallbacks = 0;
-    mProgressSink = 0;
+    mCallbacks = nullptr;
+    mProgressSink = nullptr;
 
     #if defined(XP_WIN) || defined(MOZ_WIDGET_COCOA)
     #else
@@ -1341,11 +1058,10 @@ nsJARChannel::OnDataAvailable(nsIRequest *req, nsISupports *ctx,
         if (NS_IsMainThread()) {
             FireOnProgress(offset + count);
         } else {
-            nsCOMPtr<nsIRunnable> runnable =
-              NS_NewRunnableMethodWithArg<uint64_t>(this,
-                                                    &nsJARChannel::FireOnProgress,
-                                                    offset + count);
-            NS_DispatchToMainThread(runnable);
+            NS_DispatchToMainThread(NewRunnableMethod
+                                    <uint64_t>(this,
+                                               &nsJARChannel::FireOnProgress,
+                                               offset + count));
         }
     }
 
