@@ -45,6 +45,15 @@ public:
     return !mOldDesc ? NS_ERROR_NULL_POINTER :
                        mOldDesc->OpenOutputStream(offset, _retval);
   }
+  NS_IMETHOD OpenAlternativeOutputStream(const nsACString & type, nsIOutputStream * *_retval) override
+  {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+  NS_IMETHOD OpenAlternativeInputStream(const nsACString & type, nsIInputStream * *_retval) override
+  {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+
   NS_IMETHOD GetPredictedDataSize(int64_t *aPredictedDataSize) override
   {
     return !mOldDesc ? NS_ERROR_NULL_POINTER :
@@ -96,6 +105,11 @@ public:
                        mOldDesc->SetMetaDataElement(key, value);
   }
 
+  NS_IMETHOD GetDiskStorageSizeInKB(uint32_t *aDiskStorageSize) override
+  {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
+
   // nsICacheEntryInfo
   NS_IMETHOD GetKey(nsACString & aKey) override
   {
@@ -130,6 +144,7 @@ public:
   NS_IMETHOD MetaDataReady() override { return NS_OK; }
   NS_IMETHOD Recreate(bool, nsICacheEntry**) override;
   NS_IMETHOD GetDataSize(int64_t *size) override;
+  NS_IMETHOD GetAltDataSize(int64_t *size) override;
   NS_IMETHOD OpenInputStream(int64_t offset, nsIInputStream * *_retval) override;
   NS_IMETHOD OpenOutputStream(int64_t offset, nsIOutputStream * *_retval) override;
   NS_IMETHOD MaybeMarkValid() override;
@@ -148,7 +163,7 @@ private:
 };
 
 
-class _OldCacheLoad : public nsRunnable
+class _OldCacheLoad : public Runnable
                     , public nsICacheListener
 {
 public:
@@ -246,7 +261,7 @@ private:
   bool mHit; // set to true when the device was found
 };
 
-class _OldGetDiskConsumption : public nsRunnable,
+class _OldGetDiskConsumption : public Runnable,
                                public nsICacheVisitor
 {
 public:
