@@ -5,13 +5,10 @@
 #include "nsWifiAccessPoint.h"
 #include "nsString.h"
 #include "nsMemory.h"
-#include "prlog.h"
-
-#if defined(PR_LOGGING)
-extern PRLogModuleInfo *gWifiMonitorLog;
-#endif
-#define LOG(args)     PR_LOG(gWifiMonitorLog, PR_LOG_DEBUG, args)
+#include "mozilla/Logging.h"
 
+extern mozilla::LazyLogModule gWifiMonitorLog;
+#define LOG(args)     MOZ_LOG(gWifiMonitorLog, mozilla::LogLevel::Debug, args)
 
 NS_IMPL_ISUPPORTS(nsWifiAccessPoint, nsIWifiAccessPoint)
 
