@@ -2,23 +2,17 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import icecat_ui_tests
-
-from marionette import BaseMarionetteArguments
+from marionette_harness import BaseMarionetteArguments
 
 
 class IceCatUIBaseArguments(object):
     name = 'IceCat UI Tests'
     args = []
 
-    def parse_args_handler(self, args):
-        # If no tests are specified fall back to all icecat ui tests
-        args.tests = args.tests or [icecat_ui_tests.manifest_all]
-
 
 class IceCatUIArguments(BaseMarionetteArguments):
 
     def __init__(self, **kwargs):
-        BaseMarionetteArguments.__init__(self, **kwargs)
+        super(IceCatUIArguments, self).__init__(**kwargs)
 
         self.register_argument_container(IceCatUIBaseArguments())
