@@ -4,15 +4,16 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from marionette.runtests import cli as mn_cli
+from marionette_harness.runtests import cli as mn_cli
 
 from icecat_ui_harness.arguments import UpdateArguments
 from icecat_ui_harness.runners import UpdateTestRunner
 
 
-def cli():
+def cli(args=None):
     mn_cli(runner_class=UpdateTestRunner,
            parser_class=UpdateArguments,
+           args=args,
            )
 
 
