@@ -8,21 +8,19 @@ import tempfile
 
 import mozfile
 import mozinfo
-from marionette import BaseMarionetteTestRunner
 
-import icecat_ui_tests
-from icecat_puppeteer.testcases import IceCatTestCase
+from marionette_harness import BaseMarionetteTestRunner, MarionetteTestCase
 
 
 class IceCatUITestRunner(BaseMarionetteTestRunner):
+
     def __init__(self, **kwargs):
-        BaseMarionetteTestRunner.__init__(self, **kwargs)
+        super(IceCatUITestRunner, self).__init__(**kwargs)
+
         # select the appropriate GeckoInstance
         self.app = 'fxdesktop'
-        if not self.server_root:
-            self.server_root = icecat_ui_tests.resources
 
-        self.test_handlers = [IceCatTestCase]
+        self.test_handlers = [MarionetteTestCase]
 
     def duplicate_application(self, application_folder):
         """Creates a copy of the specified binary."""
