@@ -9,10 +9,8 @@ from collections import defaultdict
 from itertools import chain
 import logging
 import os
-import shutil
 import sys
 import warnings
-import which
 
 from mozbuild.base import (
     MachCommandBase,
@@ -25,7 +23,6 @@ from mach.decorators import (
     CommandProvider,
     Command,
 )
-import mozpack.path as mozpath
 
 here = os.path.abspath(os.path.dirname(__file__))
 
@@ -63,6 +60,15 @@ test path(s):
 Please check spelling and make sure there are mochitests living there.
 '''.lstrip()
 
+ROBOCOP_TESTS_NOT_FOUND = '''
+The robocop command could not find any tests under the following
+test path(s):
+
+{}
+
+Please check spelling and make sure the named tests exist.
+'''.lstrip()
+
 NOW_RUNNING = '''
 ######
 ### Now running mochitest-{}.
@@ -75,14 +81,17 @@ ALL_FLAVORS = {
     'mochitest': {
         'suite': 'plain',
         'aliases': ('plain', 'mochitest'),
-        'enabled_apps': ('icecat', 'b2g', 'android', 'mulet'),
+        'enabled_apps': ('icecat', 'android'),
+        'extra_args': {
+            'flavor': 'plain',
+        }
     },
     'chrome': {
         'suite': 'chrome',
         'aliases': ('chrome', 'mochitest-chrome'),
-        'enabled_apps': ('icecat', 'mulet', 'b2g', 'android'),
+        'enabled_apps': ('icecat', 'android'),
         'extra_args': {
-            'chrome': True,
+            'flavor': 'chrome',
         }
     },
     'browser-chrome': {
@@ -90,7 +99,7 @@ ALL_FLAVORS = {
         'aliases': ('browser', 'browser-chrome', 'mochitest-browser-chrome', 'bc'),
         'enabled_apps': ('icecat',),
         'extra_args': {
-            'browserChrome': True,
+            'flavor': 'browser',
         }
     },
     'jetpack-package': {
@@ -98,7 +107,7 @@ ALL_FLAVORS = {
         'aliases': ('jetpack-package', 'mochitest-jetpack-package', 'jpp'),
         'enabled_apps': ('icecat',),
         'extra_args': {
-            'jetpackPackage': True,
+            'flavor': 'jetpack-package',
         }
     },
     'jetpack-addon': {
@@ -106,7 +115,7 @@ ALL_FLAVORS = {
         'aliases': ('jetpack-addon', 'mochitest-jetpack-addon', 'jpa'),
         'enabled_apps': ('icecat',),
         'extra_args': {
-            'jetpackAddon': True,
+            'flavor': 'jetpack-addon',
         }
     },
     'a11y': {
@@ -114,31 +123,17 @@ ALL_FLAVORS = {
         'aliases': ('a11y', 'mochitest-a11y', 'accessibility'),
         'enabled_apps': ('icecat',),
         'extra_args': {
-            'a11y': True,
-        }
-    },
-    'webapprt-chrome': {
-        'suite': 'webapprt-chrome',
-        'aliases': ('webapprt-chrome', 'mochitest-webapprt-chrome'),
-        'enabled_apps': ('icecat',),
-        'extra_args': {
-            'webapprtChrome': True,
-        }
-    },
-    'webapprt-content': {
-        'suite': 'webapprt-content',
-        'aliases': ('webapprt-content', 'mochitest-webapprt-content'),
-        'enabled_apps': ('icecat',),
-        'extra_args': {
-            'webapprtContent': True,
+            'flavor': 'a11y',
         }
     },
 }
 
-SUPPORTED_APPS = ['icecat', 'b2g', 'android', 'mulet']
+SUPPORTED_APPS = ['icecat', 'android']
 SUPPORTED_FLAVORS = list(chain.from_iterable([f['aliases'] for f in ALL_FLAVORS.values()]))
 CANONICAL_FLAVORS = sorted([f['aliases'][0] for f in ALL_FLAVORS.values()])
 
+parser = None
+
 
 class MochitestRunner(MozbuildObject):
 
@@ -148,44 +143,6 @@ class MochitestRunner(MozbuildObject):
     to hook up result parsing, etc.
     """
 
-    def get_webapp_runtime_path(self):
-        import mozinfo
-        app_name = 'webapprt-stub' + mozinfo.info.get('bin_suffix', '')
-        app_path = os.path.join(self.distdir, 'bin', app_name)
-        if sys.platform.startswith('darwin'):
-            # On Mac, we copy the stub from the dist dir to the test app bundle,
-            # since we have to run it from a bundle for its windows to appear.
-            # Ideally, the build system would do this for us, and we should find
-            # a way for it to do that.
-            mac_dir_name = os.path.join(
-                self.mochitest_dir,
-                'webapprtChrome',
-                'webapprt',
-                'test',
-                'chrome',
-                'TestApp.app',
-                'Contents',
-                'MacOS')
-            mac_app_name = 'webapprt' + mozinfo.info.get('bin_suffix', '')
-            mac_app_path = os.path.join(mac_dir_name, mac_app_name)
-            shutil.copy(app_path, mac_app_path)
-            return mac_app_path
-        return app_path
-
-    # On Mac, the app invoked by runtests.py is in a different app bundle
-    # (as determined by get_webapp_runtime_path above), but the XRE path should
-    # still point to the browser's app bundle, so we set it here explicitly.
-    def get_webapp_runtime_xre_path(self):
-        if sys.platform.startswith('darwin'):
-            xre_path = os.path.join(
-                self.distdir,
-                self.substs['MOZ_MACBUNDLE_NAME'],
-                'Contents',
-                'Resources')
-        else:
-            xre_path = os.path.join(self.distdir, 'bin')
-        return xre_path
-
     def __init__(self, *args, **kwargs):
         MozbuildObject.__init__(self, *args, **kwargs)
 
@@ -210,47 +167,11 @@ class MochitestRunner(MozbuildObject):
         tests = list(resolver.resolve_tests(paths=test_paths, cwd=cwd))
         return tests
 
-    def run_b2g_test(self, context, tests=None, suite='mochitest', **kwargs):
-        """Runs a b2g mochitest."""
-        if context.target_out:
-            host_webapps_dir = os.path.join(context.target_out, 'data', 'local', 'webapps')
-            if not os.path.isdir(os.path.join(
-                    host_webapps_dir, 'test-container.gaiamobile.org')):
-                print(ENG_BUILD_REQUIRED.format(host_webapps_dir))
-                sys.exit(1)
-
-        # TODO without os.chdir, chained imports fail below
-        os.chdir(self.mochitest_dir)
-
-        # The imp module can spew warnings if the modules below have
-        # already been imported, ignore them.
-        with warnings.catch_warnings():
-            warnings.simplefilter('ignore')
-
-            import imp
-            path = os.path.join(self.mochitest_dir, 'runtestsb2g.py')
-            with open(path, 'r') as fh:
-                imp.load_module('mochitest', fh, path,
-                                ('.py', 'r', imp.PY_SOURCE))
-
-            import mochitest
-
-        options = Namespace(**kwargs)
-
-        from manifestparser import TestManifest
-        if tests:
-            manifest = TestManifest()
-            manifest.tests.extend(tests)
-            options.manifestFile = manifest
-
-        return mochitest.run_test_harness(options)
-
     def run_desktop_test(self, context, tests=None, suite=None, **kwargs):
         """Runs a mochitest.
 
         suite is the type of mochitest to run. It can be one of ('plain',
-        'chrome', 'browser', 'a11y', 'jetpack-package', 'jetpack-addon',
-        'webapprt-chrome', 'webapprt-content').
+        'chrome', 'browser', 'a11y', 'jetpack-package', 'jetpack-addon').
         """
         # runtests.py is ambiguous, so we load the file/module manually.
         if 'mochitest' not in sys.modules:
@@ -274,18 +195,8 @@ class MochitestRunner(MozbuildObject):
 
         options = Namespace(**kwargs)
 
-        if suite == 'webapprt-content':
-            if not options.app or options.app == self.get_binary_path():
-                options.app = self.get_webapp_runtime_path()
-            options.xrePath = self.get_webapp_runtime_xre_path()
-        elif suite == 'webapprt-chrome':
-            options.browserArgs.append("-test-mode")
-            if not options.app or options.app == self.get_binary_path():
-                options.app = self.get_webapp_runtime_path()
-            options.xrePath = self.get_webapp_runtime_xre_path()
-
         from manifestparser import TestManifest
-        if tests:
+        if tests and not options.manifestFile:
             manifest = TestManifest()
             manifest.tests.extend(tests)
             options.manifestFile = manifest
@@ -299,7 +210,7 @@ class MochitestRunner(MozbuildObject):
 
         # We need this to enable colorization of output.
         self.log_manager.enable_unstructured()
-        result = mochitest.run_test_harness(options)
+        result = mochitest.run_test_harness(parser, options)
         self.log_manager.disable_unstructured()
         return result
 
@@ -318,12 +229,12 @@ class MochitestRunner(MozbuildObject):
         options = Namespace(**kwargs)
 
         from manifestparser import TestManifest
-        if tests:
+        if tests and not options.manifestFile:
             manifest = TestManifest()
             manifest.tests.extend(tests)
             options.manifestFile = manifest
 
-        return runtestsremote.run_test_harness(options)
+        return runtestsremote.run_test_harness(parser, options)
 
     def run_robocop_test(self, context, tests, suite=None, **kwargs):
         host_ret = verify_host_bin()
@@ -340,15 +251,16 @@ class MochitestRunner(MozbuildObject):
         options = Namespace(**kwargs)
 
         from manifestparser import TestManifest
-        if tests:
+        if tests and not options.manifestFile:
             manifest = TestManifest()
             manifest.tests.extend(tests)
             options.manifestFile = manifest
 
-        return runrobocop.run_test_harness(options)
+        return runrobocop.run_test_harness(parser, options)
 
 # parser
 
+
 def setup_argument_parser():
     build_obj = MozbuildObject.from_environment(cwd=here)
 
@@ -377,7 +289,9 @@ def setup_argument_parser():
         from mozrunner.devices.android_device import verify_android_device
         verify_android_device(build_obj, install=True, xre=True)
 
-    return MochitestArgumentParser()
+    global parser
+    parser = MochitestArgumentParser()
+    return parser
 
 
 # condition filters
@@ -399,7 +313,8 @@ def verify_host_bin():
     # validate MOZ_HOST_BIN environment variables for Android tests
     MOZ_HOST_BIN = os.environ.get('MOZ_HOST_BIN')
     if not MOZ_HOST_BIN:
-        print('environment variable MOZ_HOST_BIN must be set to a directory containing host xpcshell')
+        print('environment variable MOZ_HOST_BIN must be set to a directory containing host '
+              'xpcshell')
         return 1
     elif not os.path.isdir(MOZ_HOST_BIN):
         print('$MOZ_HOST_BIN does not specify a directory')
@@ -441,31 +356,17 @@ class MachCommands(MachCommandBase):
         from mozbuild.controller.building import BuildDriver
         self._ensure_state_subdir_exists('.')
 
-        driver = self._spawn(BuildDriver)
-        driver.install_tests(remove=False)
-
         test_paths = kwargs['test_paths']
         kwargs['test_paths'] = []
 
-        if test_paths and buildapp == 'b2g':
-            # In B2G there is often a 'gecko' directory, though topsrcdir is actually
-            # elsewhere. This little hack makes test paths like 'gecko/dom' work, even if
-            # GECKO_PATH is set in the .userconfig
-            gecko_path = mozpath.abspath(mozpath.join(kwargs['b2gPath'], 'gecko'))
-            if gecko_path != self.topsrcdir:
-                new_paths = []
-                for tp in test_paths:
-                    if mozpath.abspath(tp).startswith(gecko_path):
-                        new_paths.append(mozpath.relpath(tp, gecko_path))
-                    else:
-                        new_paths.append(tp)
-                test_paths = new_paths
-
         mochitest = self._spawn(MochitestRunner)
         tests = []
         if resolve_tests:
             tests = mochitest.resolve_tests(test_paths, test_objects, cwd=self._mach_context.cwd)
 
+        driver = self._spawn(BuildDriver)
+        driver.install_tests(tests)
+
         subsuite = kwargs.get('subsuite')
         if subsuite == 'default':
             kwargs['subsuite'] = None
@@ -477,22 +378,32 @@ class MachCommands(MachCommandBase):
             if test['flavor'] not in ALL_FLAVORS:
                 continue
 
-            key = (test['flavor'], test['subsuite'])
+            key = (test['flavor'], test.get('subsuite', ''))
             if test['flavor'] not in flavors:
                 unsupported.add(key)
                 continue
 
             if subsuite == 'default':
                 # "--subsuite default" means only run tests that don't have a subsuite
-                if test['subsuite']:
+                if test.get('subsuite'):
                     unsupported.add(key)
                     continue
-            elif subsuite and test['subsuite'] != subsuite:
+            elif subsuite and test.get('subsuite', '') != subsuite:
                 unsupported.add(key)
                 continue
 
             suites[key].append(test)
 
+        if ('mochitest', 'media') in suites:
+            req = os.path.join('testing', 'tools', 'websocketprocessbridge',
+                               'websocketprocessbridge_requirements.txt')
+            self.virtualenv_manager.activate()
+            self.virtualenv_manager.install_pip_requirements(req, require_hashes=False)
+
+            # sys.executable is used to start the websocketprocessbridge, though for some
+            # reason it doesn't get set when calling `activate_this.py` in the virtualenv.
+            sys.executable = self.virtualenv_manager.python_path
+
         # This is a hack to introduce an option in mach to not send
         # filtered tests to the mochitest harness. Mochitest harness will read
         # the master manifest in that case.
@@ -525,9 +436,9 @@ class MachCommands(MachCommandBase):
                 buildapp, '\n'.join(sorted(msg))))
             return 1
 
-        if buildapp in ('b2g',):
-            run_mochitest = mochitest.run_b2g_test
-        elif buildapp == 'android':
+        if buildapp == 'android':
+            from mozrunner.devices.android_device import grant_runtime_permissions
+            grant_runtime_permissions(self)
             run_mochitest = mochitest.run_android_test
         else:
             run_mochitest = mochitest.run_desktop_test
@@ -565,8 +476,8 @@ class RobocopCommands(MachCommandBase):
              description='Run a Robocop test.',
              parser=setup_argument_parser)
     @CommandArgument('--serve', default=False, action='store_true',
-        help='Run no tests but start the mochi.test web server and launch '
-             'IceCatMobile with a test profile.')
+                     help='Run no tests but start the mochi.test web server '
+                     'and launch IceCatMobile with a test profile.')
     def run_robocop(self, serve=False, **kwargs):
         if serve:
             kwargs['autorun'] = False
@@ -583,16 +494,23 @@ class RobocopCommands(MachCommandBase):
         from mozbuild.controller.building import BuildDriver
         self._ensure_state_subdir_exists('.')
 
-        driver = self._spawn(BuildDriver)
-        driver.install_tests(remove=False)
-
         test_paths = kwargs['test_paths']
         kwargs['test_paths'] = []
 
         from mozbuild.testing import TestResolver
         resolver = self._spawn(TestResolver)
         tests = list(resolver.resolve_tests(paths=test_paths, cwd=self._mach_context.cwd,
-            flavor='instrumentation', subsuite='robocop'))
+                                            flavor='instrumentation', subsuite='robocop'))
+        driver = self._spawn(BuildDriver)
+        driver.install_tests(tests)
+
+        if len(tests) < 1:
+            print(ROBOCOP_TESTS_NOT_FOUND.format('\n'.join(
+                sorted(list(test_paths)))))
+            return 1
+
+        from mozrunner.devices.android_device import grant_runtime_permissions
+        grant_runtime_permissions(self)
 
         mochitest = self._spawn(MochitestRunner)
         return mochitest.run_robocop_test(self._mach_context, tests, 'robocop', **kwargs)
@@ -647,11 +565,3 @@ class DeprecatedCommands(MachCommandBase):
     @Command('jetpack-package', category='testing', conditions=[REMOVED])
     def jetpack_package(self):
         pass
-
-    @Command('webapprt-test-chrome', category='testing', conditions=[REMOVED])
-    def webapprt_chrome(self):
-        pass
-
-    @Command('webapprt-test-content', category='testing', conditions=[REMOVED])
-    def webapprt_content(self):
-        pass
