@@ -16,18 +16,17 @@ sys.path.insert(
 
 from automation import Automation
 from remoteautomation import RemoteAutomation, icecatmobileLogcatFilters
-from runtests import MochitestDesktop, MessageLogger
+from runtests import KeyValueParseError, MochitestDesktop, MessageLogger, parseKeyValue
 from mochitest_options import MochitestArgumentParser
 
 from manifestparser import TestManifest
 from manifestparser.filters import chunk_by_slice
-import devicemanager
+import mozdevice
 import mozinfo
 
 SCRIPT_DIR = os.path.abspath(os.path.realpath(os.path.dirname(__file__)))
 
 
-# TODO inherit from MochitestBase instead
 class RobocopTestRunner(MochitestDesktop):
     """
        A test harness for Robocop. Robocop tests are UI tests for IceCat for Android,
@@ -60,7 +59,7 @@ class RobocopTestRunner(MochitestDesktop):
         self.remoteLog = options.remoteLogFile
         self.auto.setRemoteLog(self.remoteLog)
         self.remoteScreenshots = "/mnt/sdcard/Robotium-Screenshots"
-        self.remoteNSPR = os.path.join(options.remoteTestRoot, "nspr")
+        self.remoteMozLog = os.path.join(options.remoteTestRoot, "mozlog")
         self.auto.setServerInfo(
             self.options.webServer, self.options.httpPort, self.options.sslPort)
         self.localLog = options.logFile
@@ -92,8 +91,8 @@ class RobocopTestRunner(MochitestDesktop):
         self.auto.deleteTombstones()
         self.dm.killProcess(self.options.app.split('/')[-1])
         self.dm.removeDir(self.remoteScreenshots)
-        self.dm.removeDir(self.remoteNSPR)
-        self.dm.mkDir(self.remoteNSPR)
+        self.dm.removeDir(self.remoteMozLog)
+        self.dm.mkDir(self.remoteMozLog)
         self.dm.mkDir(os.path.dirname(self.options.remoteLogFile))
         # Add Android version (SDK level) to mozinfo so that manifest entries
         # can be conditional on android_version.
@@ -128,9 +127,9 @@ class RobocopTestRunner(MochitestDesktop):
         self.dm.killProcess(self.options.app.split('/')[-1])
         blobberUploadDir = os.environ.get('MOZ_UPLOAD_DIR', None)
         if blobberUploadDir:
-            self.log.debug("Pulling any remote nspr logs and screenshots to %s." %
+            self.log.debug("Pulling any remote moz logs and screenshots to %s." %
                            blobberUploadDir)
-            self.dm.getDirectory(self.remoteNSPR, blobberUploadDir)
+            self.dm.getDirectory(self.remoteMozLog, blobberUploadDir)
             self.dm.getDirectory(self.remoteScreenshots, blobberUploadDir)
         MochitestDesktop.cleanup(self, self.options)
         if self.localProfile:
@@ -138,7 +137,7 @@ class RobocopTestRunner(MochitestDesktop):
         self.dm.removeDir(self.remoteProfile)
         self.dm.removeDir(self.remoteProfileCopy)
         self.dm.removeDir(self.remoteScreenshots)
-        self.dm.removeDir(self.remoteNSPR)
+        self.dm.removeDir(self.remoteMozLog)
         self.dm.removeFile(self.remoteConfigFile)
         if self.dm.fileExists(self.remoteLog):
             self.dm.removeFile(self.remoteLog)
@@ -226,18 +225,26 @@ class RobocopTestRunner(MochitestDesktop):
         self.options.extraPrefs.append('browser.snippets.enabled=false')
         self.options.extraPrefs.append('browser.casting.enabled=true')
         self.options.extraPrefs.append('extensions.autoupdate.enabled=false')
+
+        # Override the telemetry init delay for integration testing.
+        self.options.extraPrefs.append('toolkit.telemetry.initDelay=1')
+
+        self.options.extensionsToExclude.extend([
+            'mochikit@mozilla.org',
+            'worker-test@mozilla.org.xpi',
+            'workerbootstrap-test@mozilla.org.xpi',
+            'indexedDB-test@mozilla.org.xpi',
+        ])
+
         manifest = MochitestDesktop.buildProfile(self, self.options)
         self.localProfile = self.options.profilePath
         self.log.debug("Profile created at %s" % self.localProfile)
         # some files are not needed for robocop; save time by not pushing
         shutil.rmtree(os.path.join(self.localProfile, 'webapps'))
-        desktop_extensions = ['mochikit@mozilla.org', 'worker-test@mozilla.org', 'workerbootstrap-test@mozilla.org']
-        for ext in desktop_extensions:
-            shutil.rmtree(os.path.join(self.localProfile, 'extensions', 'staged', ext))
         os.remove(os.path.join(self.localProfile, 'userChrome.css'))
         try:
             self.dm.pushDir(self.localProfile, self.remoteProfileCopy)
-        except devicemanager.DMError:
+        except mozdevice.DMError:
             self.log.error(
                 "Automation Error: Unable to copy profile to device.")
             raise
@@ -255,7 +262,7 @@ class RobocopTestRunner(MochitestDesktop):
                 self.dm.shellCheckOutput(
                     ['cp', '-r', self.remoteProfileCopy, self.remoteProfile],
                     root=True, timeout=60)
-            except devicemanager.DMError:
+            except mozdevice.DMError:
                 # For instance, cp is not available on some older versions of
                 # Android.
                 self.log.info(
@@ -308,7 +315,7 @@ class RobocopTestRunner(MochitestDesktop):
 
     def logTestSummary(self):
         """
-           Print a summary of all tests run to stdout, for treeherder parsing 
+           Print a summary of all tests run to stdout, for treeherder parsing
            (logging via self.log does not work here).
         """
         print("0 INFO TEST-START | Shutdown")
@@ -345,7 +352,7 @@ class RobocopTestRunner(MochitestDesktop):
                 else:
                     self.log.info("  %s: %s" % (category, devinfo[category]))
             self.log.info("Test root: %s" % self.dm.deviceRoot)
-        except devicemanager.DMError:
+        except mozdevice.DMError:
             self.log.warning("Error getting device information")
 
     def setupRobotiumConfig(self, browserEnv):
@@ -368,9 +375,9 @@ class RobocopTestRunner(MochitestDesktop):
             for key, value in browserEnv.items():
                 try:
                     value.index(',')
-                    self.log.error(
-                        "setupRobotiumConfig: browserEnv - Found a ',' in our value, unable to process value. key=%s,value=%s" %
-                        (key, value))
+                    self.log.error("setupRobotiumConfig: browserEnv - Found a ',' "
+                                   "in our value, unable to process value. key=%s,value=%s" %
+                                   (key, value))
                     self.log.error("browserEnv=%s" % browserEnv)
                 except ValueError:
                     envstr += "%s%s=%s" % (delim, key, value)
@@ -395,9 +402,20 @@ class RobocopTestRunner(MochitestDesktop):
             del browserEnv["MOZ_WIN_INHERIT_STD_HANDLES_PRE_VISTA"]
         if "XPCOM_MEM_BLOAT_LOG" in browserEnv:
             del browserEnv["XPCOM_MEM_BLOAT_LOG"]
-        browserEnv["NSPR_LOG_FILE"] = os.path.join(
-            self.remoteNSPR,
-            self.nsprLogName)
+        browserEnv["MOZ_LOG_FILE"] = os.path.join(
+            self.remoteMozLog,
+            self.mozLogName)
+
+        try:
+            browserEnv.update(
+                dict(
+                    parseKeyValue(
+                        self.options.environment,
+                        context='--setenv')))
+        except KeyValueParseError as e:
+            self.log.error(str(e))
+            return None
+
         return browserEnv
 
     def runSingleTest(self, test):
@@ -405,7 +423,7 @@ class RobocopTestRunner(MochitestDesktop):
            Run the specified test.
         """
         self.log.debug("Running test %s" % test['name'])
-        self.nsprLogName = "nspr-%s.log" % test['name']
+        self.mozLogName = "moz-%s.log" % test['name']
         browserEnv = self.buildBrowserEnv()
         self.setupRobotiumConfig(browserEnv)
         self.setupRemoteProfile()
@@ -426,21 +444,24 @@ class RobocopTestRunner(MochitestDesktop):
             # This does not launch a test at all. It launches an activity
             # that starts IceCatMobile and then waits indefinitely, since cat
             # never returns.
-            browserArgs = ["start",
-                           "-n", "org.mozilla.roboexample.test/org.mozilla.gecko.LaunchIceCatMobileWithConfigurationActivity",
-                           "&&", "cat"]
+            browserArgs = ["start", "-n",
+                           "org.mozilla.roboexample.test/org.mozilla."
+                           "gecko.LaunchIceCatMobileWithConfigurationActivity", "&&", "cat"]
             self.dm.default_timeout = sys.maxint  # Forever.
             self.log.info("")
             self.log.info("Serving mochi.test Robocop root at http://%s:%s/tests/robocop/" %
-                         (self.options.remoteWebServer, self.options.httpPort))
+                          (self.options.remoteWebServer, self.options.httpPort))
             self.log.info("")
         result = -1
         log_result = -1
         try:
             self.dm.recordLogcat()
+            timeout = self.options.timeout
+            if not timeout:
+                timeout = self.NO_OUTPUT_TIMEOUT
             result = self.auto.runApp(
                 None, browserEnv, "am", self.localProfile, browserArgs,
-                timeout=self.NO_OUTPUT_TIMEOUT, symbolsPath=self.options.symbolsPath)
+                timeout=timeout, symbolsPath=self.options.symbolsPath)
             self.log.debug("runApp completes with status %d" % result)
             if result != 0:
                 self.log.error("runApp() exited with code %s" % result)
@@ -491,7 +512,7 @@ class RobocopTestRunner(MochitestDesktop):
                 continue
             if 'disabled' in test:
                 self.log.info('TEST-INFO | skipping %s | %s' %
-                             (test['name'], test['disabled']))
+                              (test['name'], test['disabled']))
                 continue
             active_tests.append(test)
         self.log.suite_start([t['name'] for t in active_tests])
@@ -513,7 +534,9 @@ class RobocopTestRunner(MochitestDesktop):
         return worstTestResult
 
 
-def run_test_harness(options):
+def run_test_harness(parser, options):
+    parser.validate(options)
+
     if options is None:
         raise ValueError(
             "Invalid options specified, use --help for a list of valid options")
@@ -548,7 +571,7 @@ def run_test_harness(options):
     finally:
         try:
             robocop.cleanup()
-        except devicemanager.DMError:
+        except mozdevice.DMError:
             # ignore device error while cleaning up
             pass
         message_logger.finish()
@@ -558,7 +581,7 @@ def run_test_harness(options):
 def main(args=sys.argv[1:]):
     parser = MochitestArgumentParser(app='android')
     options = parser.parse_args(args)
-    return run_test_harness(options)
+    return run_test_harness(parser, options)
 
 if __name__ == "__main__":
     sys.exit(main())
