@@ -5,14 +5,16 @@
 import logging
 import re
 import os
-import shutil
 import tempfile
 import time
 import traceback
 
+from distutils import dir_util
+
 from devicemanager import DeviceManager, DMError
 from mozprocess import ProcessHandler
 import mozfile
+import version_codes
 
 
 class DeviceManagerADB(DeviceManager):
@@ -32,10 +34,12 @@ class DeviceManagerADB(DeviceManager):
     _pollingInterval = 0.01
     _packageName = None
     _tempDir = None
+    _adb_version = None
+    _sdk_version = None
     connected = False
 
     def __init__(self, host=None, port=5555, retryLimit=5, packageName='icecatmobile',
-                 adbPath='adb', deviceSerial=None, deviceRoot=None,
+                 adbPath=None, deviceSerial=None, deviceRoot=None,
                  logLevel=logging.ERROR, autoconnect=True, runAdbAsRoot=False,
                  serverHost=None, serverPort=None, **kwargs):
         DeviceManager.__init__(self, logLevel=logLevel,
@@ -48,7 +52,7 @@ class DeviceManagerADB(DeviceManager):
         self._serverPort = serverPort
 
         # the path to adb, or 'adb' to assume that it's on the PATH
-        self._adbPath = adbPath
+        self._adbPath = adbPath or 'adb'
 
         # The serial number of the device to use with adb, used in cases
         # where multiple devices are being managed by the same adb instance.
@@ -81,6 +85,15 @@ class DeviceManagerADB(DeviceManager):
             # verify that we can connect to the device. can't continue
             self._verifyDevice()
 
+            # Note SDK version
+            try:
+                proc = self._runCmd(["shell", "getprop", "ro.build.version.sdk"],
+                                    timeout=self.short_timeout)
+                self._sdk_version = int(proc.output[0])
+            except (OSError, ValueError):
+                self._sdk_version = 0
+            self._logger.info("Detected Android sdk %d" % self._sdk_version)
+
             # Some commands require root to work properly, even with ADB (e.g.
             # grabbing APKs out of /data). For these cases, we check whether
             # we're running as root. If that isn't true, check for the
@@ -131,7 +144,7 @@ class DeviceManagerADB(DeviceManager):
             cmdline = envstr + "; " + cmdline
 
         # all output should be in stdout
-        args=[self._adbPath]
+        args = [self._adbPath]
         if self._serverHost is not None:
             args.extend(['-H', self._serverHost])
         if self._serverPort is not None:
@@ -147,7 +160,8 @@ class DeviceManagerADB(DeviceManager):
         proc = ProcessHandler(args, processOutputLine=self._log, onTimeout=_timeout)
 
         if not timeout:
-            # We are asserting that all commands will complete in this time unless otherwise specified
+            # We are asserting that all commands will complete in this time unless
+            # otherwise specified
             timeout = self.default_timeout
 
         timeout = int(timeout)
@@ -164,7 +178,7 @@ class DeviceManagerADB(DeviceManager):
                     for line in output:
                         outputfile.write(line + '\n')
                     outputfile.seek(-2, 2)
-                    outputfile.truncate() # truncate off the return code
+                    outputfile.truncate()  # truncate off the return code
                     return int(return_code)
 
         return None
@@ -184,6 +198,18 @@ class DeviceManagerADB(DeviceManager):
         if not self._checkCmd(['forward', local, remote], timeout=self.short_timeout) == 0:
             raise DMError("Failed to forward socket connection.")
 
+    def remove_forward(self, local=None):
+        """
+        Turn off forwarding of socket connection.
+        """
+        cmd = ['forward']
+        if local is None:
+            cmd.extend(['--remove-all'])
+        else:
+            cmd.extend(['--remove', local])
+        if not self._checkCmd(cmd, timeout=self.short_timeout) == 0:
+            raise DMError("Failed to remove connection forwarding.")
+
     def remount(self):
         "Remounts the /system partition on the device read-write."
         return self._checkCmd(['remount'], timeout=self.short_timeout)
@@ -191,7 +217,7 @@ class DeviceManagerADB(DeviceManager):
     def devices(self):
         "Return a list of connected devices as (serial, status) tuples."
         proc = self._runCmd(['devices'])
-        proc.output.pop(0) # ignore first line of output
+        proc.output.pop(0)  # ignore first line of output
         devices = []
         for line in proc.output:
             result = re.match('(.*?)\t(.*)', line)
@@ -218,9 +244,10 @@ class DeviceManagerADB(DeviceManager):
             raise DMError("File not found: %s" % localname)
 
         proc = self._runCmd(["push", os.path.realpath(localname), destname],
-                retryLimit=retryLimit)
+                            retryLimit=retryLimit)
         if proc.returncode != 0:
-            raise DMError("Error pushing file %s -> %s; output: %s" % (localname, destname, proc.output))
+            raise DMError("Error pushing file %s -> %s; output: %s" %
+                          (localname, destname, proc.output))
 
     def mkDir(self, name):
         result = self._runCmd(["shell", "mkdir", name], timeout=self.short_timeout).output
@@ -233,14 +260,14 @@ class DeviceManagerADB(DeviceManager):
         # files; we either zip/unzip or re-copy the directory into a temporary
         # one to get around this limitation
         retryLimit = retryLimit or self.retryLimit
-        if not self.dirExists(remoteDir):
-            self.mkDirs(remoteDir+"/x")
         if self._useZip:
+            self.removeDir(remoteDir)
+            self.mkDirs(remoteDir + "/x")
             try:
                 localZip = tempfile.mktemp() + ".zip"
                 remoteZip = remoteDir + "/adbdmtmp.zip"
                 proc = ProcessHandler(["zip", "-r", localZip, '.'], cwd=localDir,
-                              processOutputLine=self._log)
+                                      processOutputLine=self._log)
                 proc.run()
                 proc.wait()
                 self.pushFile(localZip, remoteZip, retryLimit=retryLimit, createDir=False)
@@ -257,13 +284,42 @@ class DeviceManagerADB(DeviceManager):
                 self._useZip = False
                 self.pushDir(localDir, remoteDir, retryLimit=retryLimit, timeout=timeout)
         else:
-            tmpDir = tempfile.mkdtemp()
-            # copytree's target dir must not already exist, so create a subdir
-            tmpDirTarget = os.path.join(tmpDir, "tmp")
-            shutil.copytree(localDir, tmpDirTarget)
-            self._checkCmd(["push", tmpDirTarget, remoteDir],
-                           retryLimit=retryLimit, timeout=timeout)
-            mozfile.remove(tmpDir)
+            localDir = os.path.normpath(localDir)
+            remoteDir = os.path.normpath(remoteDir)
+            copyRequired = False
+            if self._adb_version >= '1.0.36' and \
+               os.path.isdir(localDir) and self.dirExists(remoteDir):
+                # See do_sync_push in
+                # https://android.googlesource.com/platform/system/core/+/master/adb/file_sync_client.cpp
+                # Work around change in behavior in adb 1.0.36 where if
+                # the remote destination directory exists, adb push will
+                # copy the source directory *into* the destination
+                # directory otherwise it will copy the source directory
+                # *onto* the destination directory.
+                #
+                # If the destination directory does exist, push to its
+                # parent directory.  If the source and destination leaf
+                # directory names are different, copy the source directory
+                # to a temporary directory with the same leaf name as the
+                # destination so that when we push to the parent, the
+                # source is copied onto the destination directory.
+                localName = os.path.basename(localDir)
+                remoteName = os.path.basename(remoteDir)
+                if localName != remoteName:
+                    copyRequired = True
+                    tempParent = tempfile.mkdtemp()
+                    newLocal = os.path.join(tempParent, remoteName)
+                    dir_util.copy_tree(localDir, newLocal)
+                    localDir = newLocal
+                remoteDir = '/'.join(remoteDir.rstrip('/').split('/')[:-1])
+            try:
+                self._checkCmd(["push", localDir, remoteDir],
+                               retryLimit=retryLimit, timeout=timeout)
+            except:
+                raise
+            finally:
+                if copyRequired:
+                    mozfile.remove(tempParent)
 
     def dirExists(self, remotePath):
         self._detectLsModifier()
@@ -323,10 +379,12 @@ class DeviceManagerADB(DeviceManager):
         return data
 
     def getProcessList(self):
+        ret = []
         p = self._runCmd(["shell", "ps"], timeout=self.short_timeout)
+        if not p or not p.output or len(p.output) < 1:
+            return ret
         # first line is the headers
         p.output.pop(0)
-        ret = []
         for proc in p.output:
             els = proc.split()
             # We need to figure out if this is "user pid name" or
@@ -345,13 +403,13 @@ class DeviceManagerADB(DeviceManager):
 
         DEPRECATED: Use shell() or launchApplication() for new code
         """
-        #strip out env vars
-        parts = appname.split('"');
+        # strip out env vars
+        parts = appname.split('"')
         if (len(parts) > 2):
             parts = parts[2:]
         return self.launchProcess(parts, failIfRunning)
 
-    def launchProcess(self, cmd, outputFile = "process.txt", cwd = '', env = '', failIfRunning=False):
+    def launchProcess(self, cmd, outputFile="process.txt", cwd='', env='', failIfRunning=False):
         """
         Launches a process, redirecting output to standard out
 
@@ -378,18 +436,18 @@ class DeviceManagerADB(DeviceManager):
         else:
             args = cmd[i:].strip()
         acmd.append("-n")
-        acmd.append(cmd[0:i] + "/.App")
+        acmd.append(cmd[0:i] + "/org.mozilla.gecko.BrowserApp")
         if args != "":
             acmd.append("--es")
             acmd.append("args")
             acmd.append(args)
-        if env != '' and env != None:
+        if env != '' and env is not None:
             envCnt = 0
             # env is expected to be a dict of environment variables
             for envkey, envval in env.iteritems():
                 acmd.append("--es")
                 acmd.append("env" + str(envCnt))
-                acmd.append(envkey + "=" + envval);
+                acmd.append(envkey + "=" + envval)
                 envCnt += 1
         if uri != "":
             acmd.append("-d")
@@ -401,15 +459,31 @@ class DeviceManagerADB(DeviceManager):
         return outputFile
 
     def killProcess(self, appname, sig=None):
+        shell_args = ["shell"]
+        if self._sdk_version >= version_codes.N:
+            # Bug 1334613 - force use of root
+            if self._haveRootShell is None and self._haveSu is None:
+                self._checkForRoot()
+            if not self._haveRootShell and not self._haveSu:
+                raise DMError(
+                    "killProcess '%s' requested to run as root but root "
+                    "is not available on this device. Root your device or "
+                    "refactor the test/harness to not require root." %
+                    appname)
+            if not self._haveRootShell:
+                shell_args.extend(["su", self._suModifier])
+
         procs = self.getProcessList()
         for (pid, name, user) in procs:
             if name == appname:
-                args = ["shell", "kill"]
+                args = list(shell_args)
+                args.append("kill")
                 if sig:
                     args.append("-%d" % sig)
                 args.append(str(pid))
                 p = self._runCmd(args, timeout=self.short_timeout)
-                if p.returncode != 0:
+                if p.returncode != 0 and len(p.output) > 0 and \
+                   'No such process' not in p.output[0]:
                     raise DMError("Error killing process "
                                   "'%s': %s" % (appname, p.output))
 
@@ -443,7 +517,37 @@ class DeviceManagerADB(DeviceManager):
         self._runPull(remoteFile, localFile)
 
     def getDirectory(self, remoteDir, localDir, checkDir=True):
+        localDir = os.path.normpath(localDir)
+        remoteDir = os.path.normpath(remoteDir)
+        copyRequired = False
+        originalLocal = localDir
+        if self._adb_version >= '1.0.36' and \
+           os.path.isdir(localDir) and self.dirExists(remoteDir):
+            # See do_sync_pull in
+            # https://android.googlesource.com/platform/system/core/+/master/adb/file_sync_client.cpp
+            # Work around change in behavior in adb 1.0.36 where if
+            # the local destination directory exists, adb pull will
+            # copy the source directory *into* the destination
+            # directory otherwise it will copy the source directory
+            # *onto* the destination directory.
+            #
+            # If the destination directory does exist, pull to its
+            # parent directory. If the source and destination leaf
+            # directory names are different, pull the source directory
+            # into a temporary directory and then copy the temporary
+            # directory onto the destination.
+            localName = os.path.basename(localDir)
+            remoteName = os.path.basename(remoteDir)
+            if localName != remoteName:
+                copyRequired = True
+                tempParent = tempfile.mkdtemp()
+                localDir = os.path.join(tempParent, remoteName)
+            else:
+                localDir = '/'.join(localDir.rstrip('/').split('/')[:-1])
         self._runCmd(["pull", remoteDir, localDir]).wait()
+        if copyRequired:
+            dir_util.copy_tree(localDir, originalLocal)
+            mozfile.remove(tempParent)
 
     def validateFile(self, remoteFile, localFile):
         md5Remote = self._getRemoteHash(remoteFile)
@@ -470,8 +574,9 @@ class DeviceManagerADB(DeviceManager):
         # we must determine the device root ourselves
         paths = [('/storage/sdcard0', 'tests'),
                  ('/storage/sdcard1', 'tests'),
-                 ('/sdcard', 'tests'),
+                 ('/storage/sdcard', 'tests'),
                  ('/mnt/sdcard', 'tests'),
+                 ('/sdcard', 'tests'),
                  ('/data/local', 'tests')]
         for (basePath, subPath) in paths:
             if self.dirExists(basePath):
@@ -483,7 +588,7 @@ class DeviceManagerADB(DeviceManager):
                     pass
 
         raise DMError("Unable to set up device root using paths: [%s]"
-                        % ", ".join(["'%s'" % os.path.join(b, s) for b, s in paths]))
+                      % ", ".join(["'%s'" % os.path.join(b, s) for b, s in paths]))
 
     def getTempDir(self):
         # Cache result to speed up operations depending
@@ -494,7 +599,7 @@ class DeviceManagerADB(DeviceManager):
 
         return self._tempDir
 
-    def reboot(self, wait = False, **kwargs):
+    def reboot(self, wait=False, **kwargs):
         self._checkCmd(["reboot"])
         if wait:
             self._checkCmd(["wait-for-device"])
@@ -509,7 +614,7 @@ class DeviceManagerADB(DeviceManager):
         timestr = str(self._runCmd(["shell", "date", "+%s"], timeout=self.short_timeout).output[0])
         if (not timestr or not timestr.isdigit()):
             raise DMError("Unable to get current time using date (got: '%s')" % timestr)
-        return int(timestr)*1000
+        return int(timestr) * 1000
 
     def getInfo(self, directive=None):
         directive = directive or "all"
@@ -517,7 +622,8 @@ class DeviceManagerADB(DeviceManager):
         if directive == "id" or directive == "all":
             ret["id"] = self._runCmd(["get-serialno"], timeout=self.short_timeout).output[0]
         if directive == "os" or directive == "all":
-            ret["os"] = self.shellCheckOutput(["getprop", "ro.build.display.id"], timeout=self.short_timeout)
+            ret["os"] = self.shellCheckOutput(
+                ["getprop", "ro.build.display.id"], timeout=self.short_timeout)
         if directive == "uptime" or directive == "all":
             uptime = self.shellCheckOutput(["uptime"], timeout=self.short_timeout)
             if not uptime:
@@ -539,7 +645,8 @@ class DeviceManagerADB(DeviceManager):
                 meminfo[key] = value.strip()
             ret["memtotal"] = meminfo["MemTotal"]
         if directive == "disk" or directive == "all":
-            data = self.shellCheckOutput(["df", "/data", "/system", "/sdcard"], timeout=self.short_timeout)
+            data = self.shellCheckOutput(
+                ["df", "/data", "/system", "/sdcard"], timeout=self.short_timeout)
             ret["disk"] = data.split('\n')
         self._logger.debug("getInfo: %s" % ret)
         return ret
@@ -579,10 +686,10 @@ class DeviceManagerADB(DeviceManager):
         retries = 0
         while retries < retryLimit:
             proc = ProcessHandler(finalArgs, storeOutput=True,
-                    processOutputLine=self._log, onTimeout=_timeout)
+                                  processOutputLine=self._log, onTimeout=_timeout)
             proc.run(timeout=timeout)
             proc.returncode = proc.wait()
-            if proc.returncode == None:
+            if proc.returncode is None:
                 proc.kill()
                 retries += 1
             else:
@@ -621,7 +728,7 @@ class DeviceManagerADB(DeviceManager):
             proc = ProcessHandler(finalArgs, processOutputLine=self._log, onTimeout=_timeout)
             proc.run(timeout=timeout)
             ret_code = proc.wait()
-            if ret_code == None:
+            if ret_code is None:
                 proc.kill()
                 retries += 1
             else:
@@ -631,16 +738,20 @@ class DeviceManagerADB(DeviceManager):
 
     def chmodDir(self, remoteDir, mask="777"):
         if (self.dirExists(remoteDir)):
-            files = self.listFiles(remoteDir.strip())
-            for f in files:
-                remoteEntry = remoteDir.strip() + "/" + f.strip()
-                if (self.dirExists(remoteEntry)):
-                    self.chmodDir(remoteEntry)
-                else:
-                    self._checkCmd(["shell", "chmod", mask, remoteEntry], timeout=self.short_timeout)
-                    self._logger.info("chmod %s" % remoteEntry)
-            self._checkCmd(["shell", "chmod", mask, remoteDir], timeout=self.short_timeout)
-            self._logger.debug("chmod %s" % remoteDir)
+            if '/sdcard' in remoteDir:
+                self._logger.debug("chmod %s -- skipped (/sdcard)" % remoteDir)
+            else:
+                files = self.listFiles(remoteDir.strip())
+                for f in files:
+                    remoteEntry = remoteDir.strip() + "/" + f.strip()
+                    if (self.dirExists(remoteEntry)):
+                        self.chmodDir(remoteEntry)
+                    else:
+                        self._checkCmd(["shell", "chmod", mask, remoteEntry],
+                                       timeout=self.short_timeout)
+                        self._logger.info("chmod %s" % remoteEntry)
+                self._checkCmd(["shell", "chmod", mask, remoteDir], timeout=self.short_timeout)
+                self._logger.debug("chmod %s" % remoteDir)
         else:
             self._checkCmd(["shell", "chmod", mask, remoteDir.strip()], timeout=self.short_timeout)
             self._logger.debug("chmod %s" % remoteDir.strip())
@@ -654,9 +765,14 @@ class DeviceManagerADB(DeviceManager):
                 raise DMError("invalid adb path, or adb not executable: %s" % self._adbPath)
 
         try:
-            self._checkCmd(["version"], timeout=self.short_timeout)
-        except os.error, err:
-            raise DMError("unable to execute ADB (%s): ensure Android SDK is installed and adb is in your $PATH" % err)
+            re_version = re.compile(r'Android Debug Bridge version (.*)')
+            proc = self._runCmd(["version"], timeout=self.short_timeout)
+            self._adb_version = re_version.match(proc.output[0]).group(1)
+            self._logger.info("Detected adb %s" % self._adb_version)
+        except os.error as err:
+            raise DMError(
+                "unable to execute ADB (%s): ensure Android SDK is installed "
+                "and adb is in your $PATH" % err)
 
     def _verifyDevice(self):
         # If there is a device serial number, see if adb is connected to it
@@ -667,7 +783,7 @@ class DeviceManagerADB(DeviceManager):
                 if m:
                     if self._deviceSerial == m.group(1):
                         deviceStatus = m.group(2)
-            if deviceStatus == None:
+            if deviceStatus is None:
                 raise DMError("device not found: %s" % self._deviceSerial)
             elif deviceStatus != "device":
                 raise DMError("bad status for device %s: %s" % (self._deviceSerial, deviceStatus))
@@ -706,7 +822,7 @@ class DeviceManagerADB(DeviceManager):
             retcode = None
             while (time.time() - start_time) <= 15 and retcode is None:
                 retcode = proc.poll()
-            if retcode is None: # still not terminated, kill
+            if retcode is None:  # still not terminated, kill
                 proc.kill()
 
             if proc.output and 'uid=0(root)' in proc.output[0]:
@@ -765,7 +881,7 @@ class DeviceManagerADB(DeviceManager):
             # Check if busybox -1A is required in order to get one
             # file per line.
             output = self._runCmd(["shell", "ls", "-1A", "/"],
-                         timeout=self.short_timeout).output
+                                  timeout=self.short_timeout).output
             output = ' '.join(output)
             if 'error: device not found' in output:
                 raise DMError(output)
@@ -775,4 +891,3 @@ class DeviceManagerADB(DeviceManager):
                 self._lsModifier = "-a"
             else:
                 self._lsModifier = "-1A"
-
