@@ -14,6 +14,7 @@ import unittest
 # Store file location at load time
 here = os.path.dirname(os.path.abspath(__file__))
 
+
 class TestMozInstall(unittest.TestCase):
 
     @classmethod
@@ -32,7 +33,8 @@ class TestMozInstall(unittest.TestCase):
     def tearDown(self):
         mozfile.rmtree(self.tempdir)
 
-    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe for mozinstall 1.12 and higher.")
+    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe "
+                     "for mozinstall 1.12 and higher.")
     def test_get_binary(self):
         """ Test mozinstall's get_binary method """
 
@@ -46,13 +48,13 @@ class TestMozInstall(unittest.TestCase):
                                                 os.path.join(self.tempdir, 'exe'))
             binary_exe = os.path.join(installdir_exe, 'core', 'icecat.exe')
             self.assertEqual(binary_exe, mozinstall.get_binary(installdir_exe,
-                             'icecat'))
+                                                               'icecat'))
 
             installdir_zip = mozinstall.install(self.zipfile,
                                                 os.path.join(self.tempdir, 'zip'))
             binary_zip = os.path.join(installdir_zip, 'icecat.exe')
             self.assertEqual(binary_zip, mozinstall.get_binary(installdir_zip,
-                             'icecat'))
+                                                               'icecat'))
 
         elif mozinfo.isMac:
             installdir = mozinstall.install(self.dmg, self.tempdir)
@@ -67,7 +69,8 @@ class TestMozInstall(unittest.TestCase):
                           tempdir_empty, 'icecat')
         mozfile.rmtree(tempdir_empty)
 
-    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe for mozinstall 1.12 and higher.")
+    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe "
+                     "for mozinstall 1.12 and higher.")
     def test_is_installer(self):
         """ Test we can identify a correct installer """
 
@@ -84,7 +87,7 @@ class TestMozInstall(unittest.TestCase):
             try:
                 # test stub browser file
                 # without pefile on the system this test will fail
-                import pefile
+                import pefile  # noqa
                 stub_exe = os.path.join(here, 'build_stub', 'icecat.exe')
                 self.assertFalse(mozinstall.is_installer(stub_exe))
             except ImportError:
@@ -108,7 +111,8 @@ class TestMozInstall(unittest.TestCase):
             self.assertRaises(mozinstall.InvalidSource, mozinstall.install,
                               self.bz2, 'icecat')
 
-    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe for mozinstall 1.12 and higher.")
+    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe "
+                     "for mozinstall 1.12 and higher.")
     def test_install(self):
         """ Test mozinstall's install capability """
 
@@ -132,7 +136,8 @@ class TestMozInstall(unittest.TestCase):
             self.assertEqual(os.path.join(os.path.realpath(self.tempdir),
                                           'IceCatStub.app'), installdir)
 
-    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe for mozinstall 1.12 and higher.")
+    @unittest.skipIf(mozinfo.isWin, "Bug 1157352 - We need a new icecat.exe "
+                     "for mozinstall 1.12 and higher.")
     def test_uninstall(self):
         """ Test mozinstall's uninstall capabilites """
         # Uninstall after installing
