@@ -7,27 +7,22 @@ config = {
                 ('http://archive.mozilla.org/pub', 'http://download.cdn.mozilla.net/pub'),
             ],
             'balrog_usernames': {
-                'b2g': 'b2gbld',
                 'icecat': 'ffxbld',
                 'thunderbird': 'tbirdbld',
                 'mobile': 'ffxbld',
                 'IceCatMobile': 'ffxbld',
-                'graphene': 'ffxbld',
-                'horizon': 'ffxbld',
             }
         },
-        {
-            'balrog_api_root': 'https://aus4-admin-dev.allizom.org/api',
-            'ignore_failures': True,
-            'balrog_usernames': {
-                'b2g': 'stage-b2gbld',
-                'icecat': 'stage-ffxbld',
-                'thunderbird': 'stage-tbirdbld',
-                'mobile': 'stage-ffxbld',
-                'IceCatMobile': 'stage-ffxbld',
-                'graphene': 'stage-ffxbld',
-                'horizon': 'stage-ffxbld',
-            }
-        }
+        # Bug 1261346 - temporarily disable staging balrog submissions
+        # {
+        #     'balrog_api_root': 'https://aus4-admin-dev.allizom.org/api',
+        #     'ignore_failures': True,
+        #     'balrog_usernames': {
+        #         'icecat': 'stage-ffxbld',
+        #         'thunderbird': 'stage-tbirdbld',
+        #         'mobile': 'stage-ffxbld',
+        #         'IceCatMobile': 'stage-ffxbld',
+        #     }
+        # }
     ]
 }
