@@ -4,13 +4,10 @@ config = {
             'balrog_api_root': 'https://aus4-admin-dev.allizom.org/api',
             'ignore_failures': False,
             'balrog_usernames': {
-                'b2g': 'stage-b2gbld',
                 'icecat': 'stage-ffxbld',
                 'thunderbird': 'stage-tbirdbld',
                 'mobile': 'stage-ffxbld',
                 'IceCatMobile': 'stage-ffxbld',
-                'graphene': 'stage-ffxbld',
-                'horizon': 'stage-ffxbld',
             }
         }
     ]
