@@ -12,7 +12,7 @@ config = {
         'upload-files',
         'sendchange',
         'check-test',
-        # 'generate-build-stats',
+        'generate-build-stats',
         'update',  # decided by query_is_nightly()
     ],
     'stage_platform': 'macosx64-debug',
@@ -40,6 +40,5 @@ config = {
         ##
     },
     'src_mozconfig': 'browser/config/mozconfigs/macosx64/debug',
-    'purge_minsize': 10,
     #######################
 }
