@@ -3,10 +3,11 @@ config = {
     "log_name": "updates_beta",
     "repo": {
         "repo": "https://hg.mozilla.org/build/tools",
-        "revision": "default",
+        "branch": "default",
         "dest": "tools",
         "vcs": "hg",
     },
+    "vcs_share_base": "/builds/hg-shared",
     "push_dest": "ssh://hg.mozilla.org/build/tools",
     "shipped-locales-url": "https://hg.mozilla.org/releases/mozilla-beta/raw-file/{revision}/browser/locales/shipped-locales",
     "ignore_no_changes": True,
@@ -24,9 +25,7 @@ config = {
             "requires_mirrors": True,
             "patcher_config": "mozBeta-branch-patcher2.cfg",
             "update_verify_channel": "beta-localtest",
-            "mar_channel_ids": [
-                "icecat-mozilla-beta", "icecat-mozilla-release",
-            ],
+            "mar_channel_ids": [],
             "channel_names": ["beta", "beta-localtest", "beta-cdntest"],
             "rules_to_update": ["icecat-beta-cdntest", "icecat-beta-localtest"],
             "publish_rules": ["icecat-beta"],
