@@ -3,7 +3,6 @@ config = {
     "branch": "mozilla-central",
     "en_us_binary_url": "http://ftp.mozilla.org/pub/mozilla.org/icecat/nightly/latest-mozilla-central/",
     "update_channel": "nightly",
-    "latest_mar_dir": '/pub/mozilla.org/icecat/nightly/latest-mozilla-central-l10n',
 
     # l10n
     "hg_l10n_base": "https://hg.mozilla.org/l10n-central",
@@ -16,19 +15,14 @@ config = {
     "repos": [{
         "vcs": "hg",
         "repo": "https://hg.mozilla.org/build/tools",
-        "revision": "default",
+        "branch": "default",
         "dest": "tools",
     }, {
-        "vcs": "hgtool",
+        "vcs": "hg",
         "repo": "https://hg.mozilla.org/mozilla-central",
-        "revision": "default",
+        "revision": "%(revision)s",
         "dest": "mozilla-central",
-    }, {
-        "vcs": "hgtool",
-        "repo": "https://hg.mozilla.org/build/compare-locales",
-        "revision": "RELEASE_AUTOMATION"
     }],
     # purge options
     'is_automation': True,
-    'purge_minsize': 12,
 }
