@@ -31,6 +31,7 @@ class Test(object):
     desktop = True
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
     lower_is_better = True
+    alert_threshold = 2.0
 
     @classmethod
     def name(cls):
@@ -103,7 +104,6 @@ class TsBase(Test):
         'xperf_user_providers',
         'xperf_stackwalk',
         'tpmozafterpaint',
-        'test_name_extension',
         'extensions',
         'filters',
         'setup',
@@ -150,7 +150,8 @@ class sessionrestore(TsBase):
     2. Launch IceCat.
     3. Measure the delta between firstPaint and sessionRestored.
     """
-    extensions = '${talos}/startup_test/sessionrestore/addon'
+    extensions = \
+        '${talos}/startup_test/sessionrestore/addon/sessionrestore-signed.xpi'
     cycles = 10
     timeout = 1000000
     sps_profile_startup = True
@@ -200,7 +201,7 @@ class tresize(TsBase):
     """
     This test does some resize thing.
     """
-    extensions = '${talos}/startup_test/tresize/addon'
+    extensions = '${talos}/startup_test/tresize/addon/tresize-signed.xpi'
     cycles = 20
     url = 'startup_test/tresize/addon/content/tresize-test.html'
     timeout = 150
@@ -211,17 +212,6 @@ class tresize(TsBase):
     unit = 'ms'
 
 
-# Media Test
-@register_test()
-class media_tests(TsBase):
-    """
-    Media Performance Tests
-    """
-    cycles = 5
-    desktop = True
-    url = 'http://localhost:16932/startup_test/media/html/media_tests.html'
-    timeout = 360
-
 # pageloader tests(tp5, etc)
 
 # The overall test number is determined by first calculating the median
@@ -245,8 +235,30 @@ class PageloaderTest(Test):
             'timeout', 'shutdown', 'responsiveness', 'profile_path',
             'xperf_providers', 'xperf_user_providers', 'xperf_stackwalk',
             'filters', 'preferences', 'extensions', 'setup', 'cleanup',
-            'test_name_extension', 'lower_is_better', 'unit']
+            'lower_is_better', 'alert_threshold', 'unit']
+
+
+@register_test()
+class tabpaint(PageloaderTest):
+    """
+    Tests the amount of time it takes to open new tabs, triggered from
+    both the parent process and the content process.
+    """
+    extensions = '${talos}/tests/tabpaint/tabpaint-signed.xpi'
+    tpmanifest = '${talos}/tests/tabpaint/tabpaint.manifest'
+    tppagecycles = 20
+    sps_profile_entries = 1000000
+    tploadnocache = True
     unit = 'ms'
+    preferences = {
+        # By default, Talos is configured to open links from
+        # content in new windows. We're overriding them so that
+        # they open in new tabs instead.
+        # See http://kb.mozillazine.org/Browser.link.open_newwindow
+        # and http://kb.mozillazine.org/Browser.link.open_newwindow.restriction
+        'browser.link.open_newwindow': 3,
+        'browser.link.open_newwindow.restriction': 2,
+    }
 
 
 @register_test()
@@ -254,7 +266,7 @@ class tps(PageloaderTest):
     """
     Tests the amount of time it takes to switch between tabs
     """
-    extensions = '${talos}/tests/tabswitch'
+    extensions = '${talos}/tests/tabswitch/tabswitch-signed.xpi'
     tpmanifest = '${talos}/tests/tabswitch/tps.manifest'
     tppagecycles = 5
     sps_profile_entries = 1000000
@@ -269,6 +281,7 @@ class tps(PageloaderTest):
         'addon.test.tabswitch.maxurls':
             45 if utils.PLATFORM_TYPE == 'win_' else -1,
     }
+    unit = 'ms'
 
 
 @register_test()
@@ -295,7 +308,7 @@ class tart(PageloaderTest):
       - all: average interval over all recorded intervals.
     """
     tpmanifest = '${talos}/tests/tart/tart.manifest'
-    extensions = '${talos}/tests/tart/addon'
+    extensions = '${talos}/tests/tart/addon/tart-signed.xpi'
     tpcycles = 1
     tppagecycles = 25
     tploadnocache = True
@@ -314,6 +327,7 @@ class tart(PageloaderTest):
                    'docshell.event_starvation_delay_hint': 1,
                    'dom.send_after_paint_to_content': False}
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -331,7 +345,7 @@ class cart(PageloaderTest):
     3-customize-enter-css - only the CSS animation part of entering customize
     """
     tpmanifest = '${talos}/tests/tart/cart.manifest'
-    extensions = '${talos}/tests/tart/addon'
+    extensions = '${talos}/tests/tart/addon/tart-signed.xpi'
     tpcycles = 1
     tppagecycles = 25
     tploadnocache = True
@@ -346,6 +360,7 @@ class cart(PageloaderTest):
                    'docshell.event_starvation_delay_hint': 1,
                    'dom.send_after_paint_to_content': False}
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -356,7 +371,7 @@ class damp(PageloaderTest):
     for each tool, across a very simple and very complicated page.
     """
     tpmanifest = '${talos}/tests/devtools/damp.manifest'
-    extensions = '${talos}/tests/devtools/addon'
+    extensions = '${talos}/tests/devtools/addon/devtools-signed.xpi'
     tpcycles = 1
     tppagecycles = 25
     tploadnocache = True
@@ -367,6 +382,7 @@ class damp(PageloaderTest):
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
     preferences = {'devtools.memory.enabled': True,
                    'addon.test.damp.webserver': '${webserver}'}
+    unit = 'ms'
 
 
 @register_test()
@@ -437,6 +453,7 @@ class tp5n(PageloaderTest):
     cleanup = '${talos}/xtalos/parse_xperf.py -c ${talos}/bcontroller.json'
     preferences = {'extensions.enabledScopes': '',
                    'talos.logfile': 'browser_output.txt'}
+    unit = 'ms'
 
 
 @register_test()
@@ -462,6 +479,7 @@ class tp5o(PageloaderTest):
     sps_profile_entries = 4000000
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
     timeout = 1800
+    unit = 'ms'
 
 
 @register_test()
@@ -480,7 +498,9 @@ class tp5o_scroll(PageloaderTest):
     tpmozafterpaint = False
     preferences = {'layout.frame_rate': 0,
                    'docshell.event_starvation_delay_hint': 1,
-                   'dom.send_after_paint_to_content': False}
+                   'dom.send_after_paint_to_content': False,
+                   'layout.css.scroll-behavior.spring-constant': "'10'",
+                   'toolkit.framesRecording.bufferSize': 10000}
     filters = filter.ignore_first.prepare(1) + filter.median.prepare()
     unit = '1/FPS'
 
@@ -520,6 +540,30 @@ class kraken(PageloaderTest):
     tpmozafterpaint = False
     preferences = {'dom.send_after_paint_to_content': False}
     filters = filter.mean.prepare()
+    unit = 'score'
+
+
+@register_test()
+class basic_compositor_video(PageloaderTest):
+    """
+    Video test
+    """
+    tpmanifest = '${talos}/tests/video/video.manifest'
+    tpcycles = 1
+    tppagecycles = 12
+    timeout = 10000
+    sps_profile_interval = 1
+    sps_profile_entries = 2000000
+    preferences = {'full-screen-api.allow-trusted-requests-only': False,
+                   'layers.acceleration.force-enabled': False,
+                   'layers.acceleration.disabled': True,
+                   'layout.frame_rate': 0,
+                   'docshell.event_starvation_delay_hint': 1,
+                   'full-screen-api.warning.timeout': 500,
+                   'media.ruin-av-sync.enabled': True}
+    filters = filter.ignore_first.prepare(1) + filter.median.prepare()
+    unit = 'ms/frame'
+    lower_is_better = True
 
 
 @register_test()
@@ -545,6 +589,7 @@ class dromaeo(PageloaderTest):
     """abstract base class for dramaeo tests"""
     filters = filter.dromaeo.prepare()
     lower_is_better = False
+    alert_threshold = 5.0
 
 
 @register_test()
@@ -559,6 +604,7 @@ class dromaeo_css(dromaeo):
     sps_profile_interval = 2
     sps_profile_entries = 10000000
     tpmanifest = '${talos}/tests/dromaeo/css.manifest'
+    unit = 'score'
 
 
 @register_test()
@@ -574,6 +620,7 @@ class dromaeo_dom(dromaeo):
     sps_profile_entries = 10000000
     tpmanifest = '${talos}/tests/dromaeo/dom.manifest'
     tpdisable_e10s = True
+    unit = 'score'
 
 
 @register_test()
@@ -592,6 +639,7 @@ class tsvgm(PageloaderTest):
                    'docshell.event_starvation_delay_hint': 1,
                    'dom.send_after_paint_to_content': False}
     filters = filter.ignore_first.prepare(2) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -610,6 +658,7 @@ class tsvgx(PageloaderTest):
                    'docshell.event_starvation_delay_hint': 1,
                    'dom.send_after_paint_to_content': False}
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -620,9 +669,11 @@ class tsvgr_opacity(PageloaderTest):
     tpmanifest = '${talos}/tests/svg_opacity/svg_opacity.manifest'
     tpcycles = 1
     tppagecycles = 25
+    tpmozafterpaint = True
     sps_profile_interval = 1
     sps_profile_entries = 10000000
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -639,8 +690,11 @@ class tscrollx(PageloaderTest):
     """ ASAP mode """
     preferences = {'layout.frame_rate': 0,
                    'docshell.event_starvation_delay_hint': 1,
-                   'dom.send_after_paint_to_content': False}
+                   'dom.send_after_paint_to_content': False,
+                   'layout.css.scroll-behavior.spring-constant': "'10'",
+                   'toolkit.framesRecording.bufferSize': 10000}
     filters = filter.ignore_first.prepare(5) + filter.median.prepare()
+    unit = 'ms'
 
 
 @register_test()
@@ -654,3 +708,5 @@ class a11yr(PageloaderTest):
     tppagecycles = 25
     tpmozafterpaint = True
     preferences = {'dom.send_after_paint_to_content': False}
+    unit = 'ms'
+    alert_threshold = 5.0
