@@ -3,6 +3,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import os
+import platform
 import subprocess
 import sys
 
@@ -11,24 +12,35 @@ from mozprocess import ProcessHandler
 from mozprofile import IceCatProfile, Preferences
 from mozprofile.permissions import ServerLocations
 from mozrunner import IceCatRunner
+from mozrunner.utils import get_stack_fixer_function
 from mozcrash import mozcrash
 
-from .base import get_free_port, Browser, ExecutorBrowser, require_arg, cmd_arg, browser_command
+from .base import (get_free_port,
+                   Browser,
+                   ExecutorBrowser,
+                   require_arg,
+                   cmd_arg,
+                   browser_command)
 from ..executors import executor_kwargs as base_executor_kwargs
-from ..executors.executormarionette import MarionetteTestharnessExecutor, MarionetteRefTestExecutor
+from ..executors.executormarionette import (MarionetteTestharnessExecutor,
+                                            MarionetteRefTestExecutor,
+                                            MarionetteWdspecExecutor)
 from ..environment import hostnames
 
+
 here = os.path.join(os.path.split(__file__)[0])
 
 __wptrunner__ = {"product": "icecat",
                  "check_args": "check_args",
                  "browser": "IceCatBrowser",
                  "executor": {"testharness": "MarionetteTestharnessExecutor",
-                              "reftest": "MarionetteRefTestExecutor"},
+                              "reftest": "MarionetteRefTestExecutor",
+                              "wdspec": "MarionetteWdspecExecutor"},
                  "browser_kwargs": "browser_kwargs",
                  "executor_kwargs": "executor_kwargs",
                  "env_options": "env_options",
-                 "run_info_extras": "run_info_extras"}
+                 "run_info_extras": "run_info_extras",
+                 "update_properties": "update_properties"}
 
 
 def check_args(**kwargs):
@@ -45,7 +57,8 @@ def browser_kwargs(**kwargs):
             "stackwalk_binary": kwargs["stackwalk_binary"],
             "certutil_binary": kwargs["certutil_binary"],
             "ca_certificate_path": kwargs["ssl_env"].ca_cert_path(),
-            "e10s": kwargs["gecko_e10s"]}
+            "e10s": kwargs["gecko_e10s"],
+            "stackfix_dir": kwargs["stackfix_dir"]}
 
 
 def executor_kwargs(test_type, server_config, cache_manager, run_info_data,
@@ -55,12 +68,14 @@ def executor_kwargs(test_type, server_config, cache_manager, run_info_data,
     executor_kwargs["close_after_done"] = test_type != "reftest"
     if kwargs["timeout_multiplier"] is None:
         if test_type == "reftest":
-            if run_info_data["debug"]:
+            if run_info_data["debug"] or run_info_data.get("asan"):
                 executor_kwargs["timeout_multiplier"] = 4
             else:
                 executor_kwargs["timeout_multiplier"] = 2
-        elif run_info_data["debug"]:
+        elif run_info_data["debug"] or run_info_data.get("asan"):
             executor_kwargs["timeout_multiplier"] = 3
+    if test_type == "wdspec":
+        executor_kwargs["webdriver_binary"] = kwargs.get("webdriver_binary")
     return executor_kwargs
 
 
@@ -71,15 +86,22 @@ def env_options():
             "certificate_domain": "web-platform.test",
             "supports_debugger": True}
 
+
 def run_info_extras(**kwargs):
     return {"e10s": kwargs["gecko_e10s"]}
 
+
+def update_properties():
+    return ["debug", "e10s", "os", "version", "processor", "bits"], {"debug", "e10s"}
+
+
 class IceCatBrowser(Browser):
     used_ports = set()
+    init_timeout = 60
 
     def __init__(self, logger, binary, prefs_root, debug_info=None,
                  symbols_path=None, stackwalk_binary=None, certutil_binary=None,
-                 ca_certificate_path=None, e10s=False):
+                 ca_certificate_path=None, e10s=False, stackfix_dir=None):
         Browser.__init__(self, logger)
         self.binary = binary
         self.prefs_root = prefs_root
@@ -92,6 +114,11 @@ class IceCatBrowser(Browser):
         self.ca_certificate_path = ca_certificate_path
         self.certutil_binary = certutil_binary
         self.e10s = e10s
+        if self.symbols_path and stackfix_dir:
+            self.stack_fixer = get_stack_fixer_function(stackfix_dir,
+                                                        self.symbols_path)
+        else:
+            self.stack_fixer = None
 
     def start(self):
         self.marionette_port = get_free_port(2828, exclude=self.used_ports)
@@ -109,10 +136,17 @@ class IceCatBrowser(Browser):
         self.profile.set_preferences({"marionette.defaultPrefs.enabled": True,
                                       "marionette.defaultPrefs.port": self.marionette_port,
                                       "dom.disable_open_during_load": False,
-                                      "network.dns.localDomains": ",".join(hostnames)})
+                                      "network.dns.localDomains": ",".join(hostnames),
+                                      "network.proxy.type": 0,
+                                      "places.history.enabled": False})
         if self.e10s:
             self.profile.set_preferences({"browser.tabs.remote.autostart": True})
 
+        # Bug 1262954: winxp + e10s, disable hwaccel
+        if (self.e10s and platform.system() in ("Windows", "Microsoft") and
+            '5.1' in platform.version()):
+            self.profile.set_preferences({"layers.acceleration.disabled": True})
+
         if self.ca_certificate_path is not None:
             self.setup_ssl()
 
@@ -161,8 +195,11 @@ class IceCatBrowser(Browser):
 
     def on_output(self, line):
         """Write a line of output from the icecat process to the log"""
+        data = line.decode("utf8", "replace")
+        if self.stack_fixer:
+            data = self.stack_fixer(data)
         self.logger.process_output(self.pid(),
-                                   line.decode("utf8", "replace"),
+                                   data,
                                    command=" ".join(self.runner.command))
 
     def is_alive(self):
