@@ -10,21 +10,19 @@ request.onload = function() {
   var idlArray = new IdlArray();
   var idls = request.responseText;
 
-  idlArray.add_untested_idls("interface WorkerGlobalScope {};");
-  idlArray.add_untested_idls("interface WorkerUtils {};");
-  idlArray.add_untested_idls("WorkerGlobalScope implements WorkerUtils;");
-  idlArray.add_untested_idls("interface Event { };");
-  idlArray.add_untested_idls("interface EventTarget { };");
+  idlArray.add_untested_idls("[Exposed=Worker] interface WorkerGlobalScope {};");
+  idlArray.add_untested_idls("[Exposed=(Window,Worker)] interface Event { };");
+  idlArray.add_untested_idls("[Exposed=(Window,Worker)] interface EventTarget { };");
+  idlArray.add_untested_idls("[NoInterfaceObject, Exposed=(Window,Worker)] interface WindowOrWorkerGlobalScope {};");
 
   // From Indexed DB:
-  idlArray.add_idls("WorkerUtils implements IDBEnvironment;");
+  idlArray.add_idls("WorkerGlobalScope implements WindowOrWorkerGlobalScope;");
   idlArray.add_idls(idls);
 
   idlArray.add_objects({
     IDBCursor: [],
     IDBCursorWithValue: [],
     IDBDatabase: [],
-    IDBEnvironment: [],
     IDBFactory: ["self.indexedDB"],
     IDBIndex: [],
     IDBKeyRange: ["IDBKeyRange.only(0)"],
