@@ -3,7 +3,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const PROMPT_URL = "chrome://global/content/commonDialog.xul";
-const { interfaces: Ci } = Components;
+var { interfaces: Ci } = Components;
 
 function test() {
   waitForExplicitFinish();
@@ -21,19 +21,19 @@ function test() {
 
         is(gBrowser.selectedTab, tab, "Should have selected the new tab");
 
-        domwindow.document.documentElement.cancelDialog()
+        domwindow.document.documentElement.cancelDialog();
       }, domwindow);
     },
 
     onCloseWindow: function() {
     }
-  }
+  };
 
   Services.wm.addListener(listener);
   registerCleanupFunction(() => {
     Services.wm.removeListener(listener);
     gBrowser.removeTab(tab);
-  })
+  });
 
   tab.linkedBrowser.addEventListener("load", () => {
     finish();
