@@ -32,11 +32,14 @@ TaggingService.prototype = {
    *
    * @param aTagName
    *        the name for the new tag.
+   * @param aSource
+   *        a change source constant from nsINavBookmarksService::SOURCE_*.
    * @returns the id of the new tag container.
    */
-  _createTag: function TS__createTag(aTagName) {
+  _createTag: function TS__createTag(aTagName, aSource) {
     var newFolderId = PlacesUtils.bookmarks.createFolder(
-      PlacesUtils.tagsFolderId, aTagName, PlacesUtils.bookmarks.DEFAULT_INDEX
+      PlacesUtils.tagsFolderId, aTagName, PlacesUtils.bookmarks.DEFAULT_INDEX,
+      /* aGuid */ null, aSource
     );
     // Add the folder to our local cache, so we can avoid doing this in the
     // observer that would have to check itemType.
@@ -66,7 +69,7 @@ TaggingService.prototype = {
     let stmt = db.createStatement(
       `SELECT id FROM moz_bookmarks
        WHERE parent = :tag_id
-       AND fk = (SELECT id FROM moz_places WHERE url = :page_url)`
+       AND fk = (SELECT id FROM moz_places WHERE url_hash = hash(:page_url) AND url = :page_url)`
     );
     stmt.params.tag_id = tagId;
     stmt.params.page_url = aURI.spec;
@@ -135,7 +138,7 @@ TaggingService.prototype = {
   },
 
   // nsITaggingService
-  tagURI: function TS_tagURI(aURI, aTags)
+  tagURI: function TS_tagURI(aURI, aTags, aSource)
   {
     if (!aURI || !aTags || !Array.isArray(aTags)) {
       throw Cr.NS_ERROR_INVALID_ARG;
@@ -148,14 +151,15 @@ TaggingService.prototype = {
       for (let tag of tags) {
         if (tag.id == -1) {
           // Tag does not exist yet, create it.
-          this._createTag(tag.name);
+          this._createTag(tag.name, aSource);
         }
 
         if (this._getItemIdForTaggedURI(aURI, tag.name) == -1) {
           // The provided URI is not yet tagged, add a tag for it.
           // Note that bookmarks under tag containers must have null titles.
           PlacesUtils.bookmarks.insertBookmark(
-            tag.id, aURI, PlacesUtils.bookmarks.DEFAULT_INDEX, null
+            tag.id, aURI, PlacesUtils.bookmarks.DEFAULT_INDEX,
+            /* aTitle */ null, /* aGuid */ null, aSource
           );
         }
 
@@ -164,7 +168,7 @@ TaggingService.prototype = {
         // user-typed value.
         if (PlacesUtils.bookmarks.getItemTitle(tag.id) != tag.name) {
           // this._tagFolders is updated by the bookmarks observer.
-          PlacesUtils.bookmarks.setItemTitle(tag.id, tag.name);
+          PlacesUtils.bookmarks.setItemTitle(tag.id, tag.name, aSource);
         }
       }
     };
@@ -182,8 +186,10 @@ TaggingService.prototype = {
    *
    * @param aTagId
    *        the itemId of the tag element under the tags root
+   * @param aSource
+   *        a change source constant from nsINavBookmarksService::SOURCE_*
    */
-  _removeTagIfEmpty: function TS__removeTagIfEmpty(aTagId) {
+  _removeTagIfEmpty: function TS__removeTagIfEmpty(aTagId, aSource) {
     let count = 0;
     let db = PlacesUtils.history.QueryInterface(Ci.nsPIPlacesDatabase)
                                 .DBConnection;
@@ -202,12 +208,12 @@ TaggingService.prototype = {
     }
 
     if (count == 0) {
-      PlacesUtils.bookmarks.removeItem(aTagId);
+      PlacesUtils.bookmarks.removeItem(aTagId, aSource);
     }
   },
 
   // nsITaggingService
-  untagURI: function TS_untagURI(aURI, aTags)
+  untagURI: function TS_untagURI(aURI, aTags, aSource)
   {
     if (!aURI || (aTags && !Array.isArray(aTags))) {
       throw Cr.NS_ERROR_INVALID_ARG;
@@ -235,7 +241,7 @@ TaggingService.prototype = {
           let itemId = this._getItemIdForTaggedURI(aURI, tag.name);
           if (itemId != -1) {
             // There is a tagged item.
-            PlacesUtils.bookmarks.removeItem(itemId);
+            PlacesUtils.bookmarks.removeItem(itemId, aSource);
           }
         }
       }
@@ -308,7 +314,7 @@ TaggingService.prototype = {
     return tags;
   },
 
-  __tagFolders: null, 
+  __tagFolders: null,
   get _tagFolders() {
     if (!this.__tagFolders) {
       this.__tagFolders = [];
@@ -380,7 +386,7 @@ TaggingService.prototype = {
     let stmt = db.createStatement(
       `SELECT id, parent
        FROM moz_bookmarks
-       WHERE fk = (SELECT id FROM moz_places WHERE url = :page_url)`
+       WHERE fk = (SELECT id FROM moz_places WHERE url_hash = hash(:page_url) AND url = :page_url)`
     );
     stmt.params.page_url = aURI.spec;
     try {
@@ -414,7 +420,8 @@ TaggingService.prototype = {
   },
 
   onItemRemoved: function TS_onItemRemoved(aItemId, aFolderId, aIndex,
-                                           aItemType, aURI) {
+                                           aItemType, aURI, aGuid, aParentGuid,
+                                           aSource) {
     // Item is a tag folder.
     if (aFolderId == PlacesUtils.tagsFolderId && this._tagFolders[aItemId]) {
       delete this._tagFolders[aItemId];
@@ -427,13 +434,13 @@ TaggingService.prototype = {
       let itemIds = this._getTaggedItemIdsIfUnbookmarkedURI(aURI);
       for (let i = 0; i < itemIds.length; i++) {
         try {
-          PlacesUtils.bookmarks.removeItem(itemIds[i]);
+          PlacesUtils.bookmarks.removeItem(itemIds[i], aSource);
         } catch (ex) {}
       }
     }
     // Item is a tag entry.  If this was the last entry for this tag, remove it.
     else if (aURI && this._tagFolders[aFolderId]) {
-      this._removeTagIfEmpty(aFolderId);
+      this._removeTagIfEmpty(aFolderId, aSource);
     }
   },
 
@@ -455,8 +462,7 @@ TaggingService.prototype = {
   onBeginUpdateBatch: function () {},
   onEndUpdateBatch: function () {},
 
-  //////////////////////////////////////////////////////////////////////////////
-  //// nsISupports
+  // nsISupports
 
   classID: Components.ID("{bbc23860-2553-479d-8b78-94d9038334f7}"),
 
@@ -482,7 +488,7 @@ function TagAutoCompleteResult(searchString, searchResult,
 }
 
 TagAutoCompleteResult.prototype = {
-  
+
   /**
    * The original search string
    */
@@ -522,10 +528,6 @@ TagAutoCompleteResult.prototype = {
     return this._results.length;
   },
 
-  get typeAheadResult() {
-    return false;
-  },
-
   /**
    * Get the value of the result at the given index
    */
@@ -595,7 +597,7 @@ function TagAutoCompleteSearch() {
 }
 
 TagAutoCompleteSearch.prototype = {
-  _stopped : false, 
+  _stopped : false,
 
   /*
    * Search for a given string and notify a listener (either synchronously
@@ -613,10 +615,10 @@ TagAutoCompleteSearch.prototype = {
     this._stopped = false;
 
     // only search on characters for the last tag
-    var index = Math.max(searchString.lastIndexOf(","), 
+    var index = Math.max(searchString.lastIndexOf(","),
       searchString.lastIndexOf(";"));
-    var before = ''; 
-    if (index != -1) {  
+    var before = '';
+    if (index != -1) {
       before = searchString.slice(0, index+1);
       searchString = searchString.slice(index+1);
       // skip past whitespace
@@ -633,10 +635,10 @@ TagAutoCompleteSearch.prototype = {
       listener.onSearchResult(self, newResult);
       return;
     }
-    
+
     var self = this;
     // generator: if yields true, not done
-    function doSearch() {
+    function* doSearch() {
       var i = 0;
       while (i < searchResults.length) {
         if (self._stopped)
@@ -648,7 +650,7 @@ TagAutoCompleteSearch.prototype = {
           results.push(before + searchResults[i]);
           comments.push(searchResults[i]);
         }
-    
+
         ++i;
 
         /* TODO: bug 481451
@@ -679,11 +681,10 @@ TagAutoCompleteSearch.prototype = {
       listener.onSearchResult(self, newResult);
       yield false;
     }
-    
+
     // chunk the search results via the generator
     var gen = doSearch();
-    while (gen.next());
-    gen.close();
+    while (gen.next().value);
   },
 
   /**
@@ -693,8 +694,7 @@ TagAutoCompleteSearch.prototype = {
     this._stopped = true;
   },
 
-  //////////////////////////////////////////////////////////////////////////////
-  //// nsISupports
+  // nsISupports
 
   classID: Components.ID("{1dcc23b0-d4cb-11dc-9ad6-479d56d89593}"),
 
