@@ -30,18 +30,14 @@
  * Messages sent:
  *
  *   Printing:Print
- *     This message is sent to kick off a print job for a particular content
- *     window (which is passed along with the message). We also pass print
- *     settings with this message - though bug 1088070 will have us gather
- *     those settings from the content process instead.
+ *     Kick off a print job for a nsIDOMWindow, passing the outer window ID as
+ *     windowID.
  *
  *   Printing:Preview:Enter
  *     This message is sent to put content into print preview mode. We pass
  *     the content window of the browser we're showing the preview of, and
  *     the target of the message is the browser that we'll be showing the
- *     preview in. We also pass print settings in this message, but
- *     bug 1088070 will have us gather those settings from the content process
- *     instead.
+ *     preview in.
  *
  *   Printing:Preview:Exit
  *     This message is sent to take content out of print preview mode.
@@ -69,20 +65,15 @@ var gSavePrintSettings = false;
 var gFocusedElement = null;
 
 var PrintUtils = {
-  bailOut: function () {
-    let pref = Components.classes["@mozilla.org/preferences-service;1"]
-                         .getService(Components.interfaces.nsIPrefBranch);
-    let allow_for_testing = false;
-    try {
-      allow_for_testing = pref.getBoolPref("print.enable_e10s_testing");
-    } catch(e) {
-      // The pref wasn't set, so I guess we're not overriding.
-    }
-    if (this.usingRemoteTabs && !allow_for_testing) {
-      alert("e10s printing is not implemented yet. Bug 927188.");
-      return true;
-    }
-    return false;
+  init() {
+    window.messageManager.addMessageListener("Printing:Error", this);
+  },
+
+  get bundle() {
+    let stringService = Components.classes["@mozilla.org/intl/stringbundle;1"]
+                                  .getService(Components.interfaces.nsIStringBundleService);
+    delete this.bundle;
+    return this.bundle = stringService.createBundle("chrome://global/locale/printing.properties");
   },
 
   /**
@@ -92,9 +83,6 @@ var PrintUtils = {
    * @return true on success, false on failure
    */
   showPageSetup: function () {
-    if (this.bailOut()) {
-      return;
-    }
     try {
       var printSettings = this.getPrintSettings();
       var PRINTPROMPTSVC = Components.classes["@mozilla.org/embedcomp/printingprompt-service;1"]
@@ -114,88 +102,60 @@ var PrintUtils = {
   },
 
   /**
-   * Starts printing the contents of aWindow.
+   * Starts the process of printing the contents of a window.
    *
-   * @param aWindow
-   *        An nsIDOMWindow to initiate the printing of. If the chrome window
-   *        is not running with remote tabs, this defaults to window.content if
-   *        omitted. If running with remote tabs, the caller must pass in the
-   *        content window to be printed. This function throws if that invariant
-   *        is violated.
-   * @param aBrowser (optional for non-remote browsers)
-   *        The remote <xul:browser> that contains aWindow. This argument is
-   *        not necessary if aWindow came from a non-remote browser, but is
-   *        strictly required otherwise. This function will throw if aWindow
-   *        comes from a remote browser and aBrowser is not provided. This
-   *        browser must have its type attribute set to "content",
-   *        "content-targetable", or "content-primary".
+   * @param aWindowID
+   *        The outer window ID of the nsIDOMWindow to print.
+   * @param aBrowser
+   *        The <xul:browser> that the nsIDOMWindow for aWindowID belongs to.
    */
-  print: function (aWindow, aBrowser)
+  printWindow: function (aWindowID, aBrowser)
   {
-    if (this.bailOut()) {
-      return;
+    let mm = aBrowser.messageManager;
+    mm.sendAsyncMessage("Printing:Print", {
+      windowID: aWindowID,
+      simplifiedMode: this._shouldSimplify,
+    });
+  },
+
+  /**
+   * Deprecated.
+   *
+   * Starts the process of printing the contents of window.content.
+   *
+   */
+  print: function ()
+  {
+    if (gBrowser) {
+      return this.printWindow(gBrowser.selectedBrowser.outerWindowID,
+                              gBrowser.selectedBrowser);
     }
 
-    if (!aWindow) {
-      // If we're using remote browsers, chances are that window.content will
-      // not be defined.
-      if (this.usingRemoteTabs) {
-        throw new Error("Windows running with remote tabs must explicitly pass " +
-                        "a content window to PrintUtils.print.");
-      }
-      // Otherwise, we should have access to window.content.
-      aWindow = window.content;
+    if (this.usingRemoteTabs) {
+      throw new Error("PrintUtils.print cannot be run in windows running with " +
+                      "remote tabs. Use PrintUtils.printWindow instead.");
     }
 
-    if (Components.utils.isCrossProcessWrapper(aWindow)) {
-      if (!aBrowser) {
-        throw new Error("PrintUtils.print expects a remote browser passed as " +
-                        "an argument if the content window is a CPOW.");
-      }
-    } else {
-      // For content windows coming from non-remote browsers, the browser can
-      // be resolved as the chromeEventHandler.
-      aBrowser = aWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                        .getInterface(Components.interfaces.nsIWebNavigation)
-                        .QueryInterface(Components.interfaces.nsIDocShell)
-                        .chromeEventHandler;
-    }
-
-    if (!aBrowser) {
+    let domWindow = window.content;
+    let ifReq = domWindow.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
+    let browser = ifReq.getInterface(Components.interfaces.nsIWebNavigation)
+                       .QueryInterface(Components.interfaces.nsIDocShell)
+                       .chromeEventHandler;
+    if (!browser) {
       throw new Error("PrintUtils.print could not resolve content window " +
                       "to a browser.");
     }
 
-    let printSettings = this.getPrintSettings();
+    let windowID = ifReq.getInterface(Components.interfaces.nsIDOMWindowUtils)
+                        .outerWindowID;
 
-    let mm = aBrowser.messageManager;
+    let Deprecated = Components.utils.import("resource://gre/modules/Deprecated.jsm", {}).Deprecated;
+    let msg = "PrintUtils.print is now deprecated. Please use PrintUtils.printWindow.";
+    let url = "https://developer.mozilla.org/en-US/docs/Mozilla/Tech/XUL/Printing";
+    Deprecated.warning(msg, url);
 
-    mm.addMessageListener("Printing:Print:Done", function onPrintingDone(msg) {
-      mm.removeMessageListener("Printing:Print:Done", onPrintingDone);
-      let rv = msg.data.rv;
-      let printSettings = msg.objects.printSettings;
-      if (rv != Components.results.NS_OK &&
-          rv != Components.results.NS_ERROR_ABORT) {
-        Cu.reportError(`In Printing:Print:Done handler, got unexpected rv
-                        ${rv}.`);
-      }
-
-      if (gPrintSettingsAreGlobal && gSavePrintSettings) {
-        let PSSVC =
-          Components.classes["@mozilla.org/gfx/printsettings-service;1"]
-                    .getService(Components.interfaces.nsIPrintSettingsService);
-
-        PSSVC.savePrintSettingsToPrefs(printSettings, true,
-                                       printSettings.kInitSaveAll);
-        PSSVC.savePrintSettingsToPrefs(printSettings, false,
-                                       printSettings.kInitSavePrinterName);
-      }
-    });
-
-    mm.sendAsyncMessage("Printing:Print", null, {
-      printSettings: printSettings,
-      contentWindow: aWindow,
-    });
+    this.printWindow(windowID, browser);
+    return undefined;
   },
 
   /**
@@ -230,9 +190,6 @@ var PrintUtils = {
    */
   printPreview: function (aListenerObj)
   {
-    if (this.bailOut()) {
-      return;
-    }
     // if we're already in PP mode, don't set the listener; chances
     // are it is null because someone is calling printPreview() to
     // get us to refresh the display.
@@ -241,12 +198,24 @@ var PrintUtils = {
       this._sourceBrowser = aListenerObj.getSourceBrowser();
       this._originalTitle = this._sourceBrowser.contentTitle;
       this._originalURL = this._sourceBrowser.currentURI.spec;
+
+      // Here we log telemetry data for when the user enters print preview.
+      this.logTelemetry("PRINT_PREVIEW_OPENED_COUNT");
     } else {
       // collapse the browser here -- it will be shown in
       // enterPrintPreview; this forces a reflow which fixes display
       // issues in bug 267422.
+      // We use the print preview browser as the source browser to avoid
+      // re-initializing print preview with a document that might now have changed.
       this._sourceBrowser = this._listener.getPrintPreviewBrowser();
       this._sourceBrowser.collapsed = true;
+
+      // If the user transits too quickly within preview and we have a pending
+      // progress dialog, we will close it before opening a new one.
+      if (this._webProgressPP && this._webProgressPP.value) {
+        this._webProgressPP.value.onStateChange(null, null,
+          Components.interfaces.nsIWebProgressListener.STATE_STOP, 0);
+      }
     }
 
     this._webProgressPP = {};
@@ -333,9 +302,7 @@ var PrintUtils = {
     return document.getElementById("print-preview-toolbar") != null;
   },
 
-  ////////////////////////////////////////////////////
-  // "private" methods and members. Don't use them. //
-  ///////////////////////////////////////////////////
+  // "private" methods and members. Don't use them.
 
   _listener: null,
   _closeHandlerPP: null,
@@ -343,6 +310,7 @@ var PrintUtils = {
   _sourceBrowser: null,
   _originalTitle: "",
   _originalURL: "",
+  _shouldSimplify: false,
 
   get usingRemoteTabs() {
     // We memoize this, since it's highly unlikely to change over the lifetime
@@ -356,11 +324,81 @@ var PrintUtils = {
     return this.usingRemoteTabs = usingRemoteTabs;
   },
 
+  displayPrintingError(nsresult, isPrinting) {
+    // The nsresults from a printing error are mapped to strings that have
+    // similar names to the errors themselves. For example, for error
+    // NS_ERROR_GFX_PRINTER_NO_PRINTER_AVAILABLE, the name of the string
+    // for the error message is: PERR_GFX_PRINTER_NO_PRINTER_AVAILABLE. What's
+    // more, if we're in the process of doing a print preview, it's possible
+    // that there are strings specific for print preview for these errors -
+    // if so, the names of those strings have _PP as a suffix. It's possible
+    // that no print preview specific strings exist, in which case it is fine
+    // to fall back to the original string name.
+    const MSG_CODES = [
+      "GFX_PRINTER_NO_PRINTER_AVAILABLE",
+      "GFX_PRINTER_NAME_NOT_FOUND",
+      "GFX_PRINTER_COULD_NOT_OPEN_FILE",
+      "GFX_PRINTER_STARTDOC",
+      "GFX_PRINTER_ENDDOC",
+      "GFX_PRINTER_STARTPAGE",
+      "GFX_PRINTER_DOC_IS_BUSY",
+      "ABORT",
+      "NOT_AVAILABLE",
+      "NOT_IMPLEMENTED",
+      "OUT_OF_MEMORY",
+      "UNEXPECTED",
+    ];
+
+    // PERR_FAILURE is the catch-all error message if we've gotten one that
+    // we don't recognize.
+    msgName = "PERR_FAILURE";
+
+    for (let code of MSG_CODES) {
+      let nsErrorResult = "NS_ERROR_" + code;
+      if (Components.results[nsErrorResult] == nsresult) {
+        msgName = "PERR_" + code;
+        break;
+      }
+    }
+
+    let msg, title;
+
+    if (!isPrinting) {
+      // Try first with _PP suffix.
+      let ppMsgName = msgName + "_PP";
+      try {
+        msg = this.bundle.GetStringFromName(ppMsgName);
+      } catch (e) {
+        // We allow localizers to not have the print preview error string,
+        // and just fall back to the printing error string.
+      }
+    }
+
+    if (!msg) {
+      msg = this.bundle.GetStringFromName(msgName);
+    }
+
+    title = this.bundle.GetStringFromName(isPrinting ? "print_error_dialog_title"
+                                                     : "printpreview_error_dialog_title");
+
+    let promptSvc = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
+                              .getService(Components.interfaces.nsIPromptService);
+    promptSvc.alert(window, title, msg);
+  },
+
   receiveMessage(aMessage) {
+    if (aMessage.name == "Printing:Error") {
+      this.displayPrintingError(aMessage.data.nsresult,
+                                aMessage.data.isPrinting);
+      return undefined;
+    }
+
+    // If we got here, then the message we've received must involve
+    // updating the print progress UI.
     if (!this._webProgressPP.value) {
       // We somehow didn't get a nsIWebProgressListener to be updated...
       // I guess there's nothing to do.
-      return;
+      return undefined;
     }
 
     let listener = this._webProgressPP.value;
@@ -374,7 +412,6 @@ var PrintUtils = {
                                          data.maxSelfProgress,
                                          data.curTotalProgress,
                                          data.maxTotalProgress);
-        break;
       }
 
       case "Printing:Preview:StateChange": {
@@ -392,9 +429,9 @@ var PrintUtils = {
         return listener.onStateChange(null, null,
                                       data.stateFlags,
                                       data.status);
-        break;
       }
     }
+    return undefined;
   },
 
   setPrinterDefaultsForSelectedPrinter: function (aPSSVC, aPrintSettings)
@@ -402,7 +439,7 @@ var PrintUtils = {
     if (!aPrintSettings.printerName)
       aPrintSettings.printerName = aPSSVC.defaultPrinterName;
 
-    // First get any defaults from the printer 
+    // First get any defaults from the printer
     aPSSVC.initPrintSettingsFromPrinter(aPrintSettings.printerName, aPrintSettings);
     // now augment them with any values from last time
     aPSSVC.initPrintSettingsFromPrefs(aPrintSettings, true,  aPrintSettings.kInitSaveAll);
@@ -452,6 +489,11 @@ var PrintUtils = {
     }
   },
 
+  setSimplifiedMode: function (shouldSimplify)
+  {
+    this._shouldSimplify = shouldSimplify;
+  },
+
   enterPrintPreview: function ()
   {
     // Send a message to the print preview browser to initialize
@@ -461,12 +503,49 @@ var PrintUtils = {
     // listener.
     let ppBrowser = this._listener.getPrintPreviewBrowser();
     let mm = ppBrowser.messageManager;
-    let printSettings = this.getPrintSettings();
-    mm.sendAsyncMessage("Printing:Preview:Enter", null, {
-      printSettings: printSettings,
-      contentWindow: this._sourceBrowser.contentWindowAsCPOW ||
-                     this._sourceBrowser.contentWindow,
-    });
+
+    let sendEnterPreviewMessage = function (browser, simplified) {
+      mm.sendAsyncMessage("Printing:Preview:Enter", {
+        windowID: browser.outerWindowID,
+        simplifiedMode: simplified,
+      });
+    };
+
+    // If we happen to have gotten simplify page checked, we will lazily
+    // instantiate a new tab that parses the original page using ReaderMode
+    // primitives. When it's ready, and in order to enter on preview, we send
+    // over a message to print preview browser passing up the simplified tab as
+    // reference. If not, we pass the original tab instead as content source.
+    if (this._shouldSimplify) {
+      let simplifiedBrowser = this._listener.getSimplifiedSourceBrowser();
+      if (simplifiedBrowser) {
+        sendEnterPreviewMessage(simplifiedBrowser, true);
+      } else {
+        simplifiedBrowser = this._listener.createSimplifiedBrowser();
+
+        // After instantiating the simplified tab, we attach a listener as
+        // callback. Once we discover reader mode has been loaded, we fire
+        // up a message to enter on print preview.
+        let spMM = simplifiedBrowser.messageManager;
+        spMM.addMessageListener("Printing:Preview:ReaderModeReady", function onReaderReady() {
+          spMM.removeMessageListener("Printing:Preview:ReaderModeReady", onReaderReady);
+          sendEnterPreviewMessage(simplifiedBrowser, true);
+        });
+
+        // Here, we send down a message to simplified browser in order to parse
+        // the original page. After we have parsed it, content will tell parent
+        // that the document is ready for print previewing.
+        spMM.sendAsyncMessage("Printing:Preview:ParseDocument", {
+          URL: this._originalURL,
+          windowID: this._sourceBrowser.outerWindowID,
+        });
+
+        // Here we log telemetry data for when the user enters simplify mode.
+        this.logTelemetry("PRINT_PREVIEW_SIMPLIFY_PAGE_OPENED_COUNT");
+      }
+    } else {
+      sendEnterPreviewMessage(this._sourceBrowser, false);
+    }
 
     if (this._webProgressPP.value) {
       mm.addMessageListener("Printing:Preview:StateChange", this);
@@ -498,7 +577,11 @@ var PrintUtils = {
 
       // Set the original window as an active window so any mozPrintCallbacks can
       // run without delayed setTimeouts.
-      this._sourceBrowser.docShellIsActive = true;
+      if (this._listener.activateBrowser) {
+        this._listener.activateBrowser(this._sourceBrowser);
+      } else {
+        this._sourceBrowser.docShellIsActive = true;
+      }
 
       // show the toolbar after we go into print preview mode so
       // that we can initialize the toolbar with total num pages
@@ -506,12 +589,21 @@ var PrintUtils = {
         "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
       printPreviewTB = document.createElementNS(XUL_NS, "toolbar");
       printPreviewTB.setAttribute("printpreview", true);
+      printPreviewTB.setAttribute("fullscreentoolbar", true);
       printPreviewTB.id = "print-preview-toolbar";
 
       let navToolbox = this._listener.getNavToolbox();
       navToolbox.parentNode.insertBefore(printPreviewTB, navToolbox);
       printPreviewTB.initialize(ppBrowser);
 
+      // Enable simplify page checkbox when the page is an article
+      if (this._sourceBrowser.isArticle) {
+        printPreviewTB.enableSimplifyPage();
+      } else {
+        this.logTelemetry("PRINT_PREVIEW_SIMPLIFY_PAGE_UNAVAILABLE_COUNT");
+        printPreviewTB.disableSimplifyPage();
+      }
+
       // copy the window close handler
       if (document.documentElement.hasAttribute("onclose"))
         this._closeHandlerPP = document.documentElement.getAttribute("onclose");
@@ -556,9 +648,17 @@ var PrintUtils = {
       this._sourceBrowser.focus();
     gFocusedElement = null;
 
+    this.setSimplifiedMode(false);
+
     this._listener.onExit();
   },
 
+  logTelemetry: function (ID)
+  {
+    let histogram = Services.telemetry.getHistogramById(ID);
+    histogram.add(true);
+  },
+
   onKeyDownPP: function (aEvent)
   {
     // Esc exits the PP
@@ -596,3 +696,5 @@ var PrintUtils = {
     }
   }
 }
+
+PrintUtils.init();
