@@ -68,12 +68,12 @@ add_task(function* test_transfer_args() {
   for (let i = 0; i < 4; ++i) {
     array[i] = i;
   }
-  Assert.equal(array.buffer.byteLength, 4, "The buffer is not neutered yet");
+  Assert.equal(array.buffer.byteLength, 4, "The buffer is not detached yet");
 
   let result = (yield worker.post("bounce", [array.buffer], [], [array.buffer]))[0];
 
   // Check that the buffer has been sent
-  Assert.equal(array.buffer.byteLength, 0, "The buffer has been neutered");
+  Assert.equal(array.buffer.byteLength, 0, "The buffer has been detached");
 
   // Check that the result is correct
   Assert.equal(result.byteLength, 4, "The result has the right size");
@@ -89,16 +89,17 @@ add_task(function* test_transfer_with_meta() {
   for (let i = 0; i < 4; ++i) {
     array[i] = i;
   }
-  Assert.equal(array.buffer.byteLength, 4, "The buffer is not neutered yet");
+  Assert.equal(array.buffer.byteLength, 4, "The buffer is not detached yet");
 
   let message = new BasePromiseWorker.Meta(array, {transfers: [array.buffer]});
   let result = (yield worker.post("bounce", [message]))[0];
 
   // Check that the buffer has been sent
-  Assert.equal(array.buffer.byteLength, 0, "The buffer has been neutered");
+  Assert.equal(array.buffer.byteLength, 0, "The buffer has been detached");
 
   // Check that the result is correct
-  Assert.equal(result.toString(), "[object Uint8Array]", "The result appears to be a Typed Array");
+  Assert.equal(Object.prototype.toString.call(result), "[object Uint8Array]",
+               "The result appears to be a Typed Array");
   Assert.equal(result.byteLength, 4, "The result has the right size");
 
   for (let i = 0; i < 4; ++i) {
@@ -106,6 +107,11 @@ add_task(function* test_transfer_with_meta() {
   }
 });
 
-function run_test() {
-  run_next_test();
-}
+add_task(function* test_throw_error() {
+  try {
+    yield worker.post("throwError", ["error message"]);
+    Assert.ok(false, "should have thrown");
+  } catch (ex) {
+    Assert.equal(ex.message, "Error: error message");
+  }
+});
