@@ -15,7 +15,13 @@ function whenBrowserLoaded(browser, callback) {
 }
 
 function waitForOnBeforeUnloadDialog(browser, callback) {
-  browser.addEventListener("DOMWillOpenModalDialog", function onModalDialog() {
+  browser.addEventListener("DOMWillOpenModalDialog", function onModalDialog(event) {
+    if (Cu.isCrossProcessWrapper(event.target)) {
+      // This event fires in both the content and chrome processes. We
+      // want to ignore the one in the content process.
+      return;
+    }
+
     browser.removeEventListener("DOMWillOpenModalDialog", onModalDialog, true);
 
     executeSoon(() => {
