@@ -4,99 +4,16 @@
 const INT_MAX = 0x7FFFFFFF;
 
 Cu.import("resource://gre/modules/Services.jsm", this);
+Cu.import("resource://gre/modules/TelemetryUtils.jsm", this);
 
-function test_expired_histogram() {
-  var histogram_id = "FOOBAR";
-  var test_expired_id = "TELEMETRY_TEST_EXPIRED";
-  var clone_id = "ExpiredClone";
-  var dummy = Telemetry.newHistogram(histogram_id, "28.0a1", Telemetry.HISTOGRAM_EXPONENTIAL, 1, 2, 3);
-  var dummy_clone = Telemetry.histogramFrom(clone_id, test_expired_id);
-  var rh = Telemetry.registeredHistograms(Ci.nsITelemetry.DATASET_RELEASE_CHANNEL_OPTIN, []);
-  Assert.ok(!!rh);
-
-  dummy.add(1);
-  dummy_clone.add(1);
-
-  do_check_eq(Telemetry.histogramSnapshots["__expired__"], undefined);
-  do_check_eq(Telemetry.histogramSnapshots[histogram_id], undefined);
-  do_check_eq(Telemetry.histogramSnapshots[test_expired_id], undefined);
-  do_check_eq(Telemetry.histogramSnapshots[clone_id], undefined);
-  do_check_eq(rh[test_expired_id], undefined);
-}
-
-function test_histogram(histogram_type, name, min, max, bucket_count) {
-  var h = Telemetry.newHistogram(name, "never", histogram_type, min, max, bucket_count);
-  var r = h.snapshot().ranges;
-  var sum = 0;
-  var log_sum = 0;
-  var log_sum_squares = 0;
-  for(var i=0;i<r.length;i++) {
-    var v = r[i];
-    sum += v;
-    if (histogram_type == Telemetry.HISTOGRAM_EXPONENTIAL) {
-      var log_v = Math.log(1+v);
-      log_sum += log_v;
-      log_sum_squares += log_v*log_v;
-    }
-    h.add(v);
+// Return an array of numbers from lower up to, excluding, upper
+function numberRange(lower, upper)
+{
+  let a = [];
+  for (let i=lower; i<upper; ++i) {
+    a.push(i);
   }
-  var s = h.snapshot();
-  // verify properties
-  do_check_eq(sum, s.sum);
-  if (histogram_type == Telemetry.HISTOGRAM_EXPONENTIAL) {
-    // We do the log with float precision in C++ and double precision in
-    // JS, so there's bound to be tiny discrepancies.  Just check the
-    // integer part.
-    do_check_eq(Math.floor(log_sum), Math.floor(s.log_sum));
-    do_check_eq(Math.floor(log_sum_squares), Math.floor(s.log_sum_squares));
-    do_check_false("sum_squares_lo" in s);
-    do_check_false("sum_squares_hi" in s);
-  } else {
-    // Doing the math to verify sum_squares was reflected correctly is
-    // tedious in JavaScript.  Just make sure we have something.
-    do_check_neq(s.sum_squares_lo + s.sum_squares_hi, 0);
-    do_check_false("log_sum" in s);
-    do_check_false("log_sum_squares" in s);
-  }
-
-  // there should be exactly one element per bucket
-  for (var i of s.counts) {
-    do_check_eq(i, 1);
-  }
-  var hgrams = Telemetry.histogramSnapshots
-  gh = hgrams[name]
-  do_check_eq(gh.histogram_type, histogram_type);
-
-  do_check_eq(gh.min, min)
-  do_check_eq(gh.max, max)
-
-  // Check that booleans work with nonboolean histograms
-  h.add(false);
-  h.add(true);
-  var s = h.snapshot().counts;
-  do_check_eq(s[0], 2)
-  do_check_eq(s[1], 2)
-
-  // Check that clearing works.
-  h.clear();
-  var s = h.snapshot();
-  for (var i of s.counts) {
-    do_check_eq(i, 0);
-  }
-  do_check_eq(s.sum, 0);
-  if (histogram_type == Telemetry.HISTOGRAM_EXPONENTIAL) {
-    do_check_eq(s.log_sum, 0);
-    do_check_eq(s.log_sum_squares, 0);
-  } else {
-    do_check_eq(s.sum_squares_lo, 0);
-    do_check_eq(s.sum_squares_hi, 0);
-  }
-
-  h.add(0);
-  h.add(1);
-  var c = h.snapshot().counts;
-  do_check_eq(c[0], 1);
-  do_check_eq(c[1], 1);
+  return a;
 }
 
 function expect_fail(f) {
@@ -121,16 +38,115 @@ function expect_success(f) {
   do_check_true(succeeded);
 }
 
-function test_boolean_histogram()
-{
-  var h = Telemetry.newHistogram("test::boolean histogram", "never", Telemetry.HISTOGRAM_BOOLEAN);
+function compareHistograms(h1, h2) {
+  let s1 = h1.snapshot();
+  let s2 = h2.snapshot();
+
+  do_check_eq(s1.histogram_type, s2.histogram_type);
+  do_check_eq(s1.min, s2.min);
+  do_check_eq(s1.max, s2.max);
+  do_check_eq(s1.sum, s2.sum);
+
+  do_check_eq(s1.counts.length, s2.counts.length);
+  for (let i = 0; i < s1.counts.length; i++)
+    do_check_eq(s1.counts[i], s2.counts[i]);
+
+  do_check_eq(s1.ranges.length, s2.ranges.length);
+  for (let i = 0; i < s1.ranges.length; i++)
+    do_check_eq(s1.ranges[i], s2.ranges[i]);
+}
+
+function check_histogram(histogram_type, name, min, max, bucket_count) {
+  var h = Telemetry.getHistogramById(name);
+  var r = h.snapshot().ranges;
+  var sum = 0;
+  for (let i=0;i<r.length;i++) {
+    var v = r[i];
+    sum += v;
+    h.add(v);
+  }
+  var s = h.snapshot();
+  // verify properties
+  do_check_eq(sum, s.sum);
+
+  // there should be exactly one element per bucket
+  for (let i of s.counts) {
+    do_check_eq(i, 1);
+  }
+  var hgrams = Telemetry.histogramSnapshots
+  let gh = hgrams[name]
+  do_check_eq(gh.histogram_type, histogram_type);
+
+  do_check_eq(gh.min, min)
+  do_check_eq(gh.max, max)
+
+  // Check that booleans work with nonboolean histograms
+  h.add(false);
+  h.add(true);
+  s = h.snapshot().counts;
+  do_check_eq(s[0], 2)
+  do_check_eq(s[1], 2)
+
+  // Check that clearing works.
+  h.clear();
+  s = h.snapshot();
+  for (var i of s.counts) {
+    do_check_eq(i, 0);
+  }
+  do_check_eq(s.sum, 0);
+
+  h.add(0);
+  h.add(1);
+  var c = h.snapshot().counts;
+  do_check_eq(c[0], 1);
+  do_check_eq(c[1], 1);
+}
+
+// This MUST be the very first test of this file.
+add_task({
+  skip_if: () => gIsAndroid
+},
+function* test_instantiate() {
+  const ID = "TELEMETRY_TEST_COUNT";
+  let h = Telemetry.getHistogramById(ID);
+
+  // Instantiate the subsession histogram through |add| and make sure they match.
+  // This MUST be the first use of "TELEMETRY_TEST_COUNT" in this file, otherwise
+  // |add| will not instantiate the histogram.
+  h.add(1);
+  let snapshot = h.snapshot();
+  let subsession = Telemetry.snapshotSubsessionHistograms();
+  Assert.equal(snapshot.sum, subsession[ID].sum,
+               "Histogram and subsession histogram sum must match.");
+  // Clear the histogram, so we don't void the assumptions from the other tests.
+  h.clear();
+});
+
+add_task(function* test_parameterChecks() {
+  let kinds = [Telemetry.HISTOGRAM_EXPONENTIAL, Telemetry.HISTOGRAM_LINEAR]
+  let testNames = ["TELEMETRY_TEST_EXPONENTIAL", "TELEMETRY_TEST_LINEAR"]
+  for (let i = 0; i < kinds.length; i++) {
+    let histogram_type = kinds[i];
+    let test_type = testNames[i];
+    let [min, max, bucket_count] = [1, INT_MAX - 1, 10]
+    check_histogram(histogram_type, test_type, min, max, bucket_count);
+  }
+});
+
+add_task(function* test_noSerialization() {
+  // Instantiate the storage for this histogram and make sure it doesn't
+  // get reflected into JS, as it has no interesting data in it.
+  Telemetry.getHistogramById("NEWTAB_PAGE_PINNED_SITES_COUNT");
+  do_check_false("NEWTAB_PAGE_PINNED_SITES_COUNT" in Telemetry.histogramSnapshots);
+});
+
+add_task(function* test_boolean_histogram() {
+  var h = Telemetry.getHistogramById("TELEMETRY_TEST_BOOLEAN");
   var r = h.snapshot().ranges;
   // boolean histograms ignore numeric parameters
   do_check_eq(uneval(r), uneval([0, 1, 2]))
-  var sum = 0
-  for(var i=0;i<r.length;i++) {
+  for (var i=0;i<r.length;i++) {
     var v = r[i];
-    sum += v;
     h.add(v);
   }
   h.add(true);
@@ -141,11 +157,10 @@ function test_boolean_histogram()
   do_check_eq(s.counts[2], 0);
   do_check_eq(s.sum, 3);
   do_check_eq(s.counts[0], 2);
-}
+});
 
-function test_flag_histogram()
-{
-  var h = Telemetry.newHistogram("test::flag histogram", "never", Telemetry.HISTOGRAM_FLAG);
+add_task(function* test_flag_histogram() {
+  var h = Telemetry.getHistogramById("TELEMETRY_TEST_FLAG");
   var r = h.snapshot().ranges;
   // Flag histograms ignore numeric parameters.
   do_check_eq(uneval(r), uneval([0, 1, 2]));
@@ -167,11 +182,10 @@ function test_flag_histogram()
   do_check_eq(uneval(c3), uneval([0, 1, 0]));
   do_check_eq(s3, 1);
   do_check_eq(h.snapshot().histogram_type, Telemetry.HISTOGRAM_FLAG);
-}
+});
 
-function test_count_histogram()
-{
-  let h = Telemetry.newHistogram("test::count histogram", "never", Telemetry.HISTOGRAM_COUNT, 1, 2, 3);
+add_task(function* test_count_histogram() {
+  let h = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT2");
   let s = h.snapshot();
   do_check_eq(uneval(s.ranges), uneval([0, 1, 2]));
   do_check_eq(uneval(s.counts), uneval([0, 0, 0]));
@@ -184,89 +198,130 @@ function test_count_histogram()
   s = h.snapshot();
   do_check_eq(uneval(s.counts), uneval([2, 0, 0]));
   do_check_eq(s.sum, 2);
-}
+});
 
-function test_getHistogramById() {
+add_task(function* test_categorical_histogram()
+{
+  let h1 = Telemetry.getHistogramById("TELEMETRY_TEST_CATEGORICAL");
+  for (let v of ["CommonLabel", "Label2", "Label3", "Label3", 0, 0, 1]) {
+    h1.add(v);
+  }
+  for (let s of ["", "Label4", "1234"]) {
+    Assert.throws(() => h1.add(s));
+  }
+
+  let snapshot = h1.snapshot();
+  Assert.equal(snapshot.sum, 6);
+  Assert.deepEqual(snapshot.ranges, [0, 1, 2, 3]);
+  Assert.deepEqual(snapshot.counts, [3, 2, 2, 0]);
+
+  let h2 = Telemetry.getHistogramById("TELEMETRY_TEST_CATEGORICAL_OPTOUT");
+  for (let v of ["CommonLabel", "CommonLabel", "Label4", "Label5", "Label6", 0, 1]) {
+    h2.add(v);
+  }
+  for (let s of ["", "Label3", "1234"]) {
+    Assert.throws(() => h2.add(s));
+  }
+
+  snapshot = h2.snapshot();
+  Assert.equal(snapshot.sum, 7);
+  Assert.deepEqual(snapshot.ranges, [0, 1, 2, 3, 4]);
+  Assert.deepEqual(snapshot.counts, [3, 2, 1, 1, 0]);
+});
+
+add_task(function* test_getHistogramById() {
   try {
     Telemetry.getHistogramById("nonexistent");
     do_throw("This can't happen");
   } catch (e) {
-    
+
   }
   var h = Telemetry.getHistogramById("CYCLE_COLLECTOR");
   var s = h.snapshot();
   do_check_eq(s.histogram_type, Telemetry.HISTOGRAM_EXPONENTIAL);
   do_check_eq(s.min, 1);
   do_check_eq(s.max, 10000);
-}
+});
 
-function compareHistograms(h1, h2) {
-  let s1 = h1.snapshot();
-  let s2 = h2.snapshot();
-
-  do_check_eq(s1.histogram_type, s2.histogram_type);
-  do_check_eq(s1.min, s2.min);
-  do_check_eq(s1.max, s2.max);
-  do_check_eq(s1.sum, s2.sum);
-  if (s1.histogram_type == Telemetry.HISTOGRAM_EXPONENTIAL) {
-    do_check_eq(s1.log_sum, s2.log_sum);
-    do_check_eq(s1.log_sum_squares, s2.log_sum_squares);
-  } else {
-    do_check_eq(s1.sum_squares_lo, s2.sum_squares_lo);
-    do_check_eq(s1.sum_squares_hi, s2.sum_squares_hi);
-  }
-
-  do_check_eq(s1.counts.length, s2.counts.length);
-  for (let i = 0; i < s1.counts.length; i++)
-    do_check_eq(s1.counts[i], s2.counts[i]);
-
-  do_check_eq(s1.ranges.length, s2.ranges.length);
-  for (let i = 0; i < s1.ranges.length; i++)
-    do_check_eq(s1.ranges[i], s2.ranges[i]);
-}
-
-function test_histogramFrom() {
-  // Test one histogram of each type.
-  let names = [
-      "CYCLE_COLLECTOR",      // EXPONENTIAL
-      "GC_REASON_2",          // LINEAR
-      "GC_RESET",             // BOOLEAN
-      "TELEMETRY_TEST_FLAG",  // FLAG
-      "TELEMETRY_TEST_COUNT", // COUNT
-  ];
-
-  for (let name of names) {
-    let [min, max, bucket_count] = [1, INT_MAX - 1, 10]
-    let original = Telemetry.getHistogramById(name);
-    let clone = Telemetry.histogramFrom("clone" + name, name);
-    compareHistograms(original, clone);
-  }
-
-  // Additionally, set TELEMETRY_TEST_FLAG and TELEMETRY_TEST_COUNT
-  // and check they get set on the clone.
-  let testFlag = Telemetry.getHistogramById("TELEMETRY_TEST_FLAG");
-  testFlag.add(1);
-  let testCount = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT");
-  testCount.add();
-  let clone = Telemetry.histogramFrom("FlagClone", "TELEMETRY_TEST_FLAG");
-  compareHistograms(testFlag, clone);
-  clone = Telemetry.histogramFrom("CountClone", "TELEMETRY_TEST_COUNT");
-  compareHistograms(testCount, clone);
-}
-
-function test_getSlowSQL() {
+add_task(function* test_getSlowSQL() {
   var slow = Telemetry.slowSQL;
   do_check_true(("mainThread" in slow) && ("otherThreads" in slow));
-}
+});
 
-function test_getWebrtc() {
+add_task(function* test_getWebrtc() {
   var webrtc = Telemetry.webrtcStats;
   do_check_true("IceCandidatesStats" in webrtc);
   var icestats = webrtc.IceCandidatesStats;
-  do_check_true(("webrtc" in icestats) && ("loop" in icestats));
-}
+  do_check_true("webrtc" in icestats);
+});
 
-function test_addons() {
+// Check that telemetry doesn't record in private mode
+add_task(function* test_privateMode() {
+  var h = Telemetry.getHistogramById("TELEMETRY_TEST_BOOLEAN");
+  var orig = h.snapshot();
+  Telemetry.canRecordExtended = false;
+  h.add(1);
+  do_check_eq(uneval(orig), uneval(h.snapshot()));
+  Telemetry.canRecordExtended = true;
+  h.add(1);
+  do_check_neq(uneval(orig), uneval(h.snapshot()));
+});
+
+// Check that telemetry records only when it is suppose to.
+add_task(function* test_histogramRecording() {
+  // Check that no histogram is recorded if both base and extended recording are off.
+  Telemetry.canRecordBase = false;
+  Telemetry.canRecordExtended = false;
+
+  let h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTOUT");
+  h.clear();
+  let orig = h.snapshot();
+  h.add(1);
+  Assert.equal(orig.sum, h.snapshot().sum);
+
+  // Check that only base histograms are recorded.
+  Telemetry.canRecordBase = true;
+  h.add(1);
+  Assert.equal(orig.sum + 1, h.snapshot().sum,
+               "Histogram value should have incremented by 1 due to recording.");
+
+  // Extended histograms should not be recorded.
+  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTIN");
+  orig = h.snapshot();
+  h.add(1);
+  Assert.equal(orig.sum, h.snapshot().sum,
+               "Histograms should be equal after recording.");
+
+  // Runtime created histograms should not be recorded.
+  h = Telemetry.getHistogramById("TELEMETRY_TEST_BOOLEAN");
+  orig = h.snapshot();
+  h.add(1);
+  Assert.equal(orig.sum, h.snapshot().sum,
+               "Histograms should be equal after recording.");
+
+  // Check that extended histograms are recorded when required.
+  Telemetry.canRecordExtended = true;
+
+  h.add(1);
+  Assert.equal(orig.sum + 1, h.snapshot().sum,
+               "Runtime histogram value should have incremented by 1 due to recording.");
+
+  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTIN");
+  orig = h.snapshot();
+  h.add(1);
+  Assert.equal(orig.sum + 1, h.snapshot().sum,
+               "Histogram value should have incremented by 1 due to recording.");
+
+  // Check that base histograms are still being recorded.
+  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTOUT");
+  h.clear();
+  orig = h.snapshot();
+  h.add(1);
+  Assert.equal(orig.sum + 1, h.snapshot().sum,
+               "Histogram value should have incremented by 1 due to recording.");
+});
+
+add_task(function* test_addons() {
   var addon_id = "testing-addon";
   var fake_addon_id = "fake-addon";
   var name1 = "testing-histogram1";
@@ -352,102 +407,36 @@ function test_addons() {
   do_check_false(addon_id in snapshots);
   // Make sure other addons are unaffected.
   do_check_true(extra_addon in snapshots);
-}
+});
 
-// Check that telemetry doesn't record in private mode
-function test_privateMode() {
-  var h = Telemetry.newHistogram("test::private_mode_boolean", "never", Telemetry.HISTOGRAM_BOOLEAN);
-  var orig = h.snapshot();
-  Telemetry.canRecordExtended = false;
-  h.add(1);
-  do_check_eq(uneval(orig), uneval(h.snapshot()));
-  Telemetry.canRecordExtended = true;
-  h.add(1);
-  do_check_neq(uneval(orig), uneval(h.snapshot()));
-}
+add_task(function* test_expired_histogram() {
+  var test_expired_id = "TELEMETRY_TEST_EXPIRED";
+  var dummy = Telemetry.getHistogramById(test_expired_id);
+  var rh = Telemetry.registeredHistograms(Ci.nsITelemetry.DATASET_RELEASE_CHANNEL_OPTIN, []);
+  Assert.ok(!!rh);
 
-// Check that telemetry records only when it is suppose to.
-function test_histogramRecording() {
-  // Check that no histogram is recorded if both base and extended recording are off.
-  Telemetry.canRecordBase = false;
-  Telemetry.canRecordExtended = false;
+  dummy.add(1);
 
-  let h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTOUT");
-  h.clear();
-  let orig = h.snapshot();
-  h.add(1);
-  Assert.equal(orig.sum, h.snapshot().sum);
+  do_check_eq(Telemetry.histogramSnapshots["__expired__"], undefined);
+  do_check_eq(Telemetry.histogramSnapshots[test_expired_id], undefined);
+  do_check_eq(rh[test_expired_id], undefined);
+});
 
-  // Check that only base histograms are recorded.
-  Telemetry.canRecordBase = true;
-  h.add(1);
-  Assert.equal(orig.sum + 1, h.snapshot().sum,
-               "Histogram value should have incremented by 1 due to recording.");
+add_task(function* test_keyed_histogram() {
+  // Check that invalid names get rejected.
 
-  // Extended histograms should not be recorded.
-  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTIN");
-  orig = h.snapshot();
-  h.add(1);
-  Assert.equal(orig.sum, h.snapshot().sum,
-               "Histograms should be equal after recording.");
-
-  // Runtime created histograms should not be recorded.
-  h = Telemetry.newHistogram("test::runtime_created_boolean", "never", Telemetry.HISTOGRAM_BOOLEAN);
-  orig = h.snapshot();
-  h.add(1);
-  Assert.equal(orig.sum, h.snapshot().sum,
-               "Histograms should be equal after recording.");
-
-  // Check that extended histograms are recorded when required.
-  Telemetry.canRecordExtended = true;
-
-  h.add(1);
-  Assert.equal(orig.sum + 1, h.snapshot().sum,
-               "Runtime histogram value should have incremented by 1 due to recording.");
-
-  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTIN");
-  orig = h.snapshot();
-  h.add(1);
-  Assert.equal(orig.sum + 1, h.snapshot().sum,
-               "Histogram value should have incremented by 1 due to recording.");
-
-  // Check that base histograms are still being recorded.
-  h = Telemetry.getHistogramById("TELEMETRY_TEST_RELEASE_OPTOUT");
-  h.clear();
-  orig = h.snapshot();
-  h.add(1);
-  Assert.equal(orig.sum + 1, h.snapshot().sum,
-               "Histogram value should have incremented by 1 due to recording.");
-}
-
-// Check that histograms that aren't flagged as needing extended stats
-// don't record extended stats.
-function test_extended_stats() {
-  var h = Telemetry.getHistogramById("GRADIENT_DURATION");
-  var s = h.snapshot();
-  do_check_eq(s.sum, 0);
-  do_check_eq(s.log_sum, 0);
-  do_check_eq(s.log_sum_squares, 0);
-  h.add(1);
-  s = h.snapshot();
-  do_check_eq(s.sum, 1);
-  do_check_eq(s.log_sum, 0);
-  do_check_eq(s.log_sum_squares, 0);
-}
-
-// Return an array of numbers from lower up to, excluding, upper
-function numberRange(lower, upper)
-{
-  let a = [];
-  for (let i=lower; i<upper; ++i) {
-    a.push(i);
+  let threw = false;
+  try {
+    Telemetry.getKeyedHistogramById("test::unknown histogram", "never", Telemetry.HISTOGRAM_BOOLEAN);
+  } catch (e) {
+    // This should throw as it is an unknown ID
+    threw = true;
   }
-  return a;
-}
+  Assert.ok(threw, "getKeyedHistogramById should have thrown");
+});
 
-function test_keyed_boolean_histogram()
-{
-  const KEYED_ID = "test::keyed::boolean";
+add_task(function* test_keyed_boolean_histogram() {
+  const KEYED_ID = "TELEMETRY_TEST_KEYED_BOOLEAN";
   let KEYS = numberRange(0, 2).map(i => "key" + (i + 1));
   KEYS.push("漢語");
   let histogramBase = {
@@ -455,8 +444,6 @@ function test_keyed_boolean_histogram()
     "max": 2,
     "histogram_type": 2,
     "sum": 1,
-    "sum_squares_lo": 1,
-    "sum_squares_hi": 0,
     "ranges": [0, 1, 2],
     "counts": [0, 1, 0]
   };
@@ -464,7 +451,7 @@ function test_keyed_boolean_histogram()
   let testKeys = [];
   let testSnapShot = {};
 
-  let h = Telemetry.newKeyedHistogram(KEYED_ID, "never", Telemetry.HISTOGRAM_BOOLEAN);
+  let h = Telemetry.getKeyedHistogramById(KEYED_ID);
   for (let i=0; i<2; ++i) {
     let key = KEYS[i];
     h.add(key, true);
@@ -484,7 +471,6 @@ function test_keyed_boolean_histogram()
   testKeys.push(key);
   testSnapShot[key] = testHistograms[2];
   testSnapShot[key].sum = 0;
-  testSnapShot[key].sum_squares_lo = 0;
   testSnapShot[key].counts = [1, 0, 0];
   Assert.deepEqual(h.keys().sort(), testKeys);
   Assert.deepEqual(h.snapshot(), testSnapShot);
@@ -495,19 +481,16 @@ function test_keyed_boolean_histogram()
   h.clear();
   Assert.deepEqual(h.keys(), []);
   Assert.deepEqual(h.snapshot(), {});
-}
+});
 
-function test_keyed_count_histogram()
-{
-  const KEYED_ID = "test::keyed::count";
+add_task(function* test_keyed_count_histogram() {
+  const KEYED_ID = "TELEMETRY_TEST_KEYED_COUNT";
   const KEYS = numberRange(0, 5).map(i => "key" + (i + 1));
   let histogramBase = {
     "min": 1,
     "max": 2,
     "histogram_type": 4,
     "sum": 0,
-    "sum_squares_lo": 0,
-    "sum_squares_hi": 0,
     "ranges": [0, 1, 2],
     "counts": [1, 0, 0]
   };
@@ -515,7 +498,7 @@ function test_keyed_count_histogram()
   let testKeys = [];
   let testSnapShot = {};
 
-  let h = Telemetry.newKeyedHistogram(KEYED_ID, "never", Telemetry.HISTOGRAM_COUNT);
+  let h = Telemetry.getKeyedHistogramById(KEYED_ID);
   for (let i=0; i<4; ++i) {
     let key = KEYS[i];
     let value = i*2 + 1;
@@ -525,7 +508,6 @@ function test_keyed_count_histogram()
     }
     testHistograms[i].counts[0] = value;
     testHistograms[i].sum = value;
-    testHistograms[i].sum_squares_lo = value;
     testSnapShot[key] = testHistograms[i];
     testKeys.push(key);
 
@@ -543,7 +525,6 @@ function test_keyed_count_histogram()
   testKeys.push(key);
   testHistograms[4].counts[0] = 1;
   testHistograms[4].sum = 1;
-  testHistograms[4].sum_squares_lo = 1;
   testSnapShot[key] = testHistograms[4];
 
   Assert.deepEqual(h.keys().sort(), testKeys);
@@ -555,12 +536,11 @@ function test_keyed_count_histogram()
   h.clear();
   Assert.deepEqual(h.keys(), []);
   Assert.deepEqual(h.snapshot(), {});
-}
+});
 
-function test_keyed_flag_histogram()
-{
-  const KEYED_ID = "test::keyed::flag";
-  let h = Telemetry.newKeyedHistogram(KEYED_ID, "never", Telemetry.HISTOGRAM_FLAG);
+add_task(function* test_keyed_flag_histogram() {
+  const KEYED_ID = "TELEMETRY_TEST_KEYED_FLAG";
+  let h = Telemetry.getKeyedHistogramById(KEYED_ID);
 
   const KEY = "default";
   h.add(KEY, true);
@@ -571,8 +551,6 @@ function test_keyed_flag_histogram()
     "max": 2,
     "histogram_type": 3,
     "sum": 1,
-    "sum_squares_lo": 1,
-    "sum_squares_hi": 0,
     "ranges": [0, 1, 2],
     "counts": [0, 1, 0]
   };
@@ -586,9 +564,9 @@ function test_keyed_flag_histogram()
   h.clear();
   Assert.deepEqual(h.keys(), []);
   Assert.deepEqual(h.snapshot(), {});
-}
+});
 
-function test_keyed_histogram_recording() {
+add_task(function* test_keyed_histogram_recording() {
   // Check that no histogram is recorded if both base and extended recording are off.
   Telemetry.canRecordBase = false;
   Telemetry.canRecordExtended = false;
@@ -612,12 +590,6 @@ function test_keyed_histogram_recording() {
   Assert.equal(h.snapshot(TEST_KEY).sum, 0,
                "The keyed histograms should not record any data.");
 
-  // Runtime created histograms should not be recorded.
-  h = Telemetry.newKeyedHistogram("test::runtime_keyed_boolean", "never", Telemetry.HISTOGRAM_BOOLEAN);
-  h.add(TEST_KEY, 1);
-  Assert.equal(h.snapshot(TEST_KEY).sum, 0,
-               "The keyed histogram should not record any data.");
-
   // Check that extended histograms are recorded when required.
   Telemetry.canRecordExtended = true;
 
@@ -636,9 +608,9 @@ function test_keyed_histogram_recording() {
   h.clear();
   h.add(TEST_KEY, 1);
   Assert.equal(h.snapshot(TEST_KEY).sum, 1);
-}
+});
 
-function test_histogram_recording_enabled() {
+add_task(function* test_histogram_recording_enabled() {
   Telemetry.canRecordBase = true;
   Telemetry.canRecordExtended = true;
 
@@ -662,6 +634,11 @@ function test_histogram_recording_enabled() {
   Assert.equal(orig.sum + 2, h.snapshot().sum,
                "When recording is re-enabled add should record.");
 
+  // Check that we're correctly accumulating values other than 1.
+  h.clear();
+  h.add(3);
+  Assert.equal(3, h.snapshot().sum, "Recording counts greater than 1 should work.");
+
   // Check that a histogram with recording disabled by default behaves correctly
   h = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT_INIT_NO_RECORD");
   orig = h.snapshot();
@@ -680,16 +657,15 @@ function test_histogram_recording_enabled() {
   h.add(1);
   Assert.equal(orig.sum + 1, h.snapshot().sum,
                "When recording is disabled add should not record.");
+});
 
-}
-
-function test_keyed_histogram_recording_enabled() {
+add_task(function* test_keyed_histogram_recording_enabled() {
   Telemetry.canRecordBase = true;
   Telemetry.canRecordExtended = true;
 
   // Check RecordingEnabled for keyed histograms which are recording by default
   const TEST_KEY = "record_foo";
-  h = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_RELEASE_OPTOUT");
+  let h = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_RELEASE_OPTOUT");
 
   h.clear();
   h.add(TEST_KEY, 1);
@@ -725,39 +701,9 @@ function test_keyed_histogram_recording_enabled() {
   h.add(TEST_KEY, 1);
   Assert.equal(h.snapshot(TEST_KEY).sum, 1,
     "Keyed histogram add should not record when recording is disabled");
-}
+});
 
-function test_keyed_histogram() {
-  // Check that invalid names get rejected.
-
-  let threw = false;
-  try {
-    Telemetry.newKeyedHistogram("test::invalid # histogram", "never", Telemetry.HISTOGRAM_BOOLEAN);
-  } catch (e) {
-    // This should throw as we reject names with the # separator
-    threw = true;
-  }
-  Assert.ok(threw, "newKeyedHistogram should have thrown");
-
-  threw = false;
-  try {
-    Telemetry.getKeyedHistogramById("test::unknown histogram", "never", Telemetry.HISTOGRAM_BOOLEAN);
-  } catch (e) {
-    // This should throw as it is an unknown ID
-    threw = true;
-  }
-  Assert.ok(threw, "getKeyedHistogramById should have thrown");
-
-  // Check specific keyed histogram types working properly.
-
-  test_keyed_boolean_histogram();
-  test_keyed_count_histogram();
-  test_keyed_flag_histogram();
-  test_keyed_histogram_recording();
-}
-
-function test_datasets()
-{
+add_task(function* test_datasets() {
   // Check that datasets work as expected.
 
   const RELEASE_CHANNEL_OPTOUT = Ci.nsITelemetry.DATASET_RELEASE_CHANNEL_OPTOUT;
@@ -800,35 +746,12 @@ function test_datasets()
   registered = new Set(registered);
   Assert.ok(!registered.has("TELEMETRY_TEST_KEYED_FLAG"));
   Assert.ok(registered.has("TELEMETRY_TEST_KEYED_RELEASE_OPTOUT"));
-}
-
-function test_instantiate() {
-  if (gIsAndroid) {
-    // We don't support subsessions yet on Android.
-    return;
-  }
-
-  const ID = "TELEMETRY_TEST_COUNT";
-  let h = Telemetry.getHistogramById(ID);
-
-  // Instantiate the subsession histogram through |add| and make sure they match.
-  // This MUST be the first use of "TELEMETRY_TEST_COUNT" in this file, otherwise
-  // |add| will not instantiate the histogram.
-  h.add(1);
-  let snapshot = h.snapshot();
-  let subsession = Telemetry.snapshotSubsessionHistograms();
-  Assert.equal(snapshot.sum, subsession[ID].sum,
-               "Histogram and subsession histogram sum must match.");
-  // Clear the histogram, so we don't void the assumptions from the other tests.
-  h.clear();
-}
-
-function test_subsession() {
-  if (gIsAndroid) {
-    // We don't support subsessions yet on Android.
-    return;
-  }
+});
 
+add_task({
+  skip_if: () => gIsAndroid
+},
+function* test_subsession() {
   const ID = "TELEMETRY_TEST_COUNT";
   const FLAG = "TELEMETRY_TEST_FLAG";
   let h = Telemetry.getHistogramById(ID);
@@ -909,14 +832,12 @@ function test_subsession() {
   Assert.equal(subsession[ID].sum, 0);
   Assert.equal(snapshot[FLAG].sum, 1);
   Assert.equal(subsession[FLAG].sum, 0);
-}
-
-function test_keyed_subsession() {
-  if (gIsAndroid) {
-    // We don't support subsessions yet on Android.
-    return;
-  }
+});
 
+add_task({
+  skip_if: () => gIsAndroid
+},
+function* test_keyed_subsession() {
   let h = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_FLAG");
   const KEY = "foo";
 
@@ -959,50 +880,4 @@ function test_keyed_subsession() {
   subsession = h.subsessionSnapshot();
   Assert.ok(!(KEY in subsession));
   Assert.equal(h.subsessionSnapshot(KEY).sum, 0);
-}
-
-function generateUUID() {
-  let str = Cc["@mozilla.org/uuid-generator;1"].getService(Ci.nsIUUIDGenerator).generateUUID().toString();
-  // strip {}
-  return str.substring(1, str.length - 1);
-}
-
-function run_test()
-{
-  // This MUST be the very first test of this file.
-  test_instantiate();
-
-  let kinds = [Telemetry.HISTOGRAM_EXPONENTIAL, Telemetry.HISTOGRAM_LINEAR]
-  for (let histogram_type of kinds) {
-    let [min, max, bucket_count] = [1, INT_MAX - 1, 10]
-    test_histogram(histogram_type, "test::"+histogram_type, min, max, bucket_count);
-
-    const nh = Telemetry.newHistogram;
-    expect_fail(() => nh("test::min", "never", histogram_type, 0, max, bucket_count));
-    expect_fail(() => nh("test::bucket_count", "never", histogram_type, min, max, 1));
-  }
-
-  // Instantiate the storage for this histogram and make sure it doesn't
-  // get reflected into JS, as it has no interesting data in it.
-  let h = Telemetry.getHistogramById("NEWTAB_PAGE_PINNED_SITES_COUNT");
-  do_check_false("NEWTAB_PAGE_PINNED_SITES_COUNT" in Telemetry.histogramSnapshots);
-
-  test_boolean_histogram();
-  test_flag_histogram();
-  test_count_histogram();
-  test_getHistogramById();
-  test_histogramFrom();
-  test_getSlowSQL();
-  test_getWebrtc();
-  test_privateMode();
-  test_histogramRecording();
-  test_addons();
-  test_extended_stats();
-  test_expired_histogram();
-  test_keyed_histogram();
-  test_datasets();
-  test_subsession();
-  test_keyed_subsession();
-  test_histogram_recording_enabled();
-  test_keyed_histogram_recording_enabled();
-}
+});
