@@ -6,6 +6,7 @@
 // that aren't initialized outside of a XUL app environment like AddonManager
 // and the "@mozilla.org/xre/app-info;1" component.
 
+Components.utils.import("resource://gre/modules/AppConstants.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/Troubleshoot.jsm");
 
@@ -36,7 +37,7 @@ var tests = [
         ok(true, "The snapshot should conform to the schema.");
       }
       catch (err) {
-        ok(false, err);
+        ok(false, "Schema mismatch, " + err);
       }
       done();
     });
@@ -112,6 +113,10 @@ const SNAPSHOT_SCHEMA = {
           required: true,
           type: "string",
         },
+        osVersion: {
+          required: true,
+          type: "string",
+        },
         vendor: {
           type: "string",
         },
@@ -125,6 +130,9 @@ const SNAPSHOT_SCHEMA = {
           type: "boolean",
           required: true,
         },
+        autoStartStatus: {
+          type: "number",
+        },
         numTotalWindows: {
           type: "number",
         },
@@ -221,6 +229,9 @@ const SNAPSHOT_SCHEMA = {
         supportsHardwareH264: {
           type: "string",
         },
+        currentAudioBackend: {
+          type: "string",
+        },
         numAcceleratedWindowsMessage: {
           type: "array",
         },
@@ -290,6 +301,9 @@ const SNAPSHOT_SCHEMA = {
         webglRenderer: {
           type: "string",
         },
+        webgl2Renderer: {
+          type: "string",
+        },
         info: {
           type: "object",
         },
@@ -299,10 +313,19 @@ const SNAPSHOT_SCHEMA = {
             type: "string",
           },
         },
-        direct2DEnabledMessage: {
+        indices: {
+          type: "array",
+          items: {
+            type: "number",
+          },
+        },
+        featureLog: {
+          type: "object",
+        },
+        crashGuards: {
           type: "array",
         },
-        webglRendererMessage: {
+        direct2DEnabledMessage: {
           type: "array",
         },
       },
@@ -423,19 +446,19 @@ const SNAPSHOT_SCHEMA = {
       type: "object",
       properties: {
         hasSeccompBPF: {
-          required: true,
+          required: AppConstants.platform == "linux",
           type: "boolean"
         },
         hasSeccompTSync: {
-          required: true,
+          required: AppConstants.platform == "linux",
           type: "boolean"
         },
         hasUserNamespaces: {
-          required: true,
+          required: AppConstants.platform == "linux",
           type: "boolean"
         },
         hasPrivilegedUserNamespaces: {
-          required: true,
+          required: AppConstants.platform == "linux",
           type: "boolean"
         },
         canSandboxContent: {
@@ -446,6 +469,10 @@ const SNAPSHOT_SCHEMA = {
           required: false,
           type: "boolean"
         },
+        contentSandboxLevel: {
+          required: AppConstants.MOZ_CONTENT_SANDBOX,
+          type: "number"
+        },
       },
     },
   },
@@ -483,7 +510,7 @@ function validateObject_object(obj, schema) {
   // Now check that the object doesn't have any properties not in the schema.
   for (let prop in obj)
     if (!(prop in schema.properties))
-      throw validationErr("Object has property not in schema", obj, schema);
+      throw validationErr("Object has property "+prop+" not in schema", obj, schema);
 }
 
 function validateObject_array(array, schema) {
