@@ -3,11 +3,12 @@
 
 "use strict";
 
-const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
+var {classes: Cc, interfaces: Ci, utils: Cu} = Components;
 
 do_get_profile();
 
 Cu.import("resource://gre/modules/Promise.jsm");
+Cu.import("resource://gre/modules/PromiseUtils.jsm");
 Cu.import("resource://gre/modules/osfile.jsm");
 Cu.import("resource://gre/modules/FileUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
@@ -33,17 +34,23 @@ function sleep(ms) {
   return deferred.promise;
 }
 
+// When testing finalization, use this to tell Sqlite.jsm to not throw
+// an uncatchable `Promise.reject`
+function failTestsOnAutoClose(enabled)  {
+  Cu.getGlobalForObject(Sqlite).Debugging.failTestsOnAutoClose = enabled;
+}
+
 function getConnection(dbName, extraOptions={}) {
   let path = dbName + ".sqlite";
   let options = {path: path};
-  for (let [k, v] in Iterator(extraOptions)) {
+  for (let [k, v] of Object.entries(extraOptions)) {
     options[k] = v;
   }
 
   return Sqlite.openConnection(options);
 }
 
-function getDummyDatabase(name, extraOptions={}) {
+function* getDummyDatabase(name, extraOptions={}) {
   const TABLES = {
     dirs: "id INTEGER PRIMARY KEY AUTOINCREMENT, path TEXT",
     files: "id INTEGER PRIMARY KEY AUTOINCREMENT, dir_id INTEGER, path TEXT",
@@ -52,15 +59,15 @@ function getDummyDatabase(name, extraOptions={}) {
   let c = yield getConnection(name, extraOptions);
   c._initialStatementCount = 0;
 
-  for (let [k, v] in Iterator(TABLES)) {
+  for (let [k, v] of Object.entries(TABLES)) {
     yield c.execute("CREATE TABLE " + k + "(" + v + ")");
     c._initialStatementCount++;
   }
 
-  throw new Task.Result(c);
+  return c;
 }
 
-function getDummyTempDatabase(name, extraOptions={}) {
+function* getDummyTempDatabase(name, extraOptions={}) {
   const TABLES = {
     dirs: "id INTEGER PRIMARY KEY AUTOINCREMENT, path TEXT",
     files: "id INTEGER PRIMARY KEY AUTOINCREMENT, dir_id INTEGER, path TEXT",
@@ -69,41 +76,41 @@ function getDummyTempDatabase(name, extraOptions={}) {
   let c = yield getConnection(name, extraOptions);
   c._initialStatementCount = 0;
 
-  for (let [k, v] in Iterator(TABLES)) {
+  for (let [k, v] of Object.entries(TABLES)) {
     yield c.execute("CREATE TEMP TABLE " + k + "(" + v + ")");
     c._initialStatementCount++;
   }
 
-  throw new Task.Result(c);
+  return c;
 }
 
 function run_test() {
-  Cu.import("resource://testing-common/services-common/logging.js");
+  Cu.import("resource://testing-common/services/common/logging.js");
   initTestLogging("Trace");
 
   run_next_test();
 }
 
-add_task(function test_open_normal() {
+add_task(function* test_open_normal() {
   let c = yield Sqlite.openConnection({path: "test_open_normal.sqlite"});
   yield c.close();
 });
 
-add_task(function test_open_unshared() {
+add_task(function* test_open_unshared() {
   let path = OS.Path.join(OS.Constants.Path.profileDir, "test_open_unshared.sqlite");
 
   let c = yield Sqlite.openConnection({path: path, sharedMemoryCache: false});
   yield c.close();
 });
 
-add_task(function test_get_dummy_database() {
+add_task(function* test_get_dummy_database() {
   let db = yield getDummyDatabase("get_dummy_database");
 
   do_check_eq(typeof(db), "object");
   yield db.close();
 });
 
-add_task(function test_schema_version() {
+add_task(function* test_schema_version() {
   let db = yield getDummyDatabase("schema_version");
 
   let version = yield db.getSchemaVersion();
@@ -119,7 +126,9 @@ add_task(function test_schema_version() {
       yield db.setSchemaVersion(v);
       do_print("Schema version " + v + " should have been rejected");
       success = false;
-    } catch (ex if ex.message.startsWith("Schema version must be an integer.")) {
+    } catch (ex) {
+      if (!ex.message.startsWith("Schema version must be an integer."))
+        throw ex;
       success = true;
     }
     do_check_true(success);
@@ -131,7 +140,7 @@ add_task(function test_schema_version() {
   yield db.close();
 });
 
-add_task(function test_simple_insert() {
+add_task(function* test_simple_insert() {
   let c = yield getDummyDatabase("simple_insert");
 
   let result = yield c.execute("INSERT INTO dirs VALUES (NULL, 'foo')");
@@ -140,7 +149,7 @@ add_task(function test_simple_insert() {
   yield c.close();
 });
 
-add_task(function test_simple_bound_array() {
+add_task(function* test_simple_bound_array() {
   let c = yield getDummyDatabase("simple_bound_array");
 
   let result = yield c.execute("INSERT INTO dirs VALUES (?, ?)", [1, "foo"]);
@@ -148,7 +157,7 @@ add_task(function test_simple_bound_array() {
   yield c.close();
 });
 
-add_task(function test_simple_bound_object() {
+add_task(function* test_simple_bound_object() {
   let c = yield getDummyDatabase("simple_bound_object");
   let result = yield c.execute("INSERT INTO dirs VALUES (:id, :path)",
                                {id: 1, path: "foo"});
@@ -161,7 +170,7 @@ add_task(function test_simple_bound_object() {
 });
 
 // This is mostly a sanity test to ensure simple executions work.
-add_task(function test_simple_insert_then_select() {
+add_task(function* test_simple_insert_then_select() {
   let c = yield getDummyDatabase("simple_insert_then_select");
 
   yield c.execute("INSERT INTO dirs VALUES (NULL, 'foo')");
@@ -184,7 +193,7 @@ add_task(function test_simple_insert_then_select() {
   yield c.close();
 });
 
-add_task(function test_repeat_execution() {
+add_task(function* test_repeat_execution() {
   let c = yield getDummyDatabase("repeat_execution");
 
   let sql = "INSERT INTO dirs (path) VALUES (:path)";
@@ -198,7 +207,7 @@ add_task(function test_repeat_execution() {
   yield c.close();
 });
 
-add_task(function test_table_exists() {
+add_task(function* test_table_exists() {
   let c = yield getDummyDatabase("table_exists");
 
   do_check_false(yield c.tableExists("does_not_exist"));
@@ -208,7 +217,7 @@ add_task(function test_table_exists() {
   yield c.close();
 });
 
-add_task(function test_index_exists() {
+add_task(function* test_index_exists() {
   let c = yield getDummyDatabase("index_exists");
 
   do_check_false(yield c.indexExists("does_not_exist"));
@@ -219,7 +228,7 @@ add_task(function test_index_exists() {
   yield c.close();
 });
 
-add_task(function test_temp_table_exists() {
+add_task(function* test_temp_table_exists() {
   let c = yield getDummyTempDatabase("temp_table_exists");
 
   do_check_false(yield c.tableExists("temp_does_not_exist"));
@@ -229,7 +238,7 @@ add_task(function test_temp_table_exists() {
   yield c.close();
 });
 
-add_task(function test_temp_index_exists() {
+add_task(function* test_temp_index_exists() {
   let c = yield getDummyTempDatabase("temp_index_exists");
 
   do_check_false(yield c.indexExists("temp_does_not_exist"));
@@ -240,7 +249,7 @@ add_task(function test_temp_index_exists() {
   yield c.close();
 });
 
-add_task(function test_close_cached() {
+add_task(function* test_close_cached() {
   let c = yield getDummyDatabase("close_cached");
 
   yield c.executeCached("SELECT * FROM dirs");
@@ -249,12 +258,12 @@ add_task(function test_close_cached() {
   yield c.close();
 });
 
-add_task(function test_execute_invalid_statement() {
+add_task(function* test_execute_invalid_statement() {
   let c = yield getDummyDatabase("invalid_statement");
 
   let deferred = Promise.defer();
 
-  do_check_eq(c._anonymousStatements.size, 0);
+  do_check_eq(c._connectionData._anonymousStatements.size, 0);
 
   c.execute("SELECT invalid FROM unknown").then(do_throw, function onError(error) {
     deferred.resolve();
@@ -263,12 +272,21 @@ add_task(function test_execute_invalid_statement() {
   yield deferred.promise;
 
   // Ensure we don't leak the statement instance.
-  do_check_eq(c._anonymousStatements.size, 0);
+  do_check_eq(c._connectionData._anonymousStatements.size, 0);
 
   yield c.close();
 });
 
-add_task(function test_on_row_exception_ignored() {
+add_task(function* test_incorrect_like_bindings() {
+  let c = yield getDummyDatabase("incorrect_like_bindings");
+
+  let sql = "select * from dirs where path LIKE 'non%'";
+  Assert.throws(() => c.execute(sql), /Please enter a LIKE clause/);
+  Assert.throws(() => c.executeCached(sql), /Please enter a LIKE clause/);
+
+  yield c.close();
+});
+add_task(function* test_on_row_exception_ignored() {
   let c = yield getDummyDatabase("on_row_exception_ignored");
 
   let sql = "INSERT INTO dirs (path) VALUES (?)";
@@ -277,19 +295,20 @@ add_task(function test_on_row_exception_ignored() {
   }
 
   let i = 0;
-  yield c.execute("SELECT * FROM DIRS", null, function onRow(row) {
+  let hasResult = yield c.execute("SELECT * FROM DIRS", null, function onRow(row) {
     i++;
 
     throw new Error("Some silly error.");
   });
 
+  do_check_eq(hasResult, true);
   do_check_eq(i, 10);
 
   yield c.close();
 });
 
 // Ensure StopIteration during onRow causes processing to stop.
-add_task(function test_on_row_stop_iteration() {
+add_task(function* test_on_row_stop_iteration() {
   let c = yield getDummyDatabase("on_row_stop_iteration");
 
   let sql = "INSERT INTO dirs (path) VALUES (?)";
@@ -298,7 +317,7 @@ add_task(function test_on_row_stop_iteration() {
   }
 
   let i = 0;
-  let result = yield c.execute("SELECT * FROM dirs", null, function onRow(row) {
+  let hasResult = yield c.execute("SELECT * FROM dirs", null, function onRow(row) {
     i++;
 
     if (i == 5) {
@@ -306,34 +325,43 @@ add_task(function test_on_row_stop_iteration() {
     }
   });
 
-  do_check_null(result);
+  do_check_eq(hasResult, true);
   do_check_eq(i, 5);
 
   yield c.close();
 });
 
-add_task(function test_invalid_transaction_type() {
-  let c = yield getDummyDatabase("invalid_transaction_type");
+// Ensure execute resolves to false when no rows are selected.
+add_task(function* test_on_row_stop_iteration() {
+  let c = yield getDummyDatabase("no_on_row");
 
-  let errored = false;
-  try {
-    c.executeTransaction(function () {}, "foobar");
-  } catch (ex) {
-    errored = true;
-    do_check_true(ex.message.startsWith("Unknown transaction type"));
-  } finally {
-    do_check_true(errored);
-  }
+  let i = 0;
+  let hasResult = yield c.execute(`SELECT * FROM dirs WHERE path="nonexistent"`, null, function onRow(row) {
+    i++;
+  });
+
+  do_check_eq(hasResult, false);
+  do_check_eq(i, 0);
 
   yield c.close();
 });
 
-add_task(function test_execute_transaction_success() {
+add_task(function* test_invalid_transaction_type() {
+  let c = yield getDummyDatabase("invalid_transaction_type");
+
+  Assert.throws(() => c.executeTransaction(function* () {}, "foobar"),
+                /Unknown transaction type/,
+                "Unknown transaction type should throw");
+
+  yield c.close();
+});
+
+add_task(function* test_execute_transaction_success() {
   let c = yield getDummyDatabase("execute_transaction_success");
 
   do_check_false(c.transactionInProgress);
 
-  yield c.executeTransaction(function transaction(conn) {
+  yield c.executeTransaction(function* transaction(conn) {
     do_check_eq(c, conn);
     do_check_true(conn.transactionInProgress);
 
@@ -348,12 +376,12 @@ add_task(function test_execute_transaction_success() {
   yield c.close();
 });
 
-add_task(function test_execute_transaction_rollback() {
+add_task(function* test_execute_transaction_rollback() {
   let c = yield getDummyDatabase("execute_transaction_rollback");
 
   let deferred = Promise.defer();
 
-  c.executeTransaction(function transaction(conn) {
+  c.executeTransaction(function* transaction(conn) {
     yield conn.execute("INSERT INTO dirs (path) VALUES ('foo')");
     print("Expecting error with next statement.");
     yield conn.execute("INSERT INTO invalid VALUES ('foo')");
@@ -372,23 +400,19 @@ add_task(function test_execute_transaction_rollback() {
   yield c.close();
 });
 
-add_task(function test_close_during_transaction() {
+add_task(function* test_close_during_transaction() {
   let c = yield getDummyDatabase("close_during_transaction");
 
   yield c.execute("INSERT INTO dirs (path) VALUES ('foo')");
 
-  let errored = false;
-  try {
-    yield c.executeTransaction(function transaction(conn) {
-      yield c.execute("INSERT INTO dirs (path) VALUES ('bar')");
-      yield c.close();
-    });
-  } catch (ex) {
-    errored = true;
-    do_check_eq(ex.message, "Connection being closed.");
-  } finally {
-    do_check_true(errored);
-  }
+  let promise = c.executeTransaction(function* transaction(conn) {
+    yield c.execute("INSERT INTO dirs (path) VALUES ('bar')");
+  });
+  yield c.close();
+
+  yield Assert.rejects(promise,
+                       /Transaction canceled due to a closed connection/,
+                       "closing a connection in the middle of a transaction should reject it");
 
   let c2 = yield getConnection("close_during_transaction");
   let rows = yield c2.execute("SELECT * FROM dirs");
@@ -397,32 +421,66 @@ add_task(function test_close_during_transaction() {
   yield c2.close();
 });
 
-add_task(function test_detect_multiple_transactions() {
+// Verify that we support concurrent transactions.
+add_task(function* test_multiple_transactions() {
   let c = yield getDummyDatabase("detect_multiple_transactions");
 
-  yield c.executeTransaction(function main() {
-    yield c.execute("INSERT INTO dirs (path) VALUES ('foo')");
-
-    let errored = false;
-    try {
-      yield c.executeTransaction(function child() {
-        yield c.execute("INSERT INTO dirs (path) VALUES ('bar')");
-      });
-    } catch (ex) {
-      errored = true;
-      do_check_true(ex.message.startsWith("A transaction is already active."));
-    } finally {
-      do_check_true(errored);
-    }
-  });
+  for (let i = 0; i < 10; ++i) {
+    // We don't wait for these transactions.
+    c.executeTransaction(function* () {
+      yield c.execute("INSERT INTO dirs (path) VALUES (:path)",
+                      { path: `foo${i}` });
+      yield c.execute("SELECT * FROM dirs");
+    });
+  }
+  for (let i = 0; i < 10; ++i) {
+    yield c.executeTransaction(function* () {
+      yield c.execute("INSERT INTO dirs (path) VALUES (:path)",
+                      { path: `bar${i}` });
+      yield c.execute("SELECT * FROM dirs");
+    });
+  }
 
   let rows = yield c.execute("SELECT * FROM dirs");
-  do_check_eq(rows.length, 1);
+  do_check_eq(rows.length, 20);
 
   yield c.close();
 });
 
-add_task(function test_shrink_memory() {
+// Verify that wrapped transactions ignore a BEGIN TRANSACTION failure, when
+// an externally opened transaction exists.
+add_task(function* test_wrapped_connection_transaction() {
+  let file = new FileUtils.File(OS.Path.join(OS.Constants.Path.profileDir,
+                                             "test_wrapStorageConnection.sqlite"));
+  let c = yield new Promise((resolve, reject) => {
+    Services.storage.openAsyncDatabase(file, null, (status, db) => {
+      if (Components.isSuccessCode(status)) {
+        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+      } else {
+        reject(new Error(status));
+      }
+    });
+  });
+
+  let wrapper = yield Sqlite.wrapStorageConnection({ connection: c });
+  // Start a transaction on the raw connection.
+  yield c.executeSimpleSQLAsync("BEGIN");
+  // Now use executeTransaction, it will be executed, but not in a transaction.
+  yield wrapper.executeTransaction(function* () {
+    yield wrapper.execute("CREATE TABLE test (id INTEGER PRIMARY KEY AUTOINCREMENT)");
+  });
+  // This should not fail cause the internal transaction has not been created.
+  yield c.executeSimpleSQLAsync("COMMIT");
+
+  yield wrapper.execute("SELECT * FROM test");
+
+  // Closing the wrapper should just finalize statements but not close the
+  // database.
+  yield wrapper.close();
+  yield c.asyncClose();
+});
+
+add_task(function* test_shrink_memory() {
   let c = yield getDummyDatabase("shrink_memory");
 
   // It's just a simple sanity test. We have no way of measuring whether this
@@ -432,13 +490,13 @@ add_task(function test_shrink_memory() {
   yield c.close();
 });
 
-add_task(function test_no_shrink_on_init() {
+add_task(function* test_no_shrink_on_init() {
   let c = yield getConnection("no_shrink_on_init",
                               {shrinkMemoryOnConnectionIdleMS: 200});
 
-  let oldShrink = c.shrinkMemory;
+  let oldShrink = c._connectionData.shrinkMemory;
   let count = 0;
-  Object.defineProperty(c, "shrinkMemory", {
+  Object.defineProperty(c._connectionData, "shrinkMemory", {
     value: function () {
       count++;
     },
@@ -455,19 +513,19 @@ add_task(function test_no_shrink_on_init() {
   yield c.close();
 });
 
-add_task(function test_idle_shrink_fires() {
+add_task(function* test_idle_shrink_fires() {
   let c = yield getDummyDatabase("idle_shrink_fires",
                                  {shrinkMemoryOnConnectionIdleMS: 200});
-  c._clearIdleShrinkTimer();
+  c._connectionData._clearIdleShrinkTimer();
 
-  let oldShrink = c.shrinkMemory;
+  let oldShrink = c._connectionData.shrinkMemory;
   let shrinkPromises = [];
 
   let count = 0;
-  Object.defineProperty(c, "shrinkMemory", {
+  Object.defineProperty(c._connectionData, "shrinkMemory", {
     value: function () {
       count++;
-      let promise = oldShrink.call(c);
+      let promise = oldShrink.call(c._connectionData);
       shrinkPromises.push(promise);
       return promise;
     },
@@ -475,7 +533,7 @@ add_task(function test_idle_shrink_fires() {
 
   // We reset the idle shrink timer after monkeypatching because otherwise the
   // installed timer callback will reference the non-monkeypatched function.
-  c._startIdleShrinkTimer();
+  c._connectionData._startIdleShrinkTimer();
 
   yield sleep(220);
   do_check_eq(count, 1);
@@ -497,28 +555,28 @@ add_task(function test_idle_shrink_fires() {
   yield c.close();
 });
 
-add_task(function test_idle_shrink_reset_on_operation() {
+add_task(function* test_idle_shrink_reset_on_operation() {
   const INTERVAL = 500;
   let c = yield getDummyDatabase("idle_shrink_reset_on_operation",
                                  {shrinkMemoryOnConnectionIdleMS: INTERVAL});
 
-  c._clearIdleShrinkTimer();
+  c._connectionData._clearIdleShrinkTimer();
 
-  let oldShrink = c.shrinkMemory;
+  let oldShrink = c._connectionData.shrinkMemory;
   let shrinkPromises = [];
   let count = 0;
 
-  Object.defineProperty(c, "shrinkMemory", {
+  Object.defineProperty(c._connectionData, "shrinkMemory", {
     value: function () {
       count++;
-      let promise = oldShrink.call(c);
+      let promise = oldShrink.call(c._connectionData);
       shrinkPromises.push(promise);
       return promise;
     },
   });
 
   let now = new Date();
-  c._startIdleShrinkTimer();
+  c._connectionData._startIdleShrinkTimer();
 
   let initialIdle = new Date(now.getTime() + INTERVAL);
 
@@ -545,13 +603,13 @@ add_task(function test_idle_shrink_reset_on_operation() {
   yield c.close();
 });
 
-add_task(function test_in_progress_counts() {
+add_task(function* test_in_progress_counts() {
   let c = yield getDummyDatabase("in_progress_counts");
-  do_check_eq(c._statementCounter, c._initialStatementCount);
-  do_check_eq(c._pendingStatements.size, 0);
+  do_check_eq(c._connectionData._statementCounter, c._initialStatementCount);
+  do_check_eq(c._connectionData._pendingStatements.size, 0);
   yield c.executeCached("INSERT INTO dirs (path) VALUES ('foo')");
-  do_check_eq(c._statementCounter, c._initialStatementCount + 1);
-  do_check_eq(c._pendingStatements.size, 0);
+  do_check_eq(c._connectionData._statementCounter, c._initialStatementCount + 1);
+  do_check_eq(c._connectionData._pendingStatements.size, 0);
 
   let expectOne;
   let expectTwo;
@@ -569,12 +627,12 @@ add_task(function test_in_progress_counts() {
   yield c.executeCached("SELECT * from dirs", null, function onRow() {
     // In the onRow handler, we're still an outstanding query.
     // Expect a single in-progress entry.
-    expectOne = c._pendingStatements.size;
+    expectOne = c._connectionData._pendingStatements.size;
 
     // Start another query, checking that after its statement has been created
     // there are two statements in progress.
     let p = c.executeCached("SELECT 10, path from dirs");
-    expectTwo = c._pendingStatements.size;
+    expectTwo = c._connectionData._pendingStatements.size;
 
     // Now wait for it to be done before we return from the row handler …
     p.then(function onInner() {
@@ -592,13 +650,13 @@ add_task(function test_in_progress_counts() {
 
   do_check_eq(expectOne, 1);
   do_check_eq(expectTwo, 2);
-  do_check_eq(c._statementCounter, c._initialStatementCount + 3);
-  do_check_eq(c._pendingStatements.size, 0);
+  do_check_eq(c._connectionData._statementCounter, c._initialStatementCount + 3);
+  do_check_eq(c._connectionData._pendingStatements.size, 0);
 
   yield c.close();
 });
 
-add_task(function test_discard_while_active() {
+add_task(function* test_discard_while_active() {
   let c = yield getDummyDatabase("discard_while_active");
 
   yield c.executeCached("INSERT INTO dirs (path) VALUES ('foo')");
@@ -624,25 +682,25 @@ add_task(function test_discard_while_active() {
   yield c.close();
 });
 
-add_task(function test_discard_cached() {
+add_task(function* test_discard_cached() {
   let c = yield getDummyDatabase("discard_cached");
 
   yield c.executeCached("SELECT * from dirs");
-  do_check_eq(1, c._cachedStatements.size);
+  do_check_eq(1, c._connectionData._cachedStatements.size);
 
   yield c.executeCached("SELECT * from files");
-  do_check_eq(2, c._cachedStatements.size);
+  do_check_eq(2, c._connectionData._cachedStatements.size);
 
   yield c.executeCached("SELECT * from dirs");
-  do_check_eq(2, c._cachedStatements.size);
+  do_check_eq(2, c._connectionData._cachedStatements.size);
 
   c.discardCachedStatements();
-  do_check_eq(0, c._cachedStatements.size);
+  do_check_eq(0, c._connectionData._cachedStatements.size);
 
   yield c.close();
 });
 
-add_task(function test_programmatic_binding() {
+add_task(function* test_programmatic_binding() {
   let c = yield getDummyDatabase("programmatic_binding");
 
   let bindings = [
@@ -660,7 +718,7 @@ add_task(function test_programmatic_binding() {
   yield c.close();
 });
 
-add_task(function test_programmatic_binding_transaction() {
+add_task(function* test_programmatic_binding_transaction() {
   let c = yield getDummyDatabase("programmatic_binding_transaction");
 
   let bindings = [
@@ -670,7 +728,7 @@ add_task(function test_programmatic_binding_transaction() {
   ];
 
   let sql = "INSERT INTO dirs VALUES (:id, :path)";
-  yield c.executeTransaction(function transaction() {
+  yield c.executeTransaction(function* transaction() {
     let result = yield c.execute(sql, bindings);
     do_check_eq(result.length, 0);
 
@@ -684,7 +742,7 @@ add_task(function test_programmatic_binding_transaction() {
   yield c.close();
 });
 
-add_task(function test_programmatic_binding_transaction_partial_rollback() {
+add_task(function* test_programmatic_binding_transaction_partial_rollback() {
   let c = yield getDummyDatabase("programmatic_binding_transaction_partial_rollback");
 
   let bindings = [
@@ -699,13 +757,13 @@ add_task(function test_programmatic_binding_transaction_partial_rollback() {
 
   let secondSucceeded = false;
   try {
-    yield c.executeTransaction(function transaction() {
+    yield c.executeTransaction(function* transaction() {
       // Insert one row. This won't implicitly start a transaction.
       let result = yield c.execute(sql, bindings[0]);
 
       // Insert multiple rows. mozStorage will want to start a transaction.
       // One of the inserts will fail, so the transaction should be rolled back.
-      let result = yield c.execute(sql, bindings);
+      result = yield c.execute(sql, bindings);
       secondSucceeded = true;
     });
   } catch (ex) {
@@ -723,11 +781,9 @@ add_task(function test_programmatic_binding_transaction_partial_rollback() {
   yield c.close();
 });
 
-/**
- * Just like the previous test, but relying on the implicit
- * transaction established by mozStorage.
- */
-add_task(function test_programmatic_binding_implicit_transaction() {
+// Just like the previous test, but relying on the implicit
+// transaction established by mozStorage.
+add_task(function* test_programmatic_binding_implicit_transaction() {
   let c = yield getDummyDatabase("programmatic_binding_implicit_transaction");
 
   let bindings = [
@@ -754,11 +810,9 @@ add_task(function test_programmatic_binding_implicit_transaction() {
   yield c.close();
 });
 
-/**
- * Test that direct binding of params and execution through mozStorage doesn't
- * error when we manually create a transaction. See Bug 856925.
- */
-add_task(function test_direct() {
+// Test that direct binding of params and execution through mozStorage doesn't
+// error when we manually create a transaction. See Bug 856925.
+add_task(function* test_direct() {
   let file = FileUtils.getFile("TmpD", ["test_direct.sqlite"]);
   file.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, FileUtils.PERMS_FILE);
   print("Opening " + file.path);
@@ -833,18 +887,16 @@ add_task(function test_direct() {
   yield deferred.promise;
 });
 
-/**
- * Test Sqlite.cloneStorageConnection.
- */
+// Test Sqlite.cloneStorageConnection.
 add_task(function* test_cloneStorageConnection() {
   let file = new FileUtils.File(OS.Path.join(OS.Constants.Path.profileDir,
                                              "test_cloneStorageConnection.sqlite"));
-  let c = yield new Promise((success, failure) => {
+  let c = yield new Promise((resolve, reject) => {
     Services.storage.openAsyncDatabase(file, null, (status, db) => {
       if (Components.isSuccessCode(status)) {
-        success(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
       } else {
-        failure(new Error(status));
+        reject(new Error(status));
       }
     });
   });
@@ -863,19 +915,19 @@ add_task(function* test_cloneStorageConnection() {
   yield clone.close();
 });
 
-/**
- * Test Sqlite.cloneStorageConnection invalid argument.
- */
+// Test Sqlite.cloneStorageConnection invalid argument.
 add_task(function* test_cloneStorageConnection() {
   try {
     let clone = yield Sqlite.cloneStorageConnection({ connection: null });
     do_throw(new Error("Should throw on invalid connection"));
-  } catch (ex if ex.name == "TypeError") {}
+  } catch (ex) {
+    if (ex.name != "TypeError") {
+      throw ex;
+    }
+  }
 });
 
-/**
- * Test clone() method.
- */
+// Test clone() method.
 add_task(function* test_clone() {
   let c = yield getDummyDatabase("clone");
 
@@ -887,9 +939,7 @@ add_task(function* test_clone() {
   yield clone.close();
 });
 
-/**
- * Test clone(readOnly) method.
- */
+// Test clone(readOnly) method.
 add_task(function* test_readOnly_clone() {
   let path = OS.Path.join(OS.Constants.Path.profileDir, "test_readOnly_clone.sqlite");
   let c = yield Sqlite.openConnection({path: path, sharedMemoryCache: false});
@@ -898,11 +948,147 @@ add_task(function* test_readOnly_clone() {
   // Just check that it works.
   yield clone.execute("SELECT 1");
   // But should not be able to write.
-  try {
-    yield clone.execute("CREATE TABLE test (id INTEGER PRIMARY KEY)");
-    do_throw(new Error("Should not be able to write to a read-only clone."));
-  } catch (ex) {}
+
+  yield Assert.rejects(clone.execute("CREATE TABLE test (id INTEGER PRIMARY KEY)"),
+                       /readonly/);
   // Closing order should not matter.
   yield c.close();
   yield clone.close();
 });
+
+// Test Sqlite.wrapStorageConnection.
+add_task(function* test_wrapStorageConnection() {
+  let file = new FileUtils.File(OS.Path.join(OS.Constants.Path.profileDir,
+                                             "test_wrapStorageConnection.sqlite"));
+  let c = yield new Promise((resolve, reject) => {
+    Services.storage.openAsyncDatabase(file, null, (status, db) => {
+      if (Components.isSuccessCode(status)) {
+        resolve(db.QueryInterface(Ci.mozIStorageAsyncConnection));
+      } else {
+        reject(new Error(status));
+      }
+    });
+  });
+
+  let wrapper = yield Sqlite.wrapStorageConnection({ connection: c });
+  // Just check that it works.
+  yield wrapper.execute("SELECT 1");
+  yield wrapper.executeCached("SELECT 1");
+
+  // Closing the wrapper should just finalize statements but not close the
+  // database.
+  yield wrapper.close();
+  yield c.asyncClose();
+});
+
+// Test finalization
+add_task(function* test_closed_by_witness() {
+  failTestsOnAutoClose(false);
+  let c = yield getDummyDatabase("closed_by_witness");
+
+  Services.obs.notifyObservers(null, "sqlite-finalization-witness",
+                               c._connectionData._identifier);
+  // Since we triggered finalization ourselves, tell the witness to
+  // forget the connection so it does not trigger a finalization again
+  c._witness.forget();
+  yield c._connectionData._deferredClose.promise;
+  do_check_false(c._connectionData._open);
+  failTestsOnAutoClose(true);
+});
+
+add_task(function* test_warning_message_on_finalization() {
+  failTestsOnAutoClose(false);
+  let c = yield getDummyDatabase("warning_message_on_finalization");
+  let identifier = c._connectionData._identifier;
+  let deferred = Promise.defer();
+
+  let listener = {
+    observe: function(msg) {
+      let messageText = msg.message;
+      // Make sure the message starts with a warning containing the
+      // connection identifier
+      if (messageText.indexOf("Warning: Sqlite connection '" + identifier + "'") !== -1) {
+        deferred.resolve();
+      }
+    }
+  };
+  Services.console.registerListener(listener);
+
+  Services.obs.notifyObservers(null, "sqlite-finalization-witness", identifier);
+  // Since we triggered finalization ourselves, tell the witness to
+  // forget the connection so it does not trigger a finalization again
+  c._witness.forget();
+
+  yield deferred.promise;
+  Services.console.unregisterListener(listener);
+  failTestsOnAutoClose(true);
+});
+
+add_task(function* test_error_message_on_unknown_finalization() {
+  failTestsOnAutoClose(false);
+  let deferred = Promise.defer();
+
+  let listener = {
+    observe: function(msg) {
+      let messageText = msg.message;
+      if (messageText.indexOf("Error: Attempt to finalize unknown " +
+                              "Sqlite connection: foo") !== -1) {
+        deferred.resolve();
+      }
+    }
+  };
+  Services.console.registerListener(listener);
+  Services.obs.notifyObservers(null, "sqlite-finalization-witness", "foo");
+
+  yield deferred.promise;
+  Services.console.unregisterListener(listener);
+  failTestsOnAutoClose(true);
+});
+
+add_task(function* test_forget_witness_on_close() {
+  let c = yield getDummyDatabase("forget_witness_on_close");
+
+  let forgetCalled = false;
+  let oldWitness = c._witness;
+  c._witness = {
+    forget: function () {
+      forgetCalled = true;
+      oldWitness.forget();
+    },
+  };
+
+  yield c.close();
+  // After close, witness should have forgotten the connection
+  do_check_true(forgetCalled);
+});
+
+add_task(function* test_close_database_on_gc() {
+  failTestsOnAutoClose(false);
+  let finalPromise;
+
+  {
+    let collectedPromises = [];
+    for (let i = 0; i < 100; ++i) {
+      let deferred = PromiseUtils.defer();
+      let c = yield getDummyDatabase("gc_" + i);
+      c._connectionData._deferredClose.promise.then(deferred.resolve);
+      collectedPromises.push(deferred.promise);
+    }
+    finalPromise = Promise.all(collectedPromises);
+  }
+
+  // Call getDummyDatabase once more to clear any remaining
+  // references. This is needed at the moment, otherwise
+  // garbage-collection takes place after the shutdown barrier and the
+  // test will timeout. Once that is fixed, we can remove this line
+  // and be fine as long as the connections are garbage-collected.
+  let last = yield getDummyDatabase("gc_last");
+  yield last.close();
+
+  Components.utils.forceGC();
+  Components.utils.forceCC();
+  Components.utils.forceShrinkingGC();
+
+  yield finalPromise;
+  failTestsOnAutoClose(true);
+});
