@@ -21,6 +21,6 @@ function run_test()
   // Pretend we're a download status bar also asking for a time left, but we're
   // using a different "last sec". We need to make sure we get the same time.
   let lastSec = 314;
-  for (let [time, text] in Iterator(downloadTimes))
+  for (let [time, text] of Object.entries(downloadTimes))
     do_check_eq(DownloadUtils.getTimeLeft(time, lastSec)[0], text);
 }
