@@ -60,7 +60,7 @@ writeInstallRDFToDir({
     maxVersion: "1"
   }],
   name: "Unpacked, Enabled",
-}, profileDir, null, "extraFile.js");
+}, profileDir, undefined, "extraFile.js");
 
 
 // Unpacked, disabled
@@ -75,7 +75,7 @@ writeInstallRDFToDir({
     maxVersion: "1"
   }],
   name: "Unpacked, disabled",
-}, profileDir, null, "extraFile.js");
+}, profileDir, undefined, "extraFile.js");
 
 // Keep track of the last time stamp we've used, so that we can keep moving
 // it forward (if we touch two different files in the same add-on with the same
