@@ -7,11 +7,6 @@ var Cc = Components.classes;
 var Ci = Components.interfaces;
 var Cu = Components.utils;
 
-// Maximum error in file modification times. Some file systems don't store
-// modification times exactly. As long as we are closer than this then it
-// still passes.
-const MAX_TIME_DIFFERENCE = 3000;
-
 // install.rdf size, icon.png, icon64.png size
 const ADDON1_SIZE = 705 + 16 + 16;
 
@@ -145,6 +140,8 @@ function check_test_1(installSyncGUID) {
       else {
         let iconFile = uri.QueryInterface(AM_Ci.nsIFileURL).file;
         do_check_true(iconFile.exists());
+        // Make the iconFile predictably old.
+        iconFile.lastModifiedTime = Date.now() - MAKE_FILE_OLD_DIFFERENCE;
       }
 
       // Make the pending install have a sensible date
@@ -189,10 +186,18 @@ function check_test_1(installSyncGUID) {
           do_check_true(a1.hasResource("install.rdf"));
           do_check_false(a1.hasResource("foo.bar"));
 
-          let uri = do_get_addon_root_uri(profileDir, "addon1@tests.mozilla.org");
-          do_check_eq(a1.getResourceURI("install.rdf").spec, uri + "install.rdf");
-          do_check_eq(a1.iconURL, uri + "icon.png");
-          do_check_eq(a1.icon64URL, uri + "icon64.png");
+          let uri2 = do_get_addon_root_uri(profileDir, "addon1@tests.mozilla.org");
+          do_check_eq(a1.getResourceURI("install.rdf").spec, uri2 + "install.rdf");
+          do_check_eq(a1.iconURL, uri2 + "icon.png");
+          do_check_eq(a1.icon64URL, uri2 + "icon64.png");
+
+          // Ensure that extension bundle (or icon if unpacked) has updated
+          // lastModifiedDate.
+          let testURI = a1.getResourceURI(TEST_UNPACKED ? "icon.png" : "");
+          let testFile = testURI.QueryInterface(Components.interfaces.nsIFileURL).file;
+          do_check_true(testFile.exists());
+          difference = testFile.lastModifiedTime - Date.now();
+          do_check_true(Math.abs(difference) < MAX_TIME_DIFFERENCE);
 
           a1.uninstall();
           let { id, version } = a1;
@@ -228,7 +233,7 @@ function run_test_2() {
       ], check_test_2);
 
       install.addListener({
-        onDownloadProgress: function(install) {
+        onDownloadProgress: function() {
           do_execute_soon(function() {
             Components.utils.forceGC();
           });
@@ -382,8 +387,8 @@ function check_test_5(install) {
       do_check_eq(installs[0].addon, olda2.pendingUpgrade);
       restartManager();
 
-      AddonManager.getInstallsByTypes(null, function(installs) {
-        do_check_eq(installs.length, 0);
+      AddonManager.getInstallsByTypes(null, function(installs2) {
+        do_check_eq(installs2.length, 0);
 
         AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2) {
           do_check_neq(a2, null);
@@ -1143,26 +1148,26 @@ function run_test_16() {
           do_check_true(a2.userDisabled);
           do_check_false(a2.isActive);
 
-          let url = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
-          AddonManager.getInstallForURL(url, function(aInstall) {
-            aInstall.addListener({
+          let url_2 = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
+          AddonManager.getInstallForURL(url_2, function(aInstall_2) {
+            aInstall_2.addListener({
               onInstallEnded: function() {
                do_execute_soon(function install2_2_ended() {
-                do_check_true(aInstall.addon.userDisabled);
+                do_check_true(aInstall_2.addon.userDisabled);
 
                 restartManager();
 
-                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2) {
-                  do_check_true(a2.userDisabled);
-                  do_check_false(a2.isActive);
+                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2_2) {
+                  do_check_true(a2_2.userDisabled);
+                  do_check_false(a2_2.isActive);
 
-                  a2.uninstall();
+                  a2_2.uninstall();
                   do_execute_soon(run_test_17);
                 });
                });
               }
             });
-            aInstall.install();
+            aInstall_2.install();
           }, "application/x-xpinstall");
         });
        });
@@ -1189,29 +1194,29 @@ function run_test_17() {
           do_check_false(a2.userDisabled);
           do_check_true(a2.isActive);
 
-          let url = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
-          AddonManager.getInstallForURL(url, function(aInstall) {
-            aInstall.addListener({
+          let url_2 = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
+          AddonManager.getInstallForURL(url_2, function(aInstall_2) {
+            aInstall_2.addListener({
               onInstallStarted: function() {
-                do_check_false(aInstall.addon.userDisabled);
-                aInstall.addon.userDisabled = true;
+                do_check_false(aInstall_2.addon.userDisabled);
+                aInstall_2.addon.userDisabled = true;
               },
 
               onInstallEnded: function() {
                do_execute_soon(function install2_2_ended2() {
                 restartManager();
 
-                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2) {
-                  do_check_true(a2.userDisabled);
-                  do_check_false(a2.isActive);
+                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2_2) {
+                  do_check_true(a2_2.userDisabled);
+                  do_check_false(a2_2.isActive);
 
-                  a2.uninstall();
+                  a2_2.uninstall();
                   do_execute_soon(run_test_18);
                 });
                });
               }
             });
-            aInstall.install();
+            aInstall_2.install();
           }, "application/x-xpinstall");
         });
        });
@@ -1241,29 +1246,29 @@ function run_test_18() {
           do_check_true(a2.userDisabled);
           do_check_false(a2.isActive);
 
-          let url = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
-          AddonManager.getInstallForURL(url, function(aInstall) {
-            aInstall.addListener({
+          let url_2 = "http://localhost:" + gPort + "/addons/test_install2_2.xpi";
+          AddonManager.getInstallForURL(url_2, function(aInstall_2) {
+            aInstall_2.addListener({
               onInstallStarted: function() {
-                do_check_true(aInstall.addon.userDisabled);
-                aInstall.addon.userDisabled = false;
+                do_check_true(aInstall_2.addon.userDisabled);
+                aInstall_2.addon.userDisabled = false;
               },
 
               onInstallEnded: function() {
                do_execute_soon(function install_2_2_ended3() {
                 restartManager();
 
-                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2) {
-                  do_check_false(a2.userDisabled);
-                  do_check_true(a2.isActive);
+                AddonManager.getAddonByID("addon2@tests.mozilla.org", function(a2_2) {
+                  do_check_false(a2_2.userDisabled);
+                  do_check_true(a2_2.isActive);
 
-                  a2.uninstall();
+                  a2_2.uninstall();
                   do_execute_soon(run_test_18_1);
                 });
                });
               }
             });
-            aInstall.install();
+            aInstall_2.install();
           }, "application/x-xpinstall");
         });
        });
@@ -1287,7 +1292,7 @@ function run_test_18_1() {
   let url = "http://localhost:" + gPort + "/addons/test_install2_1.xpi";
   AddonManager.getInstallForURL(url, function(aInstall) {
     aInstall.addListener({
-      onInstallEnded: function(aInstall, aAddon) {
+      onInstallEnded: function(unused, aAddon) {
        do_execute_soon(function test18_1_install_ended() {
         do_check_neq(aAddon.fullDescription, "Repository description");
 
@@ -1315,7 +1320,7 @@ function run_test_19() {
   let url = "http://localhost:" + gPort + "/addons/test_install2_1.xpi";
   AddonManager.getInstallForURL(url, function(aInstall) {
     aInstall.addListener({
-      onInstallEnded: function(aInstall, aAddon) {
+      onInstallEnded: function(unused, aAddon) {
        do_execute_soon(function test19_install_ended() {
         do_check_eq(aAddon.fullDescription, "Repository description");
 
@@ -1341,7 +1346,7 @@ function run_test_20() {
   let url = "http://localhost:" + gPort + "/addons/test_install2_1.xpi";
   AddonManager.getInstallForURL(url, function(aInstall) {
     aInstall.addListener({
-      onInstallEnded: function(aInstall, aAddon) {
+      onInstallEnded: function(unused, aAddon) {
        do_execute_soon(function test20_install_ended() {
         do_check_eq(aAddon.fullDescription, "Repository description");
 
@@ -1628,8 +1633,8 @@ function run_test_26() {
   let url = "http://localhost:" + gPort + "/redirect?/addons/test_install1.xpi";
   AddonManager.getInstallForURL(url, function(aInstall) {
     aInstall.addListener({
-      onDownloadProgress: function(aInstall) {
-        aInstall.cancel();
+      onDownloadProgress: function(aDownloadProgressInstall) {
+        aDownloadProgressInstall.cancel();
       }
     });
 
@@ -1786,7 +1791,7 @@ function run_test_29() {
 }
 
 function check_test_29(install) {
-  //ensure_test_completed();
+  // ensure_test_completed();
   do_check_eq(install.state, AddonManager.STATE_DOWNLOADED);
   do_check_neq(install.addon, null);
   do_check_false(install.addon.isCompatible);
