@@ -12,9 +12,16 @@
 
 #include "nsDebug.h"
 #include "nsColor.h"
+#include "nsTArray.h"
 
 struct gfxFontStyle;
 
+struct LookAndFeelInt
+{
+  int32_t id;
+  int32_t value;
+};
+
 namespace mozilla {
 
 class LookAndFeel
@@ -22,7 +29,7 @@ class LookAndFeel
 public:
   // When modifying this list, also modify nsXPLookAndFeel::sColorPrefs
   // in widget/xpwidgts/nsXPLookAndFeel.cpp.
-  enum ColorID {
+  enum ColorID : uint8_t {
 
     // WARNING : NO NEGATIVE VALUE IN THIS ENUMERATION
     // see patch in bug 57757 for more information
@@ -167,6 +174,9 @@ public:
     eColorID__moz_comboboxtext,
     eColorID__moz_combobox,
 
+    // GtkInfoBar
+    eColorID__moz_gtk_info_bar_text,
+
     // keep this one last, please
     eColorID_LAST_COLOR
   };
@@ -281,16 +291,6 @@ public:
      */
     eIntID_MacGraphiteTheme,
 
-    /*
-     * A Boolean value to determine whether the Mac OS X Lion-specific theming
-     * should be used.
-     *
-     * The value of this metric is not used on non-Mac platforms. These
-     * platforms should return NS_ERROR_NOT_IMPLEMENTED when queried for this
-     * metric.
-     */
-    eIntID_MacLionTheme,
-
    /*
     * A Boolean value to determine whether the Mac OS X Yosemite-specific theming
     * should be used.
@@ -336,14 +336,6 @@ public:
     eIntID_IMESelectedConvertedTextUnderline,
     eIntID_SpellCheckerUnderlineStyle,
 
-    /**
-     * If this metric != 0, show icons in menus.
-     */
-    eIntID_ImagesInMenus,
-    /**
-     * If this metric != 0, show icons in buttons.
-     */
-    eIntID_ImagesInButtons,
     /**
      * If this metric != 0, support window dragging on the menubar.
      */
@@ -397,7 +389,14 @@ public:
       * Overlay scrollbar animation constants.
       */
      eIntID_ScrollbarFadeBeginDelay,
-     eIntID_ScrollbarFadeDuration
+     eIntID_ScrollbarFadeDuration,
+      
+     /**
+      * Distance in pixels to offset the context menu from the cursor
+      * on open.
+      */
+     eIntID_ContextMenuOffsetVertical,
+     eIntID_ContextMenuOffsetHorizontal
   };
 
   /**
@@ -509,6 +508,15 @@ public:
    */
   static nsresult GetColor(ColorID aID, nscolor* aResult);
 
+   /**
+   * This variant of GetColor() takes an extra Boolean parameter that allows
+   * the caller to ask that hard-coded color values be substituted for
+   * native colors (used when it is desireable to hide system colors to
+   * avoid system fingerprinting).
+   */
+  static nsresult GetColor(ColorID aID, bool aUseStandinsForNativeColors,
+                           nscolor* aResult);
+
   /**
    * GetInt() and GetFloat() return a int or float value for aID.  The result
    * might be distance, time, some flags or a int value which has particular
@@ -584,6 +592,13 @@ public:
    * cached data would be released.
    */
   static void Refresh();
+
+  /**
+   * If the implementation is caching values, these accessors allow the
+   * cache to be exported and imported.
+   */
+  static nsTArray<LookAndFeelInt> GetIntCache();
+  static void SetIntCache(const nsTArray<LookAndFeelInt>& aLookAndFeelIntCache);
 };
 
 } // namespace mozilla
