@@ -16,9 +16,15 @@
 #ifndef BOOTANIMATION_H
 #define BOOTANIMATION_H
 
+namespace mozilla {
+
+MOZ_EXPORT __attribute__ ((weak))
 void StartBootAnimation();
 
 /* Stop the boot animation if it's still running. */
+MOZ_EXPORT __attribute__ ((weak))
 void StopBootAnimation();
 
+} // namespace mozilla
+
 #endif /* BOOTANIMATION_H */
