@@ -30,13 +30,14 @@ public:
   nsScreenGtk();
   ~nsScreenGtk();
 
-  NS_IMETHOD GetId(uint32_t* aId);
-  NS_IMETHOD GetRect(int32_t* aLeft, int32_t* aTop, int32_t* aWidth, int32_t* aHeight);
-  NS_IMETHOD GetAvailRect(int32_t* aLeft, int32_t* aTop, int32_t* aWidth, int32_t* aHeight);
-  NS_IMETHOD GetRectDisplayPix(int32_t *outLeft, int32_t *outTop, int32_t *outWidth, int32_t *outHeight);
-  NS_IMETHOD GetAvailRectDisplayPix(int32_t *outLeft, int32_t *outTop, int32_t *outWidth, int32_t *outHeight);
-  NS_IMETHOD GetPixelDepth(int32_t* aPixelDepth);
-  NS_IMETHOD GetColorDepth(int32_t* aColorDepth);
+  NS_IMETHOD GetId(uint32_t* aId) override;
+  NS_IMETHOD GetRect(int32_t* aLeft, int32_t* aTop,
+                     int32_t* aWidth, int32_t* aHeight) override;
+  NS_IMETHOD GetAvailRect(int32_t* aLeft, int32_t* aTop,
+                          int32_t* aWidth, int32_t* aHeight) override;
+  NS_IMETHOD GetPixelDepth(int32_t* aPixelDepth) override;
+  NS_IMETHOD GetColorDepth(int32_t* aColorDepth) override;
+  NS_IMETHOD GetDefaultCSSScaleFactor(double* aScaleFactor) override;
 
   void Init(GdkWindow *aRootWindow);
 #ifdef MOZ_X11
