@@ -13,7 +13,7 @@
 #include "nsCOMArray.h"
 #include "nsXULAppAPI.h"
 #include "prinrval.h"
-#include "prlog.h"
+#include "mozilla/Logging.h"
 #include "prtime.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/Services.h"
@@ -46,9 +46,7 @@ using namespace mozilla;
 // Number of seconds in a day.
 #define SECONDS_PER_DAY 86400
 
-#ifdef PR_LOGGING
 static PRLogModuleInfo *sLog = nullptr;
-#endif
 
 #define LOG_TAG "GeckoIdleService"
 #define LOG_LEVEL ANDROID_LOG_DEBUG
@@ -74,7 +72,7 @@ nsIdleServiceDaily::Observe(nsISupports *,
                             const char *aTopic,
                             const char16_t *)
 {
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("nsIdleServiceDaily: Observe '%s' (%d)",
           aTopic, mShutdownInProgress));
 
@@ -94,7 +92,7 @@ nsIdleServiceDaily::Observe(nsISupports *,
   }
   MOZ_ASSERT(strcmp(aTopic, OBSERVER_TOPIC_IDLE) == 0);
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("nsIdleServiceDaily: Notifying idle-daily observers"));
 #ifdef MOZ_WIDGET_ANDROID
   __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -130,7 +128,7 @@ nsIdleServiceDaily::Observe(nsISupports *,
     prefs->SavePrefFile(nullptr);
   }
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("nsIdleServiceDaily: Storing last idle time as %d sec.", nowSec));
 #ifdef MOZ_WIDGET_ANDROID
   __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -141,7 +139,7 @@ nsIdleServiceDaily::Observe(nsISupports *,
   mExpectedTriggerTime  = PR_Now() + ((PRTime)SECONDS_PER_DAY *
                                       (PRTime)PR_USEC_PER_SEC);
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("nsIdleServiceDaily: Restarting daily timer"));
 
   // Start timer for the next check in one day.
@@ -179,7 +177,7 @@ nsIdleServiceDaily::Init()
   }
   int32_t secondsSinceLastDaily = nowSec - lastDaily;
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("nsIdleServiceDaily: Init: seconds since last daily: %d",
           secondsSinceLastDaily));
 
@@ -190,7 +188,7 @@ nsIdleServiceDaily::Init()
     bool hasBeenLongWait = (lastDaily &&
                             (secondsSinceLastDaily > (SECONDS_PER_DAY * 2)));
 
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("nsIdleServiceDaily: has been long wait? %d",
             hasBeenLongWait));
 
@@ -198,7 +196,7 @@ nsIdleServiceDaily::Init()
     // sends the idle-daily event.
     StageIdleDaily(hasBeenLongWait);
   } else {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("nsIdleServiceDaily: Setting timer a day from now"));
 #ifdef MOZ_WIDGET_ANDROID
     __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -210,7 +208,7 @@ nsIdleServiceDaily::Init()
     int32_t milliSecLeftUntilDaily = (SECONDS_PER_DAY - secondsSinceLastDaily)
       * PR_MSEC_PER_SEC;
 
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("nsIdleServiceDaily: Seconds till next timeout: %d",
             (SECONDS_PER_DAY - secondsSinceLastDaily)));
 
@@ -229,7 +227,7 @@ nsIdleServiceDaily::Init()
   // Register for when we should terminate/pause
   nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
   if (obs) {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("nsIdleServiceDaily: Registering for system event observers."));
     obs->AddObserver(this, "xpcom-will-shutdown", true);
     obs->AddObserver(this, "profile-change-teardown", true);
@@ -250,7 +248,7 @@ void
 nsIdleServiceDaily::StageIdleDaily(bool aHasBeenLongWait)
 {
   NS_ASSERTION(mIdleService, "No idle service available?");
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
           ("nsIdleServiceDaily: Registering Idle observer callback "
            "(short wait requested? %d)", aHasBeenLongWait));
 #ifdef MOZ_WIDGET_ANDROID
@@ -267,7 +265,7 @@ nsIdleServiceDaily::StageIdleDaily(bool aHasBeenLongWait)
 void
 nsIdleServiceDaily::DailyCallback(nsITimer* aTimer, void* aClosure)
 {
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
           ("nsIdleServiceDaily: DailyCallback running"));
 #ifdef MOZ_WIDGET_ANDROID
   __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -286,7 +284,7 @@ nsIdleServiceDaily::DailyCallback(nsITimer* aTimer, void* aClosure)
     // Add 10 ms to ensure we don't undershoot, and never get a "0" timer.
     delayTime += 10 * PR_USEC_PER_MSEC;
 
-    PR_LOG(sLog, PR_LOG_DEBUG, ("nsIdleServiceDaily: DailyCallback resetting timer to %lld msec",
+    MOZ_LOG(sLog, LogLevel::Debug, ("nsIdleServiceDaily: DailyCallback resetting timer to %lld msec",
                         delayTime / PR_USEC_PER_MSEC));
 #ifdef MOZ_WIDGET_ANDROID
     __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -382,12 +380,12 @@ nsIdleServiceDaily::DailyCallback(nsITimer* aTimer, void* aClosure)
 
 namespace { 
 nsIdleService* gIdleService;
-}
+} // namespace
 
 already_AddRefed<nsIdleService>
 nsIdleService::GetInstance()
 {
-  nsRefPtr<nsIdleService> instance(gIdleService);
+  RefPtr<nsIdleService> instance(gIdleService);
   return instance.forget();
 }
 
@@ -396,13 +394,11 @@ nsIdleService::nsIdleService() : mCurrentlySetToTimeoutAt(TimeStamp()),
                                  mDeltaToNextIdleSwitchInS(UINT32_MAX),
                                  mLastUserInteraction(TimeStamp::Now())
 {
-#ifdef PR_LOGGING
   if (sLog == nullptr)
     sLog = PR_NewLogModule("idleService");
-#endif
   MOZ_ASSERT(!gIdleService);
   gIdleService = this;
-  if (XRE_GetProcessType() == GeckoProcessType_Default) {
+  if (XRE_IsParentProcess()) {
     mDailyIdle = new nsIdleServiceDaily(this);
     mDailyIdle->Init();
   }
@@ -429,13 +425,13 @@ nsIdleService::AddIdleObserver(nsIObserver* aObserver, uint32_t aIdleTimeInS)
   // high either - no more than ~136 years.
   NS_ENSURE_ARG_RANGE(aIdleTimeInS, 1, (UINT32_MAX / 10) - 1);
 
-  if (XRE_GetProcessType() == GeckoProcessType_Content) {
+  if (XRE_IsContentProcess()) {
     dom::ContentChild* cpc = dom::ContentChild::GetSingleton();
     cpc->AddIdleObserver(aObserver, aIdleTimeInS);
     return NS_OK;
   }
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
        ("idleService: Register idle observer %p for %d seconds",
         aObserver, aIdleTimeInS));
 #ifdef MOZ_WIDGET_ANDROID
@@ -463,7 +459,7 @@ nsIdleService::AddIdleObserver(nsIObserver* aObserver, uint32_t aIdleTimeInS)
   if (mDeltaToNextIdleSwitchInS > aIdleTimeInS) {
     // If it is, then this is the next to move to idle (at this point we
     // don't care if it should have switched already).
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
           ("idleService: Register: adjusting next switch from %d to %d seconds",
            mDeltaToNextIdleSwitchInS, aIdleTimeInS));
 #ifdef MOZ_WIDGET_ANDROID
@@ -488,7 +484,7 @@ nsIdleService::RemoveIdleObserver(nsIObserver* aObserver, uint32_t aTimeInS)
   NS_ENSURE_ARG_POINTER(aObserver);
   NS_ENSURE_ARG(aTimeInS);
 
-  if (XRE_GetProcessType() == GeckoProcessType_Content) {
+  if (XRE_IsContentProcess()) {
     dom::ContentChild* cpc = dom::ContentChild::GetSingleton();
     cpc->RemoveIdleObserver(aObserver, aTimeInS);
     return NS_OK;
@@ -505,7 +501,7 @@ nsIdleService::RemoveIdleObserver(nsIObserver* aObserver, uint32_t aTimeInS)
     if (mArrayListeners.ElementAt(listenerIndex).isIdle)
       mIdleObserverCount--;
     mArrayListeners.RemoveElementAt(listenerIndex);
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: Remove observer %p (%d seconds), %d remain idle",
             aObserver, aTimeInS, mIdleObserverCount));
 #ifdef MOZ_WIDGET_ANDROID
@@ -517,7 +513,7 @@ nsIdleService::RemoveIdleObserver(nsIObserver* aObserver, uint32_t aTimeInS)
   }
 
   // If we get here, we haven't removed anything:
-  PR_LOG(sLog, PR_LOG_WARNING, 
+  MOZ_LOG(sLog, LogLevel::Warning, 
          ("idleService: Failed to remove idle observer %p (%d seconds)",
           aObserver, aTimeInS));
 #ifdef MOZ_WIDGET_ANDROID
@@ -531,7 +527,7 @@ nsIdleService::RemoveIdleObserver(nsIObserver* aObserver, uint32_t aTimeInS)
 NS_IMETHODIMP
 nsIdleService::ResetIdleTimeOut(uint32_t idleDeltaInMS)
 {
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: Reset idle timeout (last interaction %u msec)",
           idleDeltaInMS));
 
@@ -541,13 +537,12 @@ nsIdleService::ResetIdleTimeOut(uint32_t idleDeltaInMS)
 
   // If no one is idle, then we are done, any existing timers can keep running.
   if (mIdleObserverCount == 0) {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: Reset idle timeout: no idle observers"));
     return NS_OK;
   }
 
   // Mark all idle services as non-idle, and calculate the next idle timeout.
-  Telemetry::AutoTimer<Telemetry::IDLE_NOTIFY_BACK_MS> timer;
   nsCOMArray<nsIObserver> notifyList;
   mDeltaToNextIdleSwitchInS = UINT32_MAX;
 
@@ -573,8 +568,6 @@ nsIdleService::ResetIdleTimeOut(uint32_t idleDeltaInMS)
   ReconfigureTimer();
 
   int32_t numberOfPendingNotifications = notifyList.Count();
-  Telemetry::Accumulate(Telemetry::IDLE_NOTIFY_BACK_LISTENERS,
-                        numberOfPendingNotifications);
 
   // Bail if nothing to do.
   if (!numberOfPendingNotifications) {
@@ -591,7 +584,7 @@ nsIdleService::ResetIdleTimeOut(uint32_t idleDeltaInMS)
 
   // Send the "non-idle" events.
   while (numberOfPendingNotifications--) {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: Reset idle timeout: tell observer %p user is back",
             notifyList[numberOfPendingNotifications]));
 #ifdef MOZ_WIDGET_ANDROID
@@ -619,7 +612,7 @@ nsIdleService::GetIdleTime(uint32_t* idleTime)
 
   bool polledIdleTimeIsValid = PollIdleTime(&polledIdleTimeMS);
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: Get idle time: polled %u msec, valid = %d",
           polledIdleTimeMS, polledIdleTimeIsValid));
   
@@ -627,7 +620,7 @@ nsIdleService::GetIdleTime(uint32_t* idleTime)
   TimeDuration timeSinceReset = TimeStamp::Now() - mLastUserInteraction;
   uint32_t timeSinceResetInMS = timeSinceReset.ToMilliseconds();
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: Get idle time: time since reset %u msec",
           timeSinceResetInMS));
 #ifdef MOZ_WIDGET_ANDROID
@@ -683,7 +676,7 @@ nsIdleService::IdleTimerCallback(void)
   uint32_t currentIdleTimeInMS;
 
   if (NS_FAILED(GetIdleTime(&currentIdleTimeInMS))) {
-    PR_LOG(sLog, PR_LOG_ALWAYS,
+    MOZ_LOG(sLog, LogLevel::Info,
            ("idleService: Idle timer callback: failed to get idle time"));
 #ifdef MOZ_WIDGET_ANDROID
     __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -692,7 +685,7 @@ nsIdleService::IdleTimerCallback(void)
     return;
   }
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: Idle timer callback: current idle time %u msec",
           currentIdleTimeInMS));
 #ifdef MOZ_WIDGET_ANDROID
@@ -759,12 +752,10 @@ nsIdleService::IdleTimerCallback(void)
   ReconfigureTimer();
 
   int32_t numberOfPendingNotifications = notifyList.Count();
-  Telemetry::Accumulate(Telemetry::IDLE_NOTIFY_IDLE_LISTENERS,
-                        numberOfPendingNotifications);
 
   // Bail if nothing to do.
   if (!numberOfPendingNotifications) {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: **** Idle timer callback: no observers to message."));
     return;
   }
@@ -775,7 +766,7 @@ nsIdleService::IdleTimerCallback(void)
 
   // Notify all listeners that just timed out.
   while (numberOfPendingNotifications--) {
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: **** Idle timer callback: tell observer %p user is idle",
             notifyList[numberOfPendingNotifications]));
 #ifdef MOZ_WIDGET_ANDROID
@@ -792,11 +783,9 @@ nsIdleService::IdleTimerCallback(void)
 void
 nsIdleService::SetTimerExpiryIfBefore(TimeStamp aNextTimeout)
 {
-#if defined(PR_LOGGING) || defined(MOZ_WIDGET_ANDROID)
   TimeDuration nextTimeoutDuration = aNextTimeout - TimeStamp::Now();
-#endif
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: SetTimerExpiryIfBefore: next timeout %0.f msec from now",
           nextTimeoutDuration.ToMilliseconds()));
 
@@ -831,7 +820,7 @@ nsIdleService::SetTimerExpiryIfBefore(TimeStamp aNextTimeout)
     mCurrentlySetToTimeoutAt += TimeDuration::FromMilliseconds(10);
 
     TimeDuration deltaTime = mCurrentlySetToTimeoutAt - currentTime;
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: IdleService reset timer expiry to %0.f msec from now",
             deltaTime.ToMilliseconds()));
 #ifdef MOZ_WIDGET_ANDROID
@@ -856,7 +845,7 @@ nsIdleService::ReconfigureTimer(void)
   if ((mIdleObserverCount == 0) && UINT32_MAX == mDeltaToNextIdleSwitchInS) {
     // If not, just let any existing timers run to completion
     // And bail out.
-    PR_LOG(sLog, PR_LOG_DEBUG,
+    MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: ReconfigureTimer: no idle or waiting observers"));
 #ifdef MOZ_WIDGET_ANDROID
   __android_log_print(LOG_LEVEL, LOG_TAG,
@@ -874,11 +863,9 @@ nsIdleService::ReconfigureTimer(void)
   TimeStamp nextTimeoutAt = mLastUserInteraction +
                             TimeDuration::FromSeconds(mDeltaToNextIdleSwitchInS);
 
-#if defined(PR_LOGGING) || defined(MOZ_WIDGET_ANDROID)
   TimeDuration nextTimeoutDuration = nextTimeoutAt - curTime;
-#endif
 
-  PR_LOG(sLog, PR_LOG_DEBUG,
+  MOZ_LOG(sLog, LogLevel::Debug,
          ("idleService: next timeout %0.f msec from now",
           nextTimeoutDuration.ToMilliseconds()));
 
@@ -894,7 +881,7 @@ nsIdleService::ReconfigureTimer(void)
         curTime + TimeDuration::FromMilliseconds(MIN_IDLE_POLL_INTERVAL_MSEC);
 
     if (nextTimeoutAt > pollTimeout) {
-      PR_LOG(sLog, PR_LOG_DEBUG,
+      MOZ_LOG(sLog, LogLevel::Debug,
            ("idleService: idle observers, reducing timeout to %lu msec from now",
             MIN_IDLE_POLL_INTERVAL_MSEC));
 #ifdef MOZ_WIDGET_ANDROID
