@@ -1,100 +1,118 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsSupportsPrimitives.h"
 #include "nsMemory.h"
-#include "prprf.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/IntegerPrintfMacros.h"
+#include "mozilla/Sprintf.h"
+#include <algorithm>
+
+template<typename T>
+static char*
+DataToString(const char* aFormat, T aData)
+{
+  static const int size = 32;
+  char buf[size];
+
+  int len = SprintfLiteral(buf, aFormat, aData);
+  MOZ_ASSERT(len >= 0);
+
+  return static_cast<char*>(nsMemory::Clone(buf, std::min(len + 1, size) *
+                                                 sizeof(char)));
+}
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsIDImpl, nsISupportsID, nsISupportsPrimitive)
+NS_IMPL_ISUPPORTS(nsSupportsID, nsISupportsID, nsISupportsPrimitive)
 
-nsSupportsIDImpl::nsSupportsIDImpl()
+nsSupportsID::nsSupportsID()
   : mData(nullptr)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsIDImpl::GetType(uint16_t* aType)
+nsSupportsID::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_ID;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsIDImpl::GetData(nsID** aData)
+nsSupportsID::GetData(nsID** aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
+
   if (mData) {
-    *aData = (nsID*)nsMemory::Clone(mData, sizeof(nsID));
-    return *aData ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    *aData = static_cast<nsID*>(nsMemory::Clone(mData, sizeof(nsID)));
+  } else {
+    *aData = nullptr;
   }
-  *aData = nullptr;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsIDImpl::SetData(const nsID* aData)
+nsSupportsID::SetData(const nsID* aData)
 {
   if (mData) {
-    nsMemory::Free(mData);
+    free(mData);
   }
+
   if (aData) {
-    mData = (nsID*)nsMemory::Clone(aData, sizeof(nsID));
+    mData = static_cast<nsID*>(nsMemory::Clone(aData, sizeof(nsID)));
   } else {
     mData = nullptr;
   }
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsIDImpl::ToString(char** aResult)
+nsSupportsID::ToString(char** aResult)
 {
-  char* result;
   NS_ASSERTION(aResult, "Bad pointer");
+
   if (mData) {
-    result = mData->ToString();
+    *aResult = mData->ToString();
   } else {
     static const char nullStr[] = "null";
-    result = (char*)nsMemory::Clone(nullStr, sizeof(nullStr));
+    *aResult = static_cast<char*>(nsMemory::Clone(nullStr, sizeof(nullStr)));
   }
 
-  *aResult = result;
-  return result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  return NS_OK;
 }
 
 /*****************************************************************************
- * nsSupportsCStringImpl
+ * nsSupportsCString
  *****************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsCStringImpl, nsISupportsCString,
+NS_IMPL_ISUPPORTS(nsSupportsCString, nsISupportsCString,
                   nsISupportsPrimitive)
 
 NS_IMETHODIMP
-nsSupportsCStringImpl::GetType(uint16_t* aType)
+nsSupportsCString::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
-
   *aType = TYPE_CSTRING;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsCStringImpl::GetData(nsACString& aData)
+nsSupportsCString::GetData(nsACString& aData)
 {
   aData = mData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsCStringImpl::ToString(char** aResult)
+nsSupportsCString::ToString(char** aResult)
 {
   *aResult = ToNewCString(mData);
-
   if (!*aResult) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -103,43 +121,42 @@ nsSupportsCStringImpl::ToString(char** aResult)
 }
 
 NS_IMETHODIMP
-nsSupportsCStringImpl::SetData(const nsACString& aData)
+nsSupportsCString::SetData(const nsACString& aData)
 {
   bool ok = mData.Assign(aData, mozilla::fallible);
   if (!ok) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
+
   return NS_OK;
 }
 
 /*****************************************************************************
- * nsSupportsStringImpl
+ * nsSupportsString
  *****************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsStringImpl, nsISupportsString,
+NS_IMPL_ISUPPORTS(nsSupportsString, nsISupportsString,
                   nsISupportsPrimitive)
 
 NS_IMETHODIMP
-nsSupportsStringImpl::GetType(uint16_t* aType)
+nsSupportsString::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
-
   *aType = TYPE_STRING;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsStringImpl::GetData(nsAString& aData)
+nsSupportsString::GetData(nsAString& aData)
 {
   aData = mData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsStringImpl::ToString(char16_t** aResult)
+nsSupportsString::ToString(char16_t** aResult)
 {
   *aResult = ToNewUnicode(mData);
-
   if (!*aResult) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -148,36 +165,36 @@ nsSupportsStringImpl::ToString(char16_t** aResult)
 }
 
 NS_IMETHODIMP
-nsSupportsStringImpl::SetData(const nsAString& aData)
+nsSupportsString::SetData(const nsAString& aData)
 {
   bool ok = mData.Assign(aData, mozilla::fallible);
   if (!ok) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
+
   return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRBoolImpl, nsISupportsPRBool,
+NS_IMPL_ISUPPORTS(nsSupportsPRBool, nsISupportsPRBool,
                   nsISupportsPrimitive)
 
-nsSupportsPRBoolImpl::nsSupportsPRBoolImpl()
+nsSupportsPRBool::nsSupportsPRBool()
   : mData(false)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRBoolImpl::GetType(uint16_t* aType)
+nsSupportsPRBool::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRBOOL;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRBoolImpl::GetData(bool* aData)
+nsSupportsPRBool::GetData(bool* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -185,42 +202,42 @@ nsSupportsPRBoolImpl::GetData(bool* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRBoolImpl::SetData(bool aData)
+nsSupportsPRBool::SetData(bool aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRBoolImpl::ToString(char** aResult)
+nsSupportsPRBool::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
   const char* str = mData ? "true" : "false";
-  *aResult = (char*)nsMemory::Clone(str, (strlen(str) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = static_cast<char*>(nsMemory::Clone(str, (strlen(str) + 1) *
+                                                     sizeof(char)));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRUint8Impl, nsISupportsPRUint8,
+NS_IMPL_ISUPPORTS(nsSupportsPRUint8, nsISupportsPRUint8,
                   nsISupportsPrimitive)
 
-nsSupportsPRUint8Impl::nsSupportsPRUint8Impl()
+nsSupportsPRUint8::nsSupportsPRUint8()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint8Impl::GetType(uint16_t* aType)
+nsSupportsPRUint8::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRUINT8;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint8Impl::GetData(uint8_t* aData)
+nsSupportsPRUint8::GetData(uint8_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -228,45 +245,40 @@ nsSupportsPRUint8Impl::GetData(uint8_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint8Impl::SetData(uint8_t aData)
+nsSupportsPRUint8::SetData(uint8_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint8Impl::ToString(char** aResult)
+nsSupportsPRUint8::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 8;
-  char buf[size];
-  PR_snprintf(buf, size, "%u", (uint16_t)mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%u", static_cast<unsigned int>(mData));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRUint16Impl, nsISupportsPRUint16,
+NS_IMPL_ISUPPORTS(nsSupportsPRUint16, nsISupportsPRUint16,
                   nsISupportsPrimitive)
 
-nsSupportsPRUint16Impl::nsSupportsPRUint16Impl()
+nsSupportsPRUint16::nsSupportsPRUint16()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint16Impl::GetType(uint16_t* aType)
+nsSupportsPRUint16::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRUINT16;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint16Impl::GetData(uint16_t* aData)
+nsSupportsPRUint16::GetData(uint16_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -274,45 +286,40 @@ nsSupportsPRUint16Impl::GetData(uint16_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint16Impl::SetData(uint16_t aData)
+nsSupportsPRUint16::SetData(uint16_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint16Impl::ToString(char** aResult)
+nsSupportsPRUint16::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 8;
-  char buf[size];
-  PR_snprintf(buf, size, "%u", (int)mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%u", static_cast<unsigned int>(mData));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRUint32Impl, nsISupportsPRUint32,
+NS_IMPL_ISUPPORTS(nsSupportsPRUint32, nsISupportsPRUint32,
                   nsISupportsPrimitive)
 
-nsSupportsPRUint32Impl::nsSupportsPRUint32Impl()
+nsSupportsPRUint32::nsSupportsPRUint32()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint32Impl::GetType(uint16_t* aType)
+nsSupportsPRUint32::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRUINT32;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint32Impl::GetData(uint32_t* aData)
+nsSupportsPRUint32::GetData(uint32_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -320,45 +327,40 @@ nsSupportsPRUint32Impl::GetData(uint32_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint32Impl::SetData(uint32_t aData)
+nsSupportsPRUint32::SetData(uint32_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint32Impl::ToString(char** aResult)
+nsSupportsPRUint32::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 16;
-  char buf[size];
-  PR_snprintf(buf, size, "%lu", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%u", mData);
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRUint64Impl, nsISupportsPRUint64,
+NS_IMPL_ISUPPORTS(nsSupportsPRUint64, nsISupportsPRUint64,
                   nsISupportsPrimitive)
 
-nsSupportsPRUint64Impl::nsSupportsPRUint64Impl()
+nsSupportsPRUint64::nsSupportsPRUint64()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint64Impl::GetType(uint16_t* aType)
+nsSupportsPRUint64::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRUINT64;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint64Impl::GetData(uint64_t* aData)
+nsSupportsPRUint64::GetData(uint64_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -366,45 +368,40 @@ nsSupportsPRUint64Impl::GetData(uint64_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint64Impl::SetData(uint64_t aData)
+nsSupportsPRUint64::SetData(uint64_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRUint64Impl::ToString(char** aResult)
+nsSupportsPRUint64::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 32;
-  char buf[size];
-  PR_snprintf(buf, size, "%llu", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%llu", mData);
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRTimeImpl, nsISupportsPRTime,
+NS_IMPL_ISUPPORTS(nsSupportsPRTime, nsISupportsPRTime,
                   nsISupportsPrimitive)
 
-nsSupportsPRTimeImpl::nsSupportsPRTimeImpl()
+nsSupportsPRTime::nsSupportsPRTime()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRTimeImpl::GetType(uint16_t* aType)
+nsSupportsPRTime::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRTIME;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRTimeImpl::GetData(PRTime* aData)
+nsSupportsPRTime::GetData(PRTime* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -412,45 +409,40 @@ nsSupportsPRTimeImpl::GetData(PRTime* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRTimeImpl::SetData(PRTime aData)
+nsSupportsPRTime::SetData(PRTime aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRTimeImpl::ToString(char** aResult)
+nsSupportsPRTime::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 32;
-  char buf[size];
-  PR_snprintf(buf, size, "%llu", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%" PRIu64, mData);
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsCharImpl, nsISupportsChar,
+NS_IMPL_ISUPPORTS(nsSupportsChar, nsISupportsChar,
                   nsISupportsPrimitive)
 
-nsSupportsCharImpl::nsSupportsCharImpl()
+nsSupportsChar::nsSupportsChar()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsCharImpl::GetType(uint16_t* aType)
+nsSupportsChar::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_CHAR;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsCharImpl::GetData(char* aData)
+nsSupportsChar::GetData(char* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -458,47 +450,43 @@ nsSupportsCharImpl::GetData(char* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsCharImpl::SetData(char aData)
+nsSupportsChar::SetData(char aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsCharImpl::ToString(char** aResult)
+nsSupportsChar::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
+  *aResult = static_cast<char*>(moz_xmalloc(2 * sizeof(char)));
+  *aResult[0] = mData;
+  *aResult[1] = '\0';
 
-  char* result = (char*)nsMemory::Alloc(2 * sizeof(char));
-  if (result) {
-    result[0] = mData;
-    result[1] = '\0';
-  }
-  *aResult = result;
-  return result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRInt16Impl, nsISupportsPRInt16,
+NS_IMPL_ISUPPORTS(nsSupportsPRInt16, nsISupportsPRInt16,
                   nsISupportsPrimitive)
 
-nsSupportsPRInt16Impl::nsSupportsPRInt16Impl()
+nsSupportsPRInt16::nsSupportsPRInt16()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt16Impl::GetType(uint16_t* aType)
+nsSupportsPRInt16::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRINT16;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt16Impl::GetData(int16_t* aData)
+nsSupportsPRInt16::GetData(int16_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -506,45 +494,40 @@ nsSupportsPRInt16Impl::GetData(int16_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt16Impl::SetData(int16_t aData)
+nsSupportsPRInt16::SetData(int16_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt16Impl::ToString(char** aResult)
+nsSupportsPRInt16::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 8;
-  char buf[size];
-  PR_snprintf(buf, size, "%d", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%d", static_cast<int>(mData));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRInt32Impl, nsISupportsPRInt32,
+NS_IMPL_ISUPPORTS(nsSupportsPRInt32, nsISupportsPRInt32,
                   nsISupportsPrimitive)
 
-nsSupportsPRInt32Impl::nsSupportsPRInt32Impl()
+nsSupportsPRInt32::nsSupportsPRInt32()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt32Impl::GetType(uint16_t* aType)
+nsSupportsPRInt32::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRINT32;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt32Impl::GetData(int32_t* aData)
+nsSupportsPRInt32::GetData(int32_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -552,45 +535,40 @@ nsSupportsPRInt32Impl::GetData(int32_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt32Impl::SetData(int32_t aData)
+nsSupportsPRInt32::SetData(int32_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt32Impl::ToString(char** aResult)
+nsSupportsPRInt32::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 16;
-  char buf[size];
-  PR_snprintf(buf, size, "%ld", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%d", mData);
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsPRInt64Impl, nsISupportsPRInt64,
+NS_IMPL_ISUPPORTS(nsSupportsPRInt64, nsISupportsPRInt64,
                   nsISupportsPrimitive)
 
-nsSupportsPRInt64Impl::nsSupportsPRInt64Impl()
+nsSupportsPRInt64::nsSupportsPRInt64()
   : mData(0)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt64Impl::GetType(uint16_t* aType)
+nsSupportsPRInt64::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_PRINT64;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt64Impl::GetData(int64_t* aData)
+nsSupportsPRInt64::GetData(int64_t* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -598,45 +576,40 @@ nsSupportsPRInt64Impl::GetData(int64_t* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt64Impl::SetData(int64_t aData)
+nsSupportsPRInt64::SetData(int64_t aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsPRInt64Impl::ToString(char** aResult)
+nsSupportsPRInt64::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 32;
-  char buf[size];
-  PR_snprintf(buf, size, "%lld", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%" PRId64, mData);
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsFloatImpl, nsISupportsFloat,
+NS_IMPL_ISUPPORTS(nsSupportsFloat, nsISupportsFloat,
                   nsISupportsPrimitive)
 
-nsSupportsFloatImpl::nsSupportsFloatImpl()
+nsSupportsFloat::nsSupportsFloat()
   : mData(float(0.0))
 {
 }
 
 NS_IMETHODIMP
-nsSupportsFloatImpl::GetType(uint16_t* aType)
+nsSupportsFloat::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_FLOAT;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsFloatImpl::GetData(float* aData)
+nsSupportsFloat::GetData(float* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -644,45 +617,40 @@ nsSupportsFloatImpl::GetData(float* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsFloatImpl::SetData(float aData)
+nsSupportsFloat::SetData(float aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsFloatImpl::ToString(char** aResult)
+nsSupportsFloat::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 32;
-  char buf[size];
-  PR_snprintf(buf, size, "%f", (double)mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%f", static_cast<double>(mData));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_ISUPPORTS(nsSupportsDoubleImpl, nsISupportsDouble,
+NS_IMPL_ISUPPORTS(nsSupportsDouble, nsISupportsDouble,
                   nsISupportsPrimitive)
 
-nsSupportsDoubleImpl::nsSupportsDoubleImpl()
+nsSupportsDouble::nsSupportsDouble()
   : mData(double(0.0))
 {
 }
 
 NS_IMETHODIMP
-nsSupportsDoubleImpl::GetType(uint16_t* aType)
+nsSupportsDouble::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_DOUBLE;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsDoubleImpl::GetData(double* aData)
+nsSupportsDouble::GetData(double* aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -690,46 +658,41 @@ nsSupportsDoubleImpl::GetData(double* aData)
 }
 
 NS_IMETHODIMP
-nsSupportsDoubleImpl::SetData(double aData)
+nsSupportsDouble::SetData(double aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsDoubleImpl::ToString(char** aResult)
+nsSupportsDouble::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-  static const int size = 32;
-  char buf[size];
-  PR_snprintf(buf, size, "%f", mData);
-
-  *aResult = (char*)nsMemory::Clone(buf, (strlen(buf) + 1) * sizeof(char));
-  return  *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = DataToString("%f", mData);
+  return  NS_OK;
 }
 
 /***************************************************************************/
 
 
-NS_IMPL_ISUPPORTS(nsSupportsVoidImpl, nsISupportsVoid,
+NS_IMPL_ISUPPORTS(nsSupportsVoid, nsISupportsVoid,
                   nsISupportsPrimitive)
 
-nsSupportsVoidImpl::nsSupportsVoidImpl()
+nsSupportsVoid::nsSupportsVoid()
   : mData(nullptr)
 {
 }
 
 NS_IMETHODIMP
-nsSupportsVoidImpl::GetType(uint16_t* aType)
+nsSupportsVoid::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_VOID;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsVoidImpl::GetData(void** aData)
+nsSupportsVoid::GetData(void** aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
   *aData = mData;
@@ -737,91 +700,87 @@ nsSupportsVoidImpl::GetData(void** aData)
 }
 
 NS_IMETHODIMP
-nsSupportsVoidImpl::SetData(void* aData)
+nsSupportsVoid::SetData(void* aData)
 {
   mData = aData;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsVoidImpl::ToString(char** aResult)
+nsSupportsVoid::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
-
   static const char str[] = "[raw data]";
-  char* result = (char*)nsMemory::Clone(str, sizeof(str));
-  *aResult = result;
-  return  result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = static_cast<char*>(nsMemory::Clone(str, sizeof(str)));
+  return NS_OK;
 }
 
 /***************************************************************************/
 
 
-NS_IMPL_ISUPPORTS(nsSupportsInterfacePointerImpl,
+NS_IMPL_ISUPPORTS(nsSupportsInterfacePointer,
                   nsISupportsInterfacePointer,
                   nsISupportsPrimitive)
 
-nsSupportsInterfacePointerImpl::nsSupportsInterfacePointerImpl()
+nsSupportsInterfacePointer::nsSupportsInterfacePointer()
   : mIID(nullptr)
 {
 }
 
-nsSupportsInterfacePointerImpl::~nsSupportsInterfacePointerImpl()
+nsSupportsInterfacePointer::~nsSupportsInterfacePointer()
 {
   if (mIID) {
-    nsMemory::Free(mIID);
+    free(mIID);
   }
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::GetType(uint16_t* aType)
+nsSupportsInterfacePointer::GetType(uint16_t* aType)
 {
   NS_ASSERTION(aType, "Bad pointer");
   *aType = TYPE_INTERFACE_POINTER;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::GetData(nsISupports** aData)
+nsSupportsInterfacePointer::GetData(nsISupports** aData)
 {
   NS_ASSERTION(aData, "Bad pointer");
-
   *aData = mData;
   NS_IF_ADDREF(*aData);
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::SetData(nsISupports* aData)
+nsSupportsInterfacePointer::SetData(nsISupports* aData)
 {
   mData = aData;
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::GetDataIID(nsID** aIID)
+nsSupportsInterfacePointer::GetDataIID(nsID** aIID)
 {
   NS_ASSERTION(aIID, "Bad pointer");
 
   if (mIID) {
-    *aIID = (nsID*)nsMemory::Clone(mIID, sizeof(nsID));
-    return *aIID ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    *aIID = static_cast<nsID*>(nsMemory::Clone(mIID, sizeof(nsID)));
+  } else {
+    *aIID = nullptr;
   }
-  *aIID = nullptr;
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::SetDataIID(const nsID* aIID)
+nsSupportsInterfacePointer::SetDataIID(const nsID* aIID)
 {
   if (mIID) {
-    nsMemory::Free(mIID);
+    free(mIID);
   }
+
   if (aIID) {
-    mIID = (nsID*)nsMemory::Clone(aIID, sizeof(nsID));
+    mIID = static_cast<nsID*>(nsMemory::Clone(aIID, sizeof(nsID)));
   } else {
     mIID = nullptr;
   }
@@ -830,16 +789,15 @@ nsSupportsInterfacePointerImpl::SetDataIID(const nsID* aIID)
 }
 
 NS_IMETHODIMP
-nsSupportsInterfacePointerImpl::ToString(char** aResult)
+nsSupportsInterfacePointer::ToString(char** aResult)
 {
   NS_ASSERTION(aResult, "Bad pointer");
 
   static const char str[] = "[interface pointer]";
-
   // jband sez: think about asking nsIInterfaceInfoManager whether
   // the interface has a known human-readable name
-  *aResult = (char*)nsMemory::Clone(str, sizeof(str));
-  return  *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  *aResult = static_cast<char*>(nsMemory::Clone(str, sizeof(str)));
+  return  NS_OK;
 }
 
 /***************************************************************************/
