@@ -7,6 +7,7 @@
 #include "nsAppFileLocationProvider.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsEnumeratorUtils.h"
 #include "nsIAtom.h"
 #include "nsIFile.h"
 #include "nsString.h"
@@ -57,7 +58,6 @@
 
 #define DEFAULTS_DIR_NAME           NS_LITERAL_CSTRING("defaults")
 #define DEFAULTS_PREF_DIR_NAME      NS_LITERAL_CSTRING("pref")
-#define DEFAULTS_PROFILE_DIR_NAME   NS_LITERAL_CSTRING("profile")
 #define RES_DIR_NAME                NS_LITERAL_CSTRING("res")
 #define CHROME_DIR_NAME             NS_LITERAL_CSTRING("chrome")
 #define PLUGINS_DIR_NAME            NS_LITERAL_CSTRING("plugins")
@@ -122,15 +122,6 @@ nsAppFileLocationProvider::GetFile(const char* aProp, bool* aPersistent,
         rv = localFile->AppendRelativeNativePath(DEFAULTS_PREF_DIR_NAME);
       }
     }
-  } else if (nsCRT::strcmp(aProp, NS_APP_PROFILE_DEFAULTS_50_DIR) == 0 ||
-             nsCRT::strcmp(aProp, NS_APP_PROFILE_DEFAULTS_NLOC_50_DIR) == 0) {
-    rv = CloneMozBinDirectory(getter_AddRefs(localFile));
-    if (NS_SUCCEEDED(rv)) {
-      rv = localFile->AppendRelativeNativePath(DEFAULTS_DIR_NAME);
-      if (NS_SUCCEEDED(rv)) {
-        rv = localFile->AppendRelativeNativePath(DEFAULTS_PROFILE_DIR_NAME);
-      }
-    }
   } else if (nsCRT::strcmp(aProp, NS_APP_USER_PROFILES_ROOT_DIR) == 0) {
     rv = GetDefaultUserProfileRoot(getter_AddRefs(localFile));
   } else if (nsCRT::strcmp(aProp, NS_APP_USER_PROFILES_LOCAL_ROOT_DIR) == 0) {
@@ -228,14 +219,15 @@ nsAppFileLocationProvider::GetFile(const char* aProp, bool* aPersistent,
   }
 
   if (localFile && NS_SUCCEEDED(rv)) {
-    return localFile->QueryInterface(NS_GET_IID(nsIFile), (void**)aResult);
+    localFile.forget(aResult);
+    return NS_OK;
   }
 
   return rv;
 }
 
 
-NS_METHOD
+nsresult
 nsAppFileLocationProvider::CloneMozBinDirectory(nsIFile** aLocalFile)
 {
   if (NS_WARN_IF(!aLocalFile)) {
@@ -283,7 +275,7 @@ nsAppFileLocationProvider::CloneMozBinDirectory(nsIFile** aLocalFile)
 // WIN    : <Application Data folder on user's machine>\Mozilla
 // Mac    : :Documents:Mozilla:
 //----------------------------------------------------------------------------------------
-NS_METHOD
+nsresult
 nsAppFileLocationProvider::GetProductDirectory(nsIFile** aLocalFile,
                                                bool aLocal)
 {
@@ -347,8 +339,7 @@ nsAppFileLocationProvider::GetProductDirectory(nsIFile** aLocalFile,
     return rv;
   }
 
-  *aLocalFile = localDir;
-  NS_ADDREF(*aLocalFile);
+  localDir.forget(aLocalFile);
 
   return rv;
 }
@@ -361,7 +352,7 @@ nsAppFileLocationProvider::GetProductDirectory(nsIFile** aLocalFile,
 // WIN    : <Application Data folder on user's machine>\Mozilla\Profiles
 // Mac    : :Documents:Mozilla:Profiles:
 //----------------------------------------------------------------------------------------
-NS_METHOD
+nsresult
 nsAppFileLocationProvider::GetDefaultUserProfileRoot(nsIFile** aLocalFile,
                                                      bool aLocal)
 {
@@ -394,8 +385,7 @@ nsAppFileLocationProvider::GetDefaultUserProfileRoot(nsIFile** aLocalFile,
   }
 #endif
 
-  *aLocalFile = localDir;
-  NS_ADDREF(*aLocalFile);
+  localDir.forget(aLocalFile);
 
   return rv;
 }
@@ -580,18 +570,21 @@ nsAppFileLocationProvider::GetFiles(const char* aProp,
     }
     *aResult = new nsPathsDirectoryEnumerator(this, keys);
 #endif
-    NS_IF_ADDREF(*aResult);
-    rv = *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(*aResult);
+    rv = NS_OK;
   }
   if (!nsCRT::strcmp(aProp, NS_APP_SEARCH_DIR_LIST)) {
-    static const char* keys[] = { nullptr, NS_APP_SEARCH_DIR, NS_APP_USER_SEARCH_DIR, nullptr };
+    static const char* keys[] = { nullptr, NS_APP_USER_SEARCH_DIR, nullptr };
     if (!keys[0] && !(keys[0] = PR_GetEnv("MOZ_SEARCH_ENGINE_PATH"))) {
       static const char nullstr = 0;
       keys[0] = &nullstr;
     }
     *aResult = new nsPathsDirectoryEnumerator(this, keys);
-    NS_IF_ADDREF(*aResult);
-    rv = *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(*aResult);
+    rv = NS_OK;
+  }
+  if (!strcmp(aProp, NS_APP_DISTRIBUTION_SEARCH_DIR_LIST)) {
+    return NS_NewEmptyEnumerator(aResult);
   }
   return rv;
 }
