@@ -24,10 +24,6 @@
 #include <shlobj.h>
 #include <stdlib.h>
 #include <stdio.h>
-
-#if defined(MOZ_CONTENT_SANDBOX)
-#include "nsIUUIDGenerator.h"
-#endif
 #elif defined(XP_UNIX)
 #include <unistd.h>
 #include <stdlib.h>
@@ -80,21 +76,16 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
   }
 
   if (dirService) {
-    nsCOMPtr <nsIFile> aLocalFile;
+    nsCOMPtr<nsIFile> localFile;
     dirService->Get(NS_XPCOM_INIT_CURRENT_PROCESS_DIR, NS_GET_IID(nsIFile),
-                    getter_AddRefs(aLocalFile));
-    if (aLocalFile) {
-      *aFile = aLocalFile;
-      NS_ADDREF(*aFile);
+                    getter_AddRefs(localFile));
+    if (localFile) {
+      localFile.forget(aFile);
       return NS_OK;
     }
   }
 
-  nsLocalFile* localFile = new nsLocalFile;
-  if (!localFile) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  NS_ADDREF(localFile);
+  RefPtr<nsLocalFile> localFile = new nsLocalFile;
 
 #ifdef XP_WIN
   wchar_t buf[MAX_PATH + 1];
@@ -108,7 +99,7 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
     }
 
     localFile->InitWithPath(nsDependentString(buf));
-    *aFile = localFile;
+    localFile.forget(aFile);
     return NS_OK;
   }
 
@@ -132,7 +123,7 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
 #endif
           rv = localFile->InitWithNativePath(nsDependentCString(buffer));
           if (NS_SUCCEEDED(rv)) {
-            *aFile = localFile;
+            localFile.forget(aFile);
           }
         }
         CFRelease(parentURL);
@@ -174,7 +165,7 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
   if (moz5 && *moz5) {
     if (realpath(moz5, buf)) {
       localFile->InitWithNativePath(nsDependentCString(buf));
-      *aFile = localFile;
+      localFile.forget(aFile);
       return NS_OK;
     }
   }
@@ -191,19 +182,17 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
   // Fall back to current directory.
   if (getcwd(buf, sizeof(buf))) {
     localFile->InitWithNativePath(nsDependentCString(buf));
-    *aFile = localFile;
+    localFile.forget(aFile);
     return NS_OK;
   }
 
 #endif
 
-  NS_RELEASE(localFile);
-
   NS_ERROR("unable to get current process directory");
   return NS_ERROR_FAILURE;
 } // GetCurrentProcessDirectory()
 
-nsDirectoryService* nsDirectoryService::gService = nullptr;
+StaticRefPtr<nsDirectoryService> nsDirectoryService::gService;
 
 nsDirectoryService::nsDirectoryService()
   : mHashtable(128)
@@ -254,15 +243,13 @@ nsDirectoryService::RealInit()
   NS_ASSERTION(!gService,
                "nsDirectoryService::RealInit Mustn't initialize twice!");
 
-  nsRefPtr<nsDirectoryService> self = new nsDirectoryService();
+  gService = new nsDirectoryService();
 
   NS_RegisterStaticAtoms(directory_atoms);
 
   // Let the list hold the only reference to the provider.
   nsAppFileLocationProvider* defaultProvider = new nsAppFileLocationProvider;
-  self->mProviders.AppendElement(defaultProvider);
-
-  self.swap(gService);
+  gService->mProviders.AppendElement(defaultProvider);
 }
 
 nsDirectoryService::~nsDirectoryService()
@@ -505,7 +492,7 @@ nsDirectoryService::UnregisterProvider(nsIDirectoryServiceProvider* aProv)
 
 #if defined(MOZ_CONTENT_SANDBOX) && defined(XP_WIN)
 static nsresult
-GetLowIntegrityTemp(nsIFile** aLowIntegrityTemp)
+GetLowIntegrityTempBase(nsIFile** aLowIntegrityTempBase)
 {
   nsCOMPtr<nsIFile> localFile;
   nsresult rv = GetSpecialSystemDirectory(Win_LocalAppdataLow,
@@ -514,37 +501,12 @@ GetLowIntegrityTemp(nsIFile** aLowIntegrityTemp)
     return rv;
   }
 
-  nsCOMPtr<nsIUUIDGenerator> uuidgen =
-    do_GetService("@mozilla.org/uuid-generator;1", &rv);
+  rv = localFile->Append(NS_LITERAL_STRING(MOZ_USER_DIR));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  nsID uuid;
-  rv = uuidgen->GenerateUUIDInPlace(&uuid);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  char uuidChars[NSID_LENGTH];
-  uuid.ToProvidedString(uuidChars);
-  rv = localFile->AppendNative(NS_LITERAL_CSTRING(MOZ_USER_DIR));
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  rv = localFile->AppendNative(NS_LITERAL_CSTRING("MozTemp-")
-                               + nsDependentCString(uuidChars));
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  rv = localFile->Create(nsIFile::DIRECTORY_TYPE, 0700);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  localFile.forget(aLowIntegrityTemp);
+  localFile.forget(aLowIntegrityTempBase);
   return rv;
 }
 #endif
@@ -564,7 +526,7 @@ nsDirectoryService::GetFile(const char* aProp, bool* aPersistent,
   *aResult = nullptr;
   *aPersistent = true;
 
-  nsCOMPtr<nsIAtom> inAtom = do_GetAtom(aProp);
+  nsCOMPtr<nsIAtom> inAtom = NS_Atomize(aProp);
 
   // check to see if it is one of our defaults
 
@@ -725,8 +687,8 @@ nsDirectoryService::GetFile(const char* aProp, bool* aPersistent,
 #if defined(MOZ_CONTENT_SANDBOX)
   } else if (inAtom == nsDirectoryService::sLocalAppdataLow) {
     rv = GetSpecialSystemDirectory(Win_LocalAppdataLow, getter_AddRefs(localFile));
-  } else if (inAtom == nsDirectoryService::sLowIntegrityTemp) {
-    rv = GetLowIntegrityTemp(getter_AddRefs(localFile));
+  } else if (inAtom == nsDirectoryService::sLowIntegrityTempBase) {
+    rv = GetLowIntegrityTempBase(getter_AddRefs(localFile));
 #endif
   } else if (inAtom == nsDirectoryService::sPrinthood) {
     rv = GetSpecialSystemDirectory(Win_Printhood, getter_AddRefs(localFile));
