@@ -74,14 +74,38 @@ nsTSubstring_CharT::MutatePrep(size_type aCapacity, char_type** aOldData,
       mFlags &= ~F_VOIDED;  // mutation clears voided flag
       return true;
     }
+  }
 
-    // Use doubling algorithm when forced to increase available capacity.
-    size_type temp = curCapacity;
-    while (temp < aCapacity) {
-      temp <<= 1;
+  if (curCapacity < aCapacity) {
+    // We increase our capacity so that the allocated buffer grows
+    // exponentially, which gives us amortized O(1) appending. Below the
+    // threshold, we use powers-of-two. Above the threshold, we grow by at
+    // least 1.125, rounding up to the nearest MiB.
+    const size_type slowGrowthThreshold = 8 * 1024 * 1024;
+
+    // nsStringBuffer allocates sizeof(nsStringBuffer) + passed size, and
+    // storageSize below wants extra 1 * sizeof(char_type).
+    const size_type neededExtraSpace =
+      sizeof(nsStringBuffer) / sizeof(char_type) + 1;
+
+    size_type temp;
+    if (aCapacity >= slowGrowthThreshold) {
+      size_type minNewCapacity = curCapacity + (curCapacity >> 3); // multiply by 1.125
+      temp = XPCOM_MAX(aCapacity, minNewCapacity) + neededExtraSpace;
+
+      // Round up to the next multiple of MiB, but ensure the expected
+      // capacity doesn't include the extra space required by nsStringBuffer
+      // and null-termination.
+      const size_t MiB = 1 << 20;
+      temp = (MiB * ((temp + MiB - 1) / MiB)) - neededExtraSpace;
+    } else {
+      // Round up to the next power of two.
+      temp =
+        mozilla::RoundUpPow2(aCapacity + neededExtraSpace) - neededExtraSpace;
     }
-    NS_ASSERTION(XPCOM_MIN(temp, kMaxCapacity) >= aCapacity,
-                 "should have hit the early return at the top");
+
+    MOZ_ASSERT(XPCOM_MIN(temp, kMaxCapacity) >= aCapacity,
+               "should have hit the early return at the top");
     aCapacity = XPCOM_MIN(temp, kMaxCapacity);
   }
 
@@ -308,11 +332,17 @@ nsTSubstring_CharT::Assign(char_type aChar, const fallible_t&)
 void
 nsTSubstring_CharT::Assign(const char_type* aData)
 {
-  if (!Assign(aData, size_type(-1), mozilla::fallible)) {
+  if (!Assign(aData, mozilla::fallible)) {
     AllocFailed(char_traits::length(aData));
   }
 }
 
+bool
+nsTSubstring_CharT::Assign(const char_type* aData, const fallible_t&)
+{
+  return Assign(aData, size_type(-1), mozilla::fallible);
+}
+
 void
 nsTSubstring_CharT::Assign(const char_type* aData, size_type aLength)
 {
