@@ -1,4 +1,5 @@
-/* vim:set ts=2 sw=2 et cindent: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,7 +10,7 @@
 #include "nsStringBuffer.h"
 #include "nsReadableUtils.h"
 #include "nsCRTGlue.h"
-#include "nsRefPtr.h"
+#include "mozilla/RefPtr.h"
 #include "nsTArray.h"
 #include "gtest/gtest.h"
 
@@ -43,6 +44,7 @@ TEST(Strings, test1)
   nsAutoString buf(aStr);
 
   int32_t n = buf.FindChar(',');
+  EXPECT_EQ(n, kNotFound);
 
   n = buf.Length();
 
@@ -131,7 +133,7 @@ TEST(Strings, findinreadable)
   // Should match the first "!/" but not the last
   EXPECT_NE(delim_end, end);
   EXPECT_STREQ(r, "!/");
-  nsMemory::Free(r);
+  free(r);
 
   delim_begin = begin;
   delim_end = end;
@@ -143,7 +145,7 @@ TEST(Strings, findinreadable)
   // Should not match the first jar:, but the second one
   EXPECT_EQ(delim_begin, begin);
   EXPECT_STREQ(r, "jar:");
-  nsMemory::Free(r);
+  free(r);
 
   // Search for jar: in a Substring
   delim_begin = begin; delim_begin++;
@@ -154,7 +156,7 @@ TEST(Strings, findinreadable)
   // Should not match the first jar:, but the second one
   EXPECT_NE(delim_begin, begin);
   EXPECT_STREQ(r, "jar:");
-  nsMemory::Free(r);
+  free(r);
 
   // Should not find a match
   EXPECT_FALSE(FindInReadable(NS_LITERAL_CSTRING("gecko"), delim_begin, delim_end));
@@ -196,7 +198,7 @@ TEST(Strings, rfindinreadable)
   // Should match the last "!/"
   EXPECT_EQ(delim_end, end);
   EXPECT_STREQ(r, "!/");
-  nsMemory::Free(r);
+  free(r);
 
   delim_begin = begin;
   delim_end = end;
@@ -208,7 +210,7 @@ TEST(Strings, rfindinreadable)
   // Should not match the first jar:, but the second one
   EXPECT_NE(delim_begin, begin);
   EXPECT_STREQ(r, "jar:");
-  nsMemory::Free(r);
+  free(r);
 
   // Search for jar: in a Substring
   delim_begin = begin;
@@ -219,7 +221,7 @@ TEST(Strings, rfindinreadable)
   // Should not match the first jar:, but the second one
   EXPECT_EQ(delim_begin, begin);
   EXPECT_STREQ(r, "jar:");
-  nsMemory::Free(r);
+  free(r);
 
   // Should not find a match
   delim_begin = begin;
@@ -472,7 +474,7 @@ TEST(Strings, xpidl_string)
   a.EndReading(end);
   char *r = ToNewCString(Substring(begin, end));
   EXPECT_STREQ(r, "");
-  nsMemory::Free(r);
+  free(r);
 
   a.Adopt(0);
   EXPECT_TRUE(a.IsVoid());
@@ -667,7 +669,7 @@ TEST(Strings, stringbuffer)
 {
   const char kData[] = "hello world";
 
-  nsRefPtr<nsStringBuffer> buf;
+  RefPtr<nsStringBuffer> buf;
 
   buf = nsStringBuffer::Alloc(sizeof(kData));
   EXPECT_TRUE(!!buf);
@@ -822,23 +824,23 @@ static void test_strip_chars_helper(const char16_t* str, const char16_t* strip,
 
 TEST(String, strip_chars)
 {
-  test_strip_chars_helper(MOZ_UTF16("foo \r \nbar"),
-                          MOZ_UTF16(" \n\r"),
+  test_strip_chars_helper(u"foo \r \nbar",
+                          u" \n\r",
                           NS_LITERAL_STRING("foobar"));
-  test_strip_chars_helper(MOZ_UTF16("\r\nfoo\r\n"),
-                          MOZ_UTF16(" \n\r"),
+  test_strip_chars_helper(u"\r\nfoo\r\n",
+                          u" \n\r",
                           NS_LITERAL_STRING("foo"));
-  test_strip_chars_helper(MOZ_UTF16("foo"),
-                          MOZ_UTF16(" \n\r"),
+  test_strip_chars_helper(u"foo",
+                          u" \n\r",
                           NS_LITERAL_STRING("foo"));
-  test_strip_chars_helper(MOZ_UTF16("foo"),
-                          MOZ_UTF16("fo"),
+  test_strip_chars_helper(u"foo",
+                          u"fo",
                           NS_LITERAL_STRING(""));
-  test_strip_chars_helper(MOZ_UTF16("foo"),
-                          MOZ_UTF16("foo"),
+  test_strip_chars_helper(u"foo",
+                          u"foo",
                           NS_LITERAL_STRING(""));
-  test_strip_chars_helper(MOZ_UTF16(" foo"),
-                          MOZ_UTF16(" "),
+  test_strip_chars_helper(u" foo",
+                          u" ",
                           NS_LITERAL_STRING(" foo"), 1);
 }
 
@@ -977,4 +979,4 @@ TEST(Strings, todouble)
   test_todouble_helper(NS_LITERAL_STRING("foo"), 0, false);
 }
 
-}
+} // namespace TestStrings
